/* --------------------------------------------------------------  */
/* (C)Copyright 2001,2007,                                         */
/* International Business Machines Corporation,                    */
/* Sony Computer Entertainment, Incorporated,                      */
/* Toshiba Corporation,                                            */
/*                                                                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#include <stdio.h>
#include <vec_types.h>

#ifndef __SPU__
#include <altivec.h>
#include <si2vmx.h>
#endif

#include "libtest.h"

typedef union {
  vec_uchar16	vuc;
  vec_char16	vc;
  vec_ushort8	vus;
  vec_short8	vs;
  vec_uint4	vui;
  vec_int4	vi;
  vec_ullong2	vull;
  vec_llong2	vll;
  vec_float4	vf;
  vec_double2	vd;
  qword		qw;
  unsigned char	uc[16];
  signed char	c[16];
  unsigned short us[8];
  signed short	s[8];
  unsigned int	ui[4];
  signed int	i[4];
  long long	ll[2];
  unsigned long long ull[2];
  float		f[4];
  double	d[2];
  void *	p[4];
} v128;
  
static char *passed = "passed\n";
static char *failed = "FAILED\n";
static v128 a, b, c, d;
static unsigned char buffer[256] __attribute__ ((aligned (16)));

static int samples = 100;

static unsigned int bias_values[4] = { 0, -1, -2, 1 };

unsigned int seed = 0;

unsigned int rand_local()
{
  seed = seed * 69607 + 54329;
  return (seed);
}

vec_uint4 rand_vui()
{
  static v128 val;

  val.ui[0] = rand_local();
  val.ui[1] = rand_local();
  val.ui[2] = rand_local();
  val.ui[3] = rand_local();

  return (val.vui);
}

unsigned long long rand_ullong()
{
  return ((unsigned long long)(rand_local) | ((unsigned long long)(rand_local) << 32));
}



float rand_float(float min, float max)
{
  union {
    float f;
    unsigned int ui;
  } x;

  x.ui = (rand_local() & 0x7FFFFF) | 0x3F800000;
  x.f -= 1.0f;
  x.f *= (max - min);
  x.f += min;
  return (x.f);
}

vec_float4 rand_vf(float min, float max)
{
  int i;
  static v128 val;

  for (i=0; i<4; i++) val.f[i] = rand_float(min, max);
  return (val.vf);
}


double rand_double(double min, double max)
{
  union {
    unsigned int ui[2];
    double d;
  } x;

  x.ui[0] = (rand_local() & 0x000FFFFF) | 0x3FF00000;
  x.ui[1] = rand_local();
  x.d -= 1.0;
  x.d *= max - min;
  x.d += min;
  return (x.d);
}

vec_double2 rand_vd(double min, double max)
{
  int i;
  static v128 val;

  for (i=0; i<2; i++) val.d[i] = rand_double(min, max);
  return (val.vd);
}


void set_buffer(void)
{
  int i;
  for (i=0; i<256; i++) buffer[i] = (unsigned char)rand_local();
}

void clear_buffer(void)
{
  int i;
  for (i=0; i<256; i++) buffer[i] = 0;
}



int test_si_a()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "a...............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_a(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = a.ui[j] + b.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_absdb()
{
  int i, j;
  unsigned char exp;

  DEBUG(1, "absdb...........");

  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_absdb(a.qw, b.qw);
    for (j=0; j<16; j++) {
      exp = (a.uc[j] > b.uc[j]) ? (a.uc[j] - b.uc[j]) : (b.uc[j] - a.uc[j]);
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.uc[j], b.uc[j], d.uc[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_addx()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "addx............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();
    d.qw = si_addx(a.qw, b.qw, c.qw);
    for (j=0; j<4; j++) {
      exp = a.ui[j] + b.ui[j] + (c.ui[j] & 1);
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=0x%x ", j, a.ui[j]);
	DEBUG(2, "b=0x%x c=0x%x d=0x%x exp=0x%x\n", b.ui[j], c.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_ah()
{
  int i, j;
  unsigned short exp;

  DEBUG(1, "ah..............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_ah(a.qw, b.qw);
    for (j=0; j<8; j++) {
      exp = a.us[j] + b.us[j];
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.us[j], b.us[j], d.us[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_ahi()
{
  int j;
  unsigned short exp;

  DEBUG(1, "ahi.............");
  a.vui = rand_vui();
  d.qw = si_ahi(a.qw, 511);
  for (j=0; j<8; j++) {
    exp = a.us[j] + 511;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=511 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_ai()
{
  int j;
  unsigned int exp;

  DEBUG(1, "ai..............");
  a.vui = rand_vui();
  d.qw = si_ai(a.qw, -512);
  for (j=0; j<4; j++) {
    exp = a.ui[j] - 512;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-512 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_and()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "and.............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_and(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = a.ui[j] & b.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_andbi()
{
  int j;
  unsigned char exp;

  DEBUG(1, "andbi...........");
  a.vui = rand_vui();
  d.qw = si_andbi(a.qw, 267);
  for (j=0; j<16; j++) {
    exp = a.uc[j] & 267;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=267 d=0x%x exp=0x%x\n", a.uc[j], d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_andc()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "andc............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_andc(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = a.ui[j] & ~b.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_andhi()
{
  int j;
  unsigned short exp;

  DEBUG(1, "andhi...........");
  a.vui = rand_vui();
  d.qw = si_andhi(a.qw, 511);
  for (j=0; j<8; j++) {
    exp = a.us[j] & 511;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=511 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_andi()
{
  int j;
  unsigned int exp;

  DEBUG(1, "andi............");
  a.vui = rand_vui();
  d.qw = si_andi(a.qw, -512);
  for (j=0; j<4; j++) {
    exp = a.ui[j] & -512;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-511 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_avgb()
{
  int i, j;
  unsigned char exp;

  DEBUG(1, "avgb............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_avgb(a.qw, b.qw);
    for (j=0; j<16; j++) {
      exp = (unsigned char)(((unsigned int)a.uc[j] + (unsigned int)(b.uc[j]) + 1) >> 1);
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.uc[j], b.uc[j], d.uc[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_bg()
{
  int i, j;
  unsigned int idx, bias;
  unsigned int exp;

  DEBUG(1, "bg..............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();

    idx = rand_local() & 3;
    bias = bias_values[rand_local() & 3];
    b.ui[idx] = a.ui[idx] + bias;

    d.qw = si_bg(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = (a.ui[j] > b.ui[j]) ? 0 : 1;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_bgx()
{
  int i, j;
  unsigned int idx, bias;
  unsigned int exp;

  DEBUG(1, "bgx.............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();

    idx = rand_local() & 3;
    bias = bias_values[rand_local() & 3];
    b.ui[idx] = a.ui[idx] + bias;

    d.qw = si_bgx(a.qw, b.qw, c.qw);

    for (j=0; j<4; j++) {
      if (c.ui[j] & 1) {
	exp = (a.ui[j] > b.ui[j]) ? 0 : 1;
      } else {
	exp = (a.ui[j] >= b.ui[j]) ? 0 : 1;
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=0x%x", j, a.ui[j]);
	DEBUG(2, " b=0x%x c=0x%x d=0x%x exp=0x%x\n", b.ui[j], c.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


#ifdef __SPU__
extern volatile int bisled_count;
extern void bisled_handler(void);


int test_si_bisled()
{
  int i;
  unsigned int count, status;
  qword handler;

  bisled_count = 0;

  DEBUG(1, "bisled..........");
  handler = si_from_ptr((void *)(bisled_handler));

  si_bisled(handler);
  if (bisled_count != 0) {
    DEBUG(1, failed);
    DEBUG(2, "\t\thandler called when not expected\n");
    return (1);
  }
  si_wrch(SPU_WrEventMask, si_from_int(0x20));
  si_wrch(SPU_WrDec, si_from_int(0));
  for (i=0; i<1000; i++) {
    si_bisled(handler);
    if (bisled_count) break;
  }
  si_wrch(SPU_WrEventMask, si_from_int(0));

  /* Process (phantom) events 
   */
  count = si_to_uint(si_rchcnt(SPU_RdEventStat));
  if (count) {
    status = si_to_uint(si_rdch(SPU_RdEventStat));
    if (status & 0x20) {
      si_wrch(SPU_WrEventAck, (si_from_uint(status)));
    }
  }

  if (bisled_count == 0) {
    /* NOTE: this test will fail at this point if it is run
     * on the stand-alone spu simulator (spusim). Successful
     * execution of this test depends upon using the full system
     * simulator.
     */
    DEBUG(1, failed);
    DEBUG(2, "\t\thandler never called when expected\n");
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_bisledd()
{
  int i;
  unsigned int count, status;
  qword handler;

  bisled_count = 0;

  DEBUG(1, "bisledd.........");
  handler = si_from_ptr((void *)(bisled_handler));

  si_bisledd(handler);
  if (bisled_count != 0) {
    DEBUG(1, failed);
    DEBUG(2, "\t\thandler called when not expected\n");
    return (1);
  }
  si_wrch(SPU_WrEventMask, si_from_int(0x20));
  si_wrch(SPU_WrDec, si_from_int(0));
  for (i=0; i<1000; i++) {
    si_bisledd(handler);
    if (bisled_count) break;
  }
  si_wrch(SPU_WrEventMask, si_from_int(0));

  /* Process (phantom) events 
   */
  count = si_to_uint(si_rchcnt(SPU_RdEventStat));
  if (count) {
    status = si_to_uint(si_rdch(SPU_RdEventStat));
    if (status & 0x20) {
      si_wrch(SPU_WrEventAck, (si_from_uint(status)));
    }
  }

  if (bisled_count == 0) {
    /* NOTE: this test will fail at this point if it is run
     * on the stand-alone spu simulator (spusim). Successful
     * execution of this test depends upon using the full system
     * simulator.
     */
    DEBUG(1, failed);
    DEBUG(2, "\t\thandler never called when expected\n");
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_bislede()
{
  qword handler;

  bisled_count = 0;

  DEBUG(1, "bislede.........");
  handler = si_from_ptr((void *)(bisled_handler));

  si_bislede(handler);
  if (bisled_count != 0) {
    DEBUG(1, failed);
    DEBUG(2, "\t\thandler called when not expected\n");
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}
#endif /* __SPU__ */


int test_si_cbd()
{
  int j;
  unsigned char exp;

  DEBUG(1, "cbd.............");
  a.vui = rand_vui();
  d.qw = si_cbd(a.qw, 63);
  for (j=0; j<16; j++) {
    exp = 16 + j;
    if (j == (int)((a.ui[0] + 63) & 0xF)) exp = 3;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x imm=63 d=0x%x exp=0x%x\n", a.ui[0], d.uc[j], exp);
      return (1);
    }
  }
  a.vui = rand_vui();
  d.qw = si_cbd(a.qw, -64);
  for (j=0; j<16; j++) {
    exp = 16 + j;
    if (j == (int)((a.ui[0] - 64) & 0xF)) exp = 3;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x imm=-64 d=0x%x exp=0x%x\n", a.ui[0], d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_cbx()
{
  int j;
  unsigned char exp;

  DEBUG(1, "cbx.............");
  a.vui = rand_vui();
  b.vui = rand_vui();
  d.qw = si_cbx(a.qw, b.qw);
  for (j=0; j<16; j++) {
    exp = 16 + j;
    if (j == (int)((a.ui[0] + b.ui[0]) & 0xF)) exp = 3;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[0], b.ui[0], d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_cdd()
{
  int j;
  unsigned char exp;

  DEBUG(1, "cdd.............");
  a.vui = rand_vui();
  d.qw = si_cdd(a.qw, 63);
  for (j=0; j<16; j++) {
    exp = 16 + j;
    if ((j & 0x8) == (int)((a.ui[0] + 63) & 0x8)) exp = j & 7;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x imm=63 d=0x%x exp=0x%x\n", a.ui[0], d.uc[j], exp);
      return (1);
    }
  }
  a.vui = rand_vui();
  d.qw = si_cdd(a.qw, -64);
  for (j=0; j<16; j++) {
    exp = 16 + j;
    if ((j & 0x8) == (int)((a.ui[0] - 64) & 0x8)) exp = j & 7;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x imm=-64 d=0x%x exp=0x%x\n", a.ui[0], d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_cdx()
{
  int j;
  unsigned char exp;

  DEBUG(1, "cdx.............");
  a.vui = rand_vui();
  b.vui = rand_vui();
  d.qw = si_cdx(a.qw, b.qw);
  for (j=0; j<16; j++) {
    exp = 16 + j;
    if ((j & 0x8) == (int)((a.ui[0] + b.ui[0]) & 0x8)) exp = j & 7;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[0], b.ui[0], d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_ceqb()
{
  int i, j;
  unsigned char exp;

  DEBUG(1, "ceqb............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    for (j=0; j<16; j++) if (b.uc[j] & 0x10) b.uc[j] = a.uc[j];
    d.qw = si_ceqb(a.qw, b.qw);
    for (j=0; j<16; j++) {
      exp = (a.uc[j] == b.uc[j]) ? 0xFF : 0;
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.uc[j], b.uc[j], d.uc[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_ceq()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "ceq.............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    for (j=0; j<4; j++) if (b.ui[j] & 0x10) b.ui[j] = a.ui[j];
    d.qw = si_ceq(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = (a.ui[j] == b.ui[j]) ? 0xFFFFFFFF : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}





int test_si_ceqbi()
{
  int j;
  unsigned char exp;

  DEBUG(1, "ceqbi...........");

  a.vui = rand_vui();
  a.uc[rand_local() & 15] = (unsigned char)291;
  d.qw = si_ceqbi(a.qw, 291);
  for (j=0; j<16; j++) {
    exp = (a.uc[j] == 35) ? 0xFF : 0;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=35 d=0x%x exp=0x%x\n", a.uc[j], d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_ceqh()
{
  int i, j;
  unsigned short exp;

  DEBUG(1, "ceqh............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    for (j=0; j<8; j++) if (b.us[j] & 0x10) b.us[j] = a.us[j];
    d.qw = si_ceqh(a.qw, b.qw);
    for (j=0; j<8; j++) {
      exp = (a.us[j] == b.us[j]) ? 0xFFFF : 0;
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.us[j], b.us[j], d.us[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_ceqhi()
{
  int j;
  unsigned short exp;

  DEBUG(1, "ceqhi...........");
  a.vui = rand_vui();
  a.us[rand_local() & 7] = 511;
  d.qw = si_ceqhi(a.qw, 511);
  for (j=0; j<8; j++) {
    exp = (a.us[j] == 511) ? 0xFFFF : 0;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=511 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_ceqi()
{
  int j;
  unsigned int exp;

  DEBUG(1, "ceqi............");
  a.vui = rand_vui();
  a.ui[rand_local() & 3] = -512;
  d.qw = si_ceqi(a.qw, -512);
  for (j=0; j<4; j++) {
    exp = (a.i[j] == -512) ? 0xFFFFFFFF : 0;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-512 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_cflts()
{
  int i, j;
  signed int exp;

  DEBUG(1, "cflts...........");
  
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-1024.0, 1024.0);
    d.qw = si_cflts(a.qw, 22);
    for (j=0; j<4; j++) {
      if (a.f[j] >= 512.0f) exp = 0x7FFFFFFF;
      else if (a.f[j] <= -512.0f) exp = 0x80000000;
      else exp = (int)(a.f[j] * 4194304.0f);

      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f d=%d exp=%d\n", a.f[j], d.i[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_cfltu()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "cfltu...........");
  
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-1024.0, 1024.0);
    d.qw = si_cfltu(a.qw, 22);
    for (j=0; j<4; j++) {
      if (a.f[j] >= 1024.0f) exp = 0xFFFFFFFF;
      else if (a.f[j] <= 0.0f) exp = 0x00000000;
      else exp = (unsigned int)(a.f[j] * 4194304.0f);
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f d=0x%x exp=0x%x\n", a.f[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_cg()
{
  int i, j;
  unsigned int idx, bias;
  unsigned int sum, exp;

  DEBUG(1, "cg..............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();

    idx = rand_local() & 3;
    bias = bias_values[rand_local() & 3];
    b.ui[idx] = a.ui[idx] + bias;

    d.qw = si_cg(a.qw, b.qw);
    for (j=0; j<4; j++) {
      sum = a.ui[j] + b.ui[j];
      exp = ((sum < a.ui[j]) && (sum < b.ui[j])) ? 1 : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_cgt()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "cgt.............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_cgt(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = (a.i[j] > b.i[j]) ? 0xFFFFFFFF : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%d b=%d d=0x%x exp=0x%x\n", a.i[j], b.i[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_cgtb()
{
  int i, j;
  unsigned char exp;

  DEBUG(1, "cgtb............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_cgtb(a.qw, b.qw);
    for (j=0; j<16; j++) {
      exp = (a.c[j] > b.c[j]) ? 0xFF : 0;
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%d b=%d d=0x%x exp=0x%x\n", a.c[j], b.c[j], d.uc[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_cgtbi()
{
  int j;
  unsigned char exp;

  DEBUG(1, "cgtbi...........");
  a.vui = rand_vui();
  a.uc[rand_local() & 15] = -27;
  d.qw = si_cgtbi(a.qw, -27);
  for (j=0; j<16; j++) {
    exp = (a.c[j] > -27) ? 0xFF : 0;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-27 d=0x%x exp=0x%x\n", a.uc[j], d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_cgth()
{
  int i, j;
  unsigned short exp;
      
  DEBUG(1, "cgth............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_cgth(a.qw, b.qw);
    for (j=0; j<8; j++) {
      exp = (a.s[j] > b.s[j]) ? 0xFFFF : 0;
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%d b=%d d=0x%x exp=0x%x\n", a.s[j], b.s[j], d.us[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_cgthi()
{
  int j;
  unsigned short exp;

  DEBUG(1, "cgthi...........");

  d.qw = si_cgthi(a.qw, 511);
  for (j=0; j<8; j++) {
    exp = (a.s[j] > 511) ? 0xFFFF : 0;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=%d b=511 d=0x%x exp=0x%x\n", a.s[j], d.us[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_cgti()
{
  int j;
  unsigned int exp;

  DEBUG(1, "cgti............");
  d.qw = si_cgti(a.qw, -512);
  for (j=0; j<4; j++) {
    exp = (a.i[j] > -512) ? 0xFFFFFFFF : 0;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=%d b=-512 d=0x%x exp=0x%x\n", a.i[j], d.ui[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_cgx()
{
  int i, j;
  unsigned int idx, bias;
  unsigned int sum, exp;

  DEBUG(1, "cgx.............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();

    idx = rand_local() & 3;
    bias = bias_values[rand_local() & 3];
    b.ui[idx] = a.ui[idx] + bias;

    d.qw = si_cgx(a.qw, b.qw, c.qw);
    for (j=0; j<4; j++) {
      sum = a.ui[j] + b.ui[j] + (c.ui[j] & 1);
      if (c.ui[j] & 1) {
	exp = ((sum <= a.ui[j]) && (sum <= b.ui[j])) ? 1 : 0;
      } else {
	exp = ((sum < a.ui[j]) && (sum < b.ui[j])) ? 1 : 0;
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_chd()
{
  int j;
  unsigned char exp;

  DEBUG(1, "chd.............");
  a.vui = rand_vui();
  d.qw = si_chd(a.qw, 63);
  for (j=0; j<16; j++) {
    exp = 16 + j;
    if ((j & 0xE) == (int)((a.ui[0] + 63) & 0xE)) exp = 2 + (j & 1);
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x imm=63 d=0x%x exp=0x%x\n", a.ui[0], d.uc[j], exp);
      return (1);
    }
  }
  a.vui = rand_vui();
  d.qw = si_chd(a.qw, -64);
  for (j=0; j<16; j++) {
    exp = 16 + j;
    if ((j & 0xE) == (int)((a.ui[0] - 64) & 0xE)) exp = 2 + (j & 1);
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x imm=-64 d=0x%x exp=0x%x\n", a.ui[0], d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_chx()
{
  int j;
  unsigned char exp;

  DEBUG(1, "chx.............");
  a.vui = rand_vui();
  b.vui = rand_vui();
  d.qw = si_chx(a.qw, b.qw);
  for (j=0; j<16; j++) {
    exp = 16 + j;
    if ((j & 0xE) == (int)((a.ui[0] + b.ui[0]) & 0xE)) exp = 2 + (j & 1);
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[0], b.ui[0], d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_clgt()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "clgt............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_clgt(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = (a.ui[j] > b.ui[j]) ? 0xFFFFFFFF : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_clgtb()
{
  int i, j;
  unsigned char exp;

  DEBUG(1, "clgtb...........");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_clgtb(a.qw, b.qw);
    for (j=0; j<16; j++) {
      exp = (a.uc[j] > b.uc[j]) ? 0xFF : 0;
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.uc[j], b.uc[j], d.uc[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_clgtbi()
{
  int j;
  unsigned char exp;

  DEBUG(1, "clgtbi..........");
  a.vui = rand_vui();
  a.uc[rand_local() & 15] = 97;
  d.qw = si_clgtbi(a.qw, 97);
  for (j=0; j<16; j++) {
    exp = (a.uc[j] > 97) ? 0xFF : 0;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=97 d=0x%x exp=0x%x\n", a.uc[j], d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_clgth()
{
  int i, j;
  unsigned short exp;

  DEBUG(1, "clgth...........");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_clgth(a.qw, b.qw);
    for (j=0; j<8; j++) {
      exp = (a.us[j] > b.us[j]) ? 0xFFFF : 0;
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.us[j], b.us[j], d.us[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_clgthi()
{
  int j;
  unsigned short exp;

  DEBUG(1, "clgthi..........");
  a.vui = rand_vui();
  a.us[rand_local() & 7] = 511;
  d.qw = si_clgthi(a.qw, 511);
  for (j=0; j<8; j++) {
    exp = (a.us[j] > 511) ? 0xFFFF : 0;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=511 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_clgti()
{
  int j;
  unsigned int exp;

  DEBUG(1, "clgti...........");
  a.vui = rand_vui();
  a.ui[rand_local() & 3] = -512;
  d.qw = si_clgti(a.qw, -512);
  for (j=0; j<4; j++) {
    exp = (a.ui[j] > (unsigned int)-512) ? 0xFFFFFFFF : 0;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-512 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_clz()
{
  int i, j;
  unsigned int exp;
  unsigned int bit;

  DEBUG(1, "clz.............");
  for (i=0; i<samples; i++) {
    a.ui[0] = (unsigned int)(rand_local) >> ((unsigned int)(rand_local() & 31));
    a.ui[1] = (unsigned int)(rand_local) >> ((unsigned int)(rand_local() & 31));
    a.ui[2] = (unsigned int)(rand_local) >> ((unsigned int)(rand_local() & 31));
    a.ui[3] = (unsigned int)(rand_local) >> ((unsigned int)(rand_local() & 31));
    d.qw = si_clz(a.qw);
    for (j=0; j<4; j++) {
      for (exp = 0, bit = 0x80000000; bit != 0; bit >>= 1) {
	if ((a.ui[j] & bit) == 0) exp++;
	else break;
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x d=%d exp=%d\n", a.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_cntb()
{
  int i, j;
  unsigned char exp;
  unsigned char element;

  DEBUG(1, "cntb............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.qw = si_cntb(a.qw);
    for (j=0; j<16; j++) {
      for (exp = 0, element = a.uc[j]; element != 0; element >>= 1) {
	if (element & 1) exp++;
      }
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x d=%d exp=%d\n", a.uc[j], d.uc[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_csflt()
{
  int i, j;
  float exp;

  DEBUG(1, "csflt...........");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.qw = si_csflt(a.qw, 2);
    for (j=0; j<4; j++) {
      exp = (float)a.i[j];
      exp *= 0.25f;
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%d d=%f exp=%f\n", a.i[j], d.f[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_cuflt()
{
  int i, j;
  float exp;

  DEBUG(1, "cuflt...........");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.qw = si_cuflt(a.qw, 5);
    for (j=0; j<4; j++) {
      exp = (float)a.ui[j];
      exp *= 0.03125f;
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x d=%f exp=%f\n", a.ui[j], d.f[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_cwd()
{
  int j;
  unsigned int exp;

  DEBUG(1, "cwd.............");
  a.vui = rand_vui();
  d.qw = si_cwd(a.qw, 63);
  for (j=0; j<16; j++) {
    exp = 16 + j;
    if ((j & 0xC) == (int)((a.ui[0] + 63) & 0xC)) exp = j & 3;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x imm=63 d=0x%x exp=0x%x\n", a.ui[0], d.uc[j], exp);
      return (1);
    }
  }
  a.vui = rand_vui();
  d.qw = si_cwd(a.qw, -64);
  for (j=0; j<16; j++) {
    exp = 16 + j;
    if ((j & 0xC) == (int)((a.ui[0] - 64) & 0xC)) exp = j & 3;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x imm=-64 d=0x%x exp=0x%x\n", a.ui[0], d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_cwx()
{
  int j;
  unsigned int exp;

  DEBUG(1, "cwx.............");
  a.vui = rand_vui();
  b.vui = rand_vui();
  d.qw = si_cwx(a.qw, b.qw);
  for (j=0; j<16; j++) {
    exp = 16 + j;
    if ((j & 0xC) == (int)((a.ui[0] + b.ui[0]) & 0xC)) exp = j & 3;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[0], b.ui[0], d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_dfa()
{
  int i, j;
  double exp;

  DEBUG(1, "dfa.............");
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0, 4.0);
    b.vd = rand_vd(-4.0, 4.0);
    d.qw = si_dfa(a.qw, b.qw);
    for (j=0; j<2; j++) {
      exp = a.d[j] + b.d[j];
      if (exp != d.d[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f b=%f d=%f exp=%f\n", a.d[j], b.d[j], d.d[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


#if defined(__SPU_EDP__) || defined(__PPU__)

int test_si_dfceq()
{
  int i, j;
  unsigned long long exp;

  DEBUG(1, "dfceq...........");
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0, 4.0);
    b.vd = rand_vd(-4.0, 4.0);
    for (j=0; j<2; j++) if (b.ui[j] & 0x10) b.ull[j] = a.ull[j];
    d.qw = si_dfceq(a.qw, b.qw);
    for (j=0; j<2; j++) {
      exp = (a.d[j] == b.d[j]) ? 0xFFFFFFFFFFFFFFFFULL : 0ULL;
      if (exp != d.ull[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f b=%f d=0x%llx exp=0x%llx\n", a.d[j], b.d[j], d.ull[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_dfcgt()
{
  int i, j;
  unsigned long long exp;

  DEBUG(1, "dfcgt...........");
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0, 4.0);
    b.vd = rand_vd(-4.0, 4.0);
    d.qw = si_dfcgt(a.qw, b.qw);
    for (j=0; j<2; j++) {
      exp = (a.d[j] > b.d[j]) ? 0xFFFFFFFFFFFFFFFFULL : 0ULL;
      if (exp != d.ull[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f b=%f d=0x%llx exp=0x%llx\n", a.d[j], b.d[j], d.ull[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_dfcmeq()
{
  int i, j;
  unsigned long long exp;

  DEBUG(1, "dfcmeq..........");
  
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0, 4.0);
    for (j=0; j<2; j++) {
      if (rand_local() & 0x10) {
	b.ull[j] = a.ull[j];
	if (rand_local() & 0x10) {
	  b.ull[j] ^= 0x8000000000000000ULL;
	}
      }
    }
    d.qw = si_dfcmeq(a.qw, b.qw);
    for (j=0; j<2; j++) {
      exp = ((a.d[j] == b.d[j]) || (-a.d[j] == b.d[j]) || (a.d[j] == -b.d[j])) ? 0xFFFFFFFFFFFFFFFFULL : 0ULL;
      if (exp != d.ull[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f b=%f d=0x%llx exp=0x%llx\n", a.d[j], b.d[j], d.ull[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}


int test_si_dfcmgt()
{
  int i, j;
  unsigned long long exp;
  double abs_a, abs_b;

  DEBUG(1, "dfcmgt..........");
  
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0, 4.0);
    b.vd = rand_vd(-4.0, 4.0);
    d.qw = si_dfcmgt(a.qw, b.qw);
    for (j=0; j<2; j++) {
      abs_a = (a.d[j] < 0.0) ? -a.d[j] : a.d[j];
      abs_b = (b.d[j] < 0.0) ? -b.d[j] : b.d[j];
      exp = (abs_a > abs_b) ? 0xFFFFFFFFFFFFFFFFULL : 0ULL;
      if (exp != d.ull[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f b=%f d=0x%llx exp=0x%llx\n", a.d[j], b.d[j], d.ull[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


#define	NEG_DENORM	1
#define	POS_DENORM	2
#define	NEG_ZERO	4
#define	POS_ZERO	8
#define NEG_INF		16
#define POS_INF		32
#define NAN		64

struct _dftsv_samples {
  unsigned long long v;
  unsigned int sv;
} dftsv_samples[] = {
  {0x0000000000000000ULL, POS_ZERO},
  {0x8000000000000000ULL, NEG_ZERO},
  {0x0000000000000001ULL, POS_DENORM},
  {0x0000000080000000ULL, POS_DENORM},
  {0x0000000100000000ULL, POS_DENORM},
  {0x0008000000000000ULL, POS_DENORM},
  {0x000FFFFFFFFFFFFFULL, POS_DENORM},
  {0x00000000FFF00000ULL, POS_DENORM},
  {0x8000000000000001ULL, NEG_DENORM},
  {0x8000000080000000ULL, NEG_DENORM},
  {0x8000000100000000ULL, NEG_DENORM},
  {0x8008000000000000ULL, NEG_DENORM},
  {0x800FFFFFFFFFFFFFULL, NEG_DENORM},
  {0x80000000FFF00000ULL, NEG_DENORM},
  {0x0010000000000000ULL, 0},
  {0x0010000000000001ULL, 0},
  {0x3FF0000000000000ULL, 0},
  {0x3FF00000FFF00000ULL, 0},
  {0xBFF0000000000000ULL, 0},
  {0xBFF00000FFF00000ULL, 0},
  {0x7FE0000000000000ULL, 0},
  {0x7FEFFFFFFFFFFFFFULL, 0},
  {0x8010000000000000ULL, 0},
  {0x8010000000000001ULL, 0},
  {0xFFE0000000000000ULL, 0},
  {0xFFEFFFFFFFFFFFFFULL, 0},
  {0x7FF0000000000000ULL, POS_INF},
  {0xFFF0000000000000ULL, NEG_INF},
  {0x7FF0000000000001ULL, NAN},
  {0x7FF0000080000000ULL, NAN},
  {0x7FF0000100000000ULL, NAN},
  {0x7FFFFFFFFFFFFFFFULL, NAN},
  {0xFFF0000000000001ULL, NAN},
  {0xFFF0000080000000ULL, NAN},
  {0xFFF0000100000000ULL, NAN},
  {0xFFFFFFFFFFFFFFFFULL, NAN}
};


int test_si_dftsv()
{
  int i, j, e[2];
  v128 d[128], a;
  unsigned long long exp;
  int cnt = sizeof(dftsv_samples)/sizeof(struct _dftsv_samples);

  DEBUG(1, "dftsv...........");
  
  /* For each sample on element 0 */
  for (e[0]=0; e[0]<cnt; e[0]++) {

    /* For each sample on element 1 */
    for (e[1]=0; e[1]<cnt; e[1]++) {
      a.ull[0] = dftsv_samples[e[0]].v;
      a.ull[1] = dftsv_samples[e[1]].v;

      d[0].qw = si_dftsv(a.qw, 0);
      d[1].qw = si_dftsv(a.qw, 1);
      d[2].qw = si_dftsv(a.qw, 2);
      d[3].qw = si_dftsv(a.qw, 3);
      d[4].qw = si_dftsv(a.qw, 4);
      d[5].qw = si_dftsv(a.qw, 5);
      d[6].qw = si_dftsv(a.qw, 6);
      d[7].qw = si_dftsv(a.qw, 7);
      d[8].qw = si_dftsv(a.qw, 8);
      d[9].qw = si_dftsv(a.qw, 9);
      d[10].qw = si_dftsv(a.qw, 10);
      d[11].qw = si_dftsv(a.qw, 11);
      d[12].qw = si_dftsv(a.qw, 12);
      d[13].qw = si_dftsv(a.qw, 13);
      d[14].qw = si_dftsv(a.qw, 14);
      d[15].qw = si_dftsv(a.qw, 15);
      d[16].qw = si_dftsv(a.qw, 16);
      d[17].qw = si_dftsv(a.qw, 17);
      d[18].qw = si_dftsv(a.qw, 18);
      d[19].qw = si_dftsv(a.qw, 19);
      d[20].qw = si_dftsv(a.qw, 20);
      d[21].qw = si_dftsv(a.qw, 21);
      d[22].qw = si_dftsv(a.qw, 22);
      d[23].qw = si_dftsv(a.qw, 23);
      d[24].qw = si_dftsv(a.qw, 24);
      d[25].qw = si_dftsv(a.qw, 25);
      d[26].qw = si_dftsv(a.qw, 26);
      d[27].qw = si_dftsv(a.qw, 27);
      d[28].qw = si_dftsv(a.qw, 28);
      d[29].qw = si_dftsv(a.qw, 29);
      d[30].qw = si_dftsv(a.qw, 30);
      d[31].qw = si_dftsv(a.qw, 31);
      d[32].qw = si_dftsv(a.qw, 32);
      d[33].qw = si_dftsv(a.qw, 33);
      d[34].qw = si_dftsv(a.qw, 34);
      d[35].qw = si_dftsv(a.qw, 35);
      d[36].qw = si_dftsv(a.qw, 36);
      d[37].qw = si_dftsv(a.qw, 37);
      d[38].qw = si_dftsv(a.qw, 38);
      d[39].qw = si_dftsv(a.qw, 39);
      d[40].qw = si_dftsv(a.qw, 40);
      d[41].qw = si_dftsv(a.qw, 41);
      d[42].qw = si_dftsv(a.qw, 42);
      d[43].qw = si_dftsv(a.qw, 43);
      d[44].qw = si_dftsv(a.qw, 44);
      d[45].qw = si_dftsv(a.qw, 45);
      d[46].qw = si_dftsv(a.qw, 46);
      d[47].qw = si_dftsv(a.qw, 47);
      d[48].qw = si_dftsv(a.qw, 48);
      d[49].qw = si_dftsv(a.qw, 49);
      d[50].qw = si_dftsv(a.qw, 50);
      d[51].qw = si_dftsv(a.qw, 51);
      d[52].qw = si_dftsv(a.qw, 52);
      d[53].qw = si_dftsv(a.qw, 53);
      d[54].qw = si_dftsv(a.qw, 54);
      d[55].qw = si_dftsv(a.qw, 55);
      d[56].qw = si_dftsv(a.qw, 56);
      d[57].qw = si_dftsv(a.qw, 57);
      d[58].qw = si_dftsv(a.qw, 58);
      d[59].qw = si_dftsv(a.qw, 59);
      d[60].qw = si_dftsv(a.qw, 60);
      d[61].qw = si_dftsv(a.qw, 61);
      d[62].qw = si_dftsv(a.qw, 62);
      d[63].qw = si_dftsv(a.qw, 63);
      d[64].qw = si_dftsv(a.qw, 64);
      d[65].qw = si_dftsv(a.qw, 65);
      d[66].qw = si_dftsv(a.qw, 66);
      d[67].qw = si_dftsv(a.qw, 67);
      d[68].qw = si_dftsv(a.qw, 68);
      d[69].qw = si_dftsv(a.qw, 69);
      d[70].qw = si_dftsv(a.qw, 70);
      d[71].qw = si_dftsv(a.qw, 71);
      d[72].qw = si_dftsv(a.qw, 72);
      d[73].qw = si_dftsv(a.qw, 73);
      d[74].qw = si_dftsv(a.qw, 74);
      d[75].qw = si_dftsv(a.qw, 75);
      d[76].qw = si_dftsv(a.qw, 76);
      d[77].qw = si_dftsv(a.qw, 77);
      d[78].qw = si_dftsv(a.qw, 78);
      d[79].qw = si_dftsv(a.qw, 79);
      d[80].qw = si_dftsv(a.qw, 80);
      d[81].qw = si_dftsv(a.qw, 81);
      d[82].qw = si_dftsv(a.qw, 82);
      d[83].qw = si_dftsv(a.qw, 83);
      d[84].qw = si_dftsv(a.qw, 84);
      d[85].qw = si_dftsv(a.qw, 85);
      d[86].qw = si_dftsv(a.qw, 86);
      d[87].qw = si_dftsv(a.qw, 87);
      d[88].qw = si_dftsv(a.qw, 88);
      d[89].qw = si_dftsv(a.qw, 89);
      d[90].qw = si_dftsv(a.qw, 90);
      d[91].qw = si_dftsv(a.qw, 91);
      d[92].qw = si_dftsv(a.qw, 92);
      d[93].qw = si_dftsv(a.qw, 93);
      d[94].qw = si_dftsv(a.qw, 94);
      d[95].qw = si_dftsv(a.qw, 95);
      d[96].qw = si_dftsv(a.qw, 96);
      d[97].qw = si_dftsv(a.qw, 97);
      d[98].qw = si_dftsv(a.qw, 98);
      d[99].qw = si_dftsv(a.qw, 99);
      d[100].qw = si_dftsv(a.qw, 100);
      d[101].qw = si_dftsv(a.qw, 101);
      d[102].qw = si_dftsv(a.qw, 102);
      d[103].qw = si_dftsv(a.qw, 103);
      d[104].qw = si_dftsv(a.qw, 104);
      d[105].qw = si_dftsv(a.qw, 105);
      d[106].qw = si_dftsv(a.qw, 106);
      d[107].qw = si_dftsv(a.qw, 107);
      d[108].qw = si_dftsv(a.qw, 108);
      d[109].qw = si_dftsv(a.qw, 109);
      d[110].qw = si_dftsv(a.qw, 110);
      d[111].qw = si_dftsv(a.qw, 111);
      d[112].qw = si_dftsv(a.qw, 112);
      d[113].qw = si_dftsv(a.qw, 113);
      d[114].qw = si_dftsv(a.qw, 114);
      d[115].qw = si_dftsv(a.qw, 115);
      d[116].qw = si_dftsv(a.qw, 116);
      d[117].qw = si_dftsv(a.qw, 117);
      d[118].qw = si_dftsv(a.qw, 118);
      d[119].qw = si_dftsv(a.qw, 119);
      d[120].qw = si_dftsv(a.qw, 120);
      d[121].qw = si_dftsv(a.qw, 121);
      d[122].qw = si_dftsv(a.qw, 122);
      d[123].qw = si_dftsv(a.qw, 123);
      d[124].qw = si_dftsv(a.qw, 124);
      d[125].qw = si_dftsv(a.qw, 125);
      d[126].qw = si_dftsv(a.qw, 126);
      d[127].qw = si_dftsv(a.qw, 127);

      // Check that the correct result is returned
      for (i=0; i<128; i++) {
	for (j=0; j<2; j++) {
	  exp = (i & dftsv_samples[e[j]].sv) ? 0xFFFFFFFFFFFFFFFFULL : 0ULL;

	  if (exp != d[i].ull[j]) {
	    DEBUG(1, failed);
	    DEBUG(2, "\t\tsv=%d element=%d samples=%d %d a=%f ", i, j, e[0], e[1], a.d[j]);
	    DEBUG(2, "d=0x%llx exp=0x%llx\n", d[i].ull[j], exp);
	    return (1);
	  }
	}
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}
#endif /* __SPU_EDP__ || __PPU__ */


int test_si_dfm()
{
  int i, j;
  double exp;

  DEBUG(1, "dfm.............");
  
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0, 4.0);
    b.vd = rand_vd(-4.0, 4.0);
    d.qw = si_dfm(a.qw, b.qw);
    for (j=0; j<2; j++) {
      exp = a.d[j] * b.d[j];
      if (exp != d.d[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%f ", j, a.d[j]);
	DEBUG(2, "b=%f d=%f exp=%f\n", b.d[j], d.d[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_dfma()
{
  int i, j;
  double exp;

  DEBUG(1, "dfma............");
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-1.0, 1.0);
    b.vd = rand_vd(-1.0, 1.0);
    c.vd = rand_vd(-1.0, 1.0);
    d.qw = si_dfma(a.qw, b.qw, c.qw);
    for (j=0; j<2; j++) {
      exp = a.d[j] * b.d[j] + c.d[j];
      if (exp != d.d[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%f ", j, a.d[j]);
	DEBUG(2, "b=%f c=%f d=%f exp=%f\n", b.d[j], c.d[j], d.d[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_dfms()
{
  int i, j;
  double exp;

  DEBUG(1, "dfms............");
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0, 4.0);
    b.vd = rand_vd(-4.0, 4.0);
    c.vd = rand_vd(-4.0, 4.0);
    d.qw = si_dfms(a.qw, b.qw, c.qw);
    for (j=0; j<2; j++) {
      exp = a.d[j] * b.d[j] - c.d[j];
      if (exp != d.d[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%f ", j, a.d[j]);
	DEBUG(2, "b=%f c=%f d=%f exp=%f\n", b.d[j], c.d[j], d.d[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_dfnma()
{
  int i, j;
  double exp, delta;

  DEBUG(1, "dfnma...........");
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0, 4.0);
    b.vd = rand_vd(-4.0, 4.0);
    c.vd = rand_vd(-4.0, 4.0);
    d.qw = si_dfnma(a.qw, b.qw, c.qw);
    for (j=0; j<2; j++) {
      exp = -(a.d[j] * b.d[j] + c.d[j]);
      /* Allow for inefficient compiler optimization of the above computation
       * of the expected results.
       */
      delta = exp - d.d[j];
      if (delta < 0.0) delta = -delta;
      if (delta > 0.000000002) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%f ", j, a.d[j]);
	DEBUG(2, "b=%f c=%f d=%f exp=%f\n", b.d[j], c.d[j], d.d[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_dfnms()
{
  int i, j;
  double exp, delta;

  DEBUG(1, "dfnms...........");
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0, 4.0);
    b.vd = rand_vd(-4.0, 4.0);
    c.vd = rand_vd(-4.0, 4.0);
    d.qw = si_dfnms(a.qw, b.qw, c.qw);
    for (j=0; j<2; j++) {
      exp = c.d[j] - a.d[j] * b.d[j];
      /* Allow for inefficient compiler optimization of the above computation
       * of the expected results.
       */
      delta = exp - d.d[j];
      if (delta < 0.0) delta = -delta;
      if (delta > 0.000000002) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%f ", j, a.d[j]);
	DEBUG(2, "b=%f c=%f d=%f exp=%f\n", b.d[j], c.d[j], d.d[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_dfs()
{
  int i, j;
  double exp;

  DEBUG(1, "dfs.............");
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0, 4.0);
    b.vd = rand_vd(-4.0, 4.0);
    d.qw = si_dfs(a.qw, b.qw);
    for (j=0; j<2; j++) {
      exp = a.d[j] - b.d[j];
      if (exp != d.d[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f b=%f d=%f exp=%f\n", a.d[j], b.d[j], d.d[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_dsync()
{
  DEBUG(1, "dsync...........");
  si_dsync();
  DEBUG(1, passed);
  return (0);
}


int test_si_eqv()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "eqv.............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_eqv(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = ~(a.ui[j] ^ b.ui[j]);
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_fa()
{
  int i, j;
  float exp;

  DEBUG(1, "fa..............");
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    b.vf = rand_vf(-4.0f, 4.0f);
    d.qw = si_fa(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = a.f[j] + b.f[j];
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f b=%f d=%f exp=%f\n", a.f[j], b.f[j], d.f[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_fceq()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "fceq............");
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    b.vf = rand_vf(-4.0f, 4.0f);
    for (j=0; j<4; j++) if (b.ui[j] & 0x10) b.ui[j] = a.ui[j];
    d.qw = si_fceq(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = (a.f[j] == b.f[j]) ? 0xFFFFFFFF : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f b=%f d=0x%x exp=0x%x\n", a.f[j], b.f[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_fcgt()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "fcgt............");
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    b.vf = rand_vf(-4.0f, 4.0f);
    d.qw = si_fcgt(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = (a.f[j] > b.f[j]) ? 0xFFFFFFFF : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f b=%f d=0x%x exp=0x%x\n", a.f[j], b.f[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_fcmeq()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "fcmeq...........");
  
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    for (j=0; j<4; j++) {
      if (rand_local() & 0x10) {
	b.ui[j] = a.ui[j];
	if (rand_local() & 0x10) {
	  b.ui[j] ^= 0x80000000;
	}
      }
    }
    d.qw = si_fcmeq(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = ((a.f[j] == b.f[j]) || (-a.f[j] == b.f[j]) || (a.f[j] == -b.f[j])) ? 0xFFFFFFFF : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f b=%f d=0x%x exp=0x%x\n", a.f[j], b.f[j], d.ui[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}


int test_si_fcmgt()
{
  int i, j;
  unsigned int exp;
  float abs_a, abs_b;

  DEBUG(1, "fcmgt...........");
  
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    b.vf = rand_vf(-4.0f, 4.0f);
    d.qw = si_fcmgt(a.qw, b.qw);
    for (j=0; j<4; j++) {
      abs_a = (a.f[j] < 0.0f) ? -a.f[j] : a.f[j];
      abs_b = (b.f[j] < 0.0f) ? -b.f[j] : b.f[j];
      exp = (abs_a > abs_b) ? 0xFFFFFFFF : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f b=%f d=0x%x exp=0x%x\n", a.f[j], b.f[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_fesd()
{
  int i, j;
  unsigned int exp, mantissa, sign, in;
  signed int exponent;

  DEBUG(1, "fesd............");

  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    d.qw = si_fesd(a.qw);
    for (j=0; j<4; j++) {
      in = a.ui[j&~1];
      mantissa = in & 0x007FFFFF;
      if (j & 1) {
	exp = mantissa << 29;
      } else {
	sign = in & 0x80000000;
	exponent  = (signed int)((in >> 23) & 0xFF) - 127;
	exp = sign | ((unsigned int)(exponent + 1023) << 20) | (mantissa >> 3);
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x d=0x%x exp=0x%x\n", in, d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_fm()
{
  int i, j;
  float exp;

  DEBUG(1, "fm..............");

  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    b.vf = rand_vf(-4.0f, 4.0f);
    d.qw = si_fm(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = a.f[j] * b.f[j];
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%f ", j, a.f[j]);
	DEBUG(2, "b=%f c=%f exp=%f\n", b.f[j], d.f[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_fma()
{
  int i, j;
  float exp;

  DEBUG(1, "fma.............");
  
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-1.0f, 1.0f);
    b.vf = rand_vf(-1.0f, 1.0f);
    c.vf = rand_vf(-1.0f, 1.0f);
    d.qw = si_fma(a.qw, b.qw, c.qw);
    for (j=0; j<4; j++) {
      exp = a.f[j] * b.f[j] + c.f[j];
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%f ", j, a.f[j]);
	DEBUG(2, "b=%f c=%f d=%f exp=%f\n", b.f[j], c.f[j], d.f[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_fms()
{
  int i, j;
  float exp;

  DEBUG(1, "fms.............");
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    b.vf = rand_vf(-4.0f, 4.0f);
    c.vf = rand_vf(-4.0f, 4.0f);
    d.qw = si_fms(a.qw, b.qw, c.qw);
    for (j=0; j<4; j++) {
      exp = a.f[j] * b.f[j] - c.f[j];
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%f ", j, a.f[j]);
	DEBUG(2, "b=%f c=%f d=%f exp=%f\n", b.f[j], c.f[j], d.f[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_fnms()
{
  int i, j;
  float exp, delta;

  DEBUG(1, "fnms............");
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    b.vf = rand_vf(-4.0f, 4.0f);
    c.vf = rand_vf(-4.0f, 4.0f);
    d.qw = si_fnms(a.qw, b.qw, c.qw);
    for (j=0; j<4; j++) {
      exp = c.f[j] - a.f[j] * b.f[j];
      /* Allow for inefficient compiler optimization of the above computation
       * of the expected results.
       */
      delta = exp - d.f[j];
      if (delta < 0.0f) delta = -delta;
      if (delta > 0.000002f) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%f ", j, a.f[j]);
	DEBUG(2, "b=%f c=%f d=%f exp=%f\n", b.f[j], c.f[j], d.f[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_frds()
{
  int i, j;
  float exp;

  DEBUG(1, "frds............");
  
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-1024.0, 1024.0);
    d.qw = si_frds(a.qw);
    for (j=0; j<4; j++) {
      exp = (j & 1) ? 0.0f : (float)(a.d[j>>1]);
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f d=%f exp=%f\n", a.d[j>>1], d.f[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}


int test_si_frest()
{
  int i, j;
  float err;

  DEBUG(1, "frest...........");

  for (i=0; i<samples; i++) {
    a.vf = rand_vf(1.0f, 3.0f);
    d.qw = si_fi(a.qw, si_frest(a.qw));
    for (j=0; j<4; j++) {
      err = a.f[j] * d.f[j];
      if ((err > 1.00025f) || (err < 0.99975)) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f d=%f err=%f\n", a.f[j], d.f[j], err);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_from_char()
{
  signed char a;

  DEBUG(1, "from_char.......");
  a = (signed char)rand_local();
  d.qw = si_from_char(a);
  if (d.c[3] != a) {
    DEBUG(1, failed);
    DEBUG(2, "\t\ta=0x%x exp=0x%x\n", a, d.c[3]);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_from_double()
{
  double a;

  DEBUG(1, "from_double.....");
  a = rand_double(-1.0, 1.0);
  d.qw = si_from_double(a);
  if (d.d[0] != a) {
    DEBUG(1, failed);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_from_float()
{
  float a;

  DEBUG(1, "from_float......");
  a = rand_float(-1.0f, 1.0f);
  d.qw = si_from_float(a);
  if (d.f[0] != a) {
    DEBUG(1, failed);
    DEBUG(2, "\t\ta=%f exp=%f\n", a, d.f[0]);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_from_int()
{
  int a;

  DEBUG(1, "from_int........");
  a = (int)rand_local();
  d.qw = si_from_int(a);
  if (d.i[0] != a) {
    DEBUG(1, failed);
    DEBUG(2, "\t\ta=0x%x exp=0x%x\n", a, d.i[0]);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_from_llong()
{
  long long a;

  DEBUG(1, "from_llong......");
  a = (long long)rand_ullong();
  d.qw = si_from_llong(a);
  if (d.ll[0] != a) {
    DEBUG(1, failed);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_from_ptr()
{
  void * a;

  DEBUG(1, "from_ptr........");
  a = (void *)rand_local();
  d.qw = si_from_ptr(a);
  if (d.p[0] != a) {
    DEBUG(1, failed);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_from_short()
{
  short a;

  DEBUG(1, "from_short......");
  a = (short)rand_local();
  d.qw = si_from_short(a);
  if (d.s[1] != a) {
    DEBUG(1, failed);
    DEBUG(2, "\t\ta=0x%x exp=0x%x\n", a, d.s[1]);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_from_uchar()
{
  unsigned char a;

  DEBUG(1, "from_uchar......");
  a = (unsigned char)rand_local();
  d.qw = si_from_uchar(a);
  if (d.uc[3] != a) {
    DEBUG(1, failed);
    DEBUG(2, "\t\ta=0x%x exp=0x%x\n", a, d.uc[3]);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_from_uint()
{
  unsigned int a;

  DEBUG(1, "from_uint.......");
  a = (unsigned int)rand_local();
  d.qw = si_from_uint(a);
  if (d.ui[0] != a) {
    DEBUG(1, failed);
    DEBUG(2, "\t\ta=0x%x exp=0x%x\n", a, d.ui[0]);
    return (1);
  }
  DEBUG(1, passed);

  return (0);
}


int test_si_from_ullong()
{
  unsigned long long a;

  DEBUG(1, "from_llong......");
  a = (unsigned long long)rand_ullong();
  d.qw = si_from_ullong(a);
  if (d.ull[0] != a) {
    DEBUG(1, failed);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_from_ushort()
{
  unsigned short a;

  DEBUG(1, "from_ushort.....");
  a = (unsigned short)rand_local();
  d.qw = si_from_ushort(a);
  if (d.us[1] != a) {
    DEBUG(1, failed);
    DEBUG(2, "\t\ta=0x%x exp=0x%x\n", a, d.us[1]);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_frsqest()
{
  int i, j;
  float err;

  DEBUG(1, "frsqest.........");
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(1.0f, 3.0f);
    d.qw = si_fi(a.qw, si_frsqest(a.qw));
    for (j=0; j<4; j++) {
      err = a.f[j] * d.f[j] * d.f[j];
      if ((err > 1.0004f) || (err < 0.9996)) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f d=%f err=%f\n", a.f[j], d.f[j], err);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_fs()
{
  int i, j;
  float exp;

  DEBUG(1, "fs..............");
  
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    b.vf = rand_vf(-4.0f, 4.0f);
    d.qw = si_fs(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = a.f[j] - b.f[j];
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f b=%f d=%f exp=%f\n", a.f[j], b.f[j], d.f[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

qword dbl_add(qword a, qword b)
{
  return (si_dfa(a, b));
}


#ifdef __SPU__
/* This function derives the round mode from a series of arithmetic
 * tests. The second parameter, loop_var, was added to keep the optimizer
 * from pulling the call to this function out of the loop. 
 */
unsigned int do_round_test(int element, int loop_var)
{
  int idx, index;
  v128 t1, t2, t3, t4;

  if (loop_var >= 0) {
    t1.qw = dbl_add((qword)spu_splats((unsigned long long)0x3FF0000000000000ULL),
		    (qword)spu_splats((unsigned long long)0x3FE0000000000001ULL));

    t2.qw = dbl_add((qword)spu_splats((unsigned long long)0x3FF0000000000000ULL),
		    (qword)spu_splats((unsigned long long)0x3FE0000000000003ULL));
    
    t3.qw = dbl_add((qword)spu_splats((unsigned long long)0xBFF0000000000000ULL),
		    (qword)spu_splats((unsigned long long)0xBFE0000000000001ULL));
    
    t4.qw = dbl_add((qword)spu_splats((unsigned long long)0xBFF0000000000000ULL),
		    (qword)spu_splats((unsigned long long)0xBFE0000000000003ULL));
	     
    idx = element * 2 + 1;
    index = (((t1.ui[idx] & 1) << 3) |
	     ((t2.ui[idx] & 1) << 2) |
	     ((t3.ui[idx] & 1) << 1) |
	     ((t4.ui[idx] & 1) << 0));

    if (index == 0) return 0;
    if (index == 5) return 1;
    
    if (index == 9) return 2;
    if (index == 6) return 3;
  }
  
  return (-1);
}
#endif 


int test_si_fscrrd()
{
#ifdef __SPU__
  int i;
#endif

  DEBUG(1, "fscrrd..........");
  a.qw = si_fscrrd();
  DEBUG(1, passed);


  /* Change the rounding mode */
  b.qw = a.qw;
  b.ui[0] ^= 0x100;

  DEBUG(1, "fscrwr..........");
  si_fscrwr(b.qw);

  c.qw = si_fscrrd();
  si_fscrwr(a.qw);

  /* Can not verify the results unless we are running on a SPU.
   */
#ifdef __SPU__
  for (i=0; i<4; i++) {
    if (b.ui[i] != c.ui[i]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", i);
	DEBUG(2, "b=0x%x c=0x%x\n", b.ui[i], c.ui[i]);
	return (1);
    }
  }

  /* Verfiy independent rounding mode controls
   */
  for (i=0; i<16; i++) {
    unsigned int r0, r1;

    b.ui[0] = (b.ui[0] & 0xFFFF00FF) | (i << 8);
    si_fscrwr(b.qw);

    r0 = do_round_test(0, i);
    r1 = do_round_test(1, i);
    
    if ((int)((r0 << 2) | r1) != i) {
      DEBUG(1, failed);
      DEBUG(2, "\tFailed validating independent rounding mode control\n");
      DEBUG(2, "\ti=%d r0=%d r1=%d\n", i, r0, r1);
      si_fscrwr(a.qw);
      return (1);
    }
  }
  si_fscrwr(a.qw);
#endif

  DEBUG(1, passed);
  return (0);
}



int test_si_fsm()
{
  int i, j;
  unsigned int exp;
  
  DEBUG(1, "fsm.............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.qw = si_fsm(a.qw);
    for (j=0; j<4; j++) {
      exp = (a.ui[0] & (0x8 >> j)) ? 0xFFFFFFFF : 0x0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x d=0x%x exp=0x%x\n", a.ui[0], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

  
int test_si_fsmb()
{
  int i, j;
  unsigned char exp;

  DEBUG(1, "fsmb............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.qw = si_fsmb(a.qw);
    for (j=0; j<16; j++) {
      exp = (a.ui[0] & (0x8000 >> j)) ? 0xFF : 0x00;
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x d=0x%x exp=0x%x\n", a.ui[0], d.uc[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_fsmbi()
{
  int j;
  unsigned char exp;

  DEBUG(1, "fsmbi...........");
  d.qw = si_fsmbi(0xA55A);
  for (j=0; j<16; j++) {
    exp = (0xA55A & (0x8000 >> j)) ? 0xFF : 0x00;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0xA55A d=0x%x exp=0x%x\n", d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_fsmh()
{
  int i, j;
  unsigned short exp;

  DEBUG(1, "fsmh............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.qw = si_fsmh(a.qw);
    for (j=0; j<8; j++) {
      exp = (a.ui[0] & (0x80 >> j)) ? 0xFFFF : 0x0000;
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x d=0x%x exp=0x%x\n", a.ui[0], d.us[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_gb()
{
  int i, j, k;
  unsigned int exp;

  DEBUG(1, "gb..............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.qw = si_gb(a.qw);
    for (j=0; j<4; j++) {
      exp = 0;
      if (j==0) {
	for (k=0; k<4; k++) {
	  if (a.ui[k] & 1) exp |= (1 << (3-k));
	}
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_gbb()
{
  int i, j, k;
  unsigned int exp;

  DEBUG(1, "gbb.............");
  /* byte */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.qw = si_gbb(a.qw);
    for (j=0; j<4; j++) {
      exp = 0;
      if (j==0) {
	for (k=0; k<16; k++) {
	  if (a.uc[k] & 1) exp |= (1 << (15-k));
	}
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_gbh()
{
  int i, j, k;
  unsigned int exp;

  DEBUG(1, "gbh.............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.qw = si_gbh(a.qw);
    for (j=0; j<4; j++) {
      exp = 0;
      if (j==0) {
	for (k=0; k<8; k++) {
	  if (a.us[k] & 1) exp |= (1 << (7-k));
	}
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_heq()
{
  int i;

  DEBUG(1, "heq.............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    if (a.ui[0] == b.ui[0]) continue;
    si_heq(a.qw, b.qw);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_heqi()
{
  int i;

  DEBUG(1, "heqi............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    if (a.ui[0] == 123) continue;
    si_heqi(a.qw, 123);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_hgt()
{
  int i;

  DEBUG(1, "hgt.............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    if (a.i[0] > b.i[0]) continue;
    si_hgt(a.qw, b.qw);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_hgti()
{
  int i;

  DEBUG(1, "hgti............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    if (a.i[0] > -3) continue;
    si_hgti(a.qw, -3);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_hlgt()
{
  int i;

  DEBUG(1, "hlgt............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    if (a.ui[0] > b.ui[0]) continue;
    si_hlgt(a.qw, b.qw);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_hlgti()
{
  int i;

  DEBUG(1, "hlgti...........");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    if (a.ui[0] > 200) continue;
    si_hlgti(a.qw, 200);
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_il()
{
  int j;

  DEBUG(1, "il..............");
  d.qw = si_il(-23113);
  for (j=0; j<4; j++) {
    if (0xFFFFA5B7 != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "d=0x%x exp=0xFFFEA5B7\n", d.ui[j]);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_ila()
{
  int j;

  DEBUG(1, "ila.............");
  d.qw = si_ila(0x2A5B7);
  for (j=0; j<4; j++) {
    if (0x2A5B7 != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "d=0x%x exp=0x2A5B7\n", d.ui[j]);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_ilh()
{
  int j;

  DEBUG(1, "ilh.............");
  d.qw = si_ilh(-23114);
  for (j=0; j<8; j++) {
    if (0xA5B6 != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "d=0x%x exp=0xA5B7\n", d.us[j]);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_ilhu()
{
  int j;

  DEBUG(1, "ilhu............");
  d.qw = si_ilhu(0xFFFFA5B2);
  for (j=0; j<4; j++) {
    if (0xA5B20000 != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "d=0x%x exp=0xA5B70000\n", d.ui[j]);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_iohl()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "iohl............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.qw = a.qw;
    d.qw = si_iohl(a.qw, 0xB345);
    for (j=0; j<4; j++) {
      exp = a.ui[j] | 0xB345;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_lnop()
{
  DEBUG(1, "lnop............");
  si_lnop();
  DEBUG(1, passed);
  return (0);
}


int test_si_lqa()
{
#ifdef __SPU__
  int j;
  unsigned char *p, exp;

  DEBUG(1, "lqa.............");

  p = (unsigned char *)(823 & ~0xF);

  d.qw = si_lqa(823);
  for (j=0; j<16; j++) {
    exp = p[j];
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "imm=23 d=0x%x exp=0x%x\n", d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
#else
  /* This intrinsic is only tested on the SPU since we can not 
   * guarantee the memory map when running in other environments.
   */
  DEBUG(1, "lqa.............untested\n");
#endif

  return (0);
}


int test_si_lqd()
{
  int j;
  unsigned char exp;

  set_buffer();
  DEBUG(1, "lqd.............");
  a.ui[0] = (unsigned int)buffer;
  d.qw = si_lqd(a.qw, 33);
  for (j=0; j<16; j++) {
    exp = buffer[(33 & 0xF0) + j];
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x imm=33 d=0x%x exp=0x%x\n", a.ui[0], d.uc[j], exp);
      return (1);
    }
  }

  a.ui[0] = (unsigned int)(buffer - 300);
  d.qw = si_lqd(a.qw, 511);
  for (j=0; j<16; j++) {
    exp = buffer[(((511 & ~0xF)-300) & 0xF0) + j];
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x imm=511 d=0x%x exp=0x%x\n", a.ui[0], d.uc[j], exp);
      return (1);
    }
  }

  a.ui[0] = (unsigned int)(buffer + 589);
  d.qw = si_lqd(a.qw, -512);
  for (j=0; j<16; j++) {
    exp = buffer[((589-512) & 0xF0) + j];
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x imm=-512 d=0x%x exp=0x%x\n", a.ui[0], d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_lqr()
{
#ifndef __SPU__
  int j;
  unsigned char exp;
#endif

  DEBUG(1, "lqr.............");
#ifdef __SPU__
  /* This intrinsic is not tested on the SPU because of the lack of ability
   * to control relative addressing in an optimized compiled environment.
   */
  DEBUG(1, "untested\n");
#else
  set_buffer();
  d.qw = si_lqr((unsigned int)(&buffer[37]));
  for (j=0; j<16; j++) {
    exp = buffer[(37 & 0xF0) + j];
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "imm=buffer[37] d=0x%x exp=0x%x\n", d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
#endif
  return (0);
}

int test_si_lqx()
{
  int j;
  unsigned char exp;

  set_buffer();
  DEBUG(1, "lqx.............");
  a.ui[0] = (unsigned int)buffer;
  b.ui[0] = rand_local() & 0xFF;
  d.qw = si_lqx(a.qw, b.qw);
  for (j=0; j<16; j++) {
    exp = buffer[(b.ui[0] & 0xF0) + j];
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[0], b.ui[0], d.uc[j], exp);
      return (1);
    }
  }
  a.ui[0] = rand_local() & 0xFF;
  b.ui[0] = (unsigned int)buffer;
  d.qw = si_lqx(a.qw, b.qw);
  for (j=0; j<16; j++) {
    exp = buffer[(a.ui[0] & 0xF0) + j];
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[0], b.ui[0], d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_mfspr()
{
  /* Since there are no defined spr registers. This test will do nothing more
   * that read an spr and write it.
   */
  qword spr;

  DEBUG(1, "mfspr...........");
  spr = si_mfspr(0);
  DEBUG(1, passed);
  DEBUG(1, "mtspr...........");
  si_mtspr(0, spr);
  DEBUG(1, passed);
  return (0);
}


int test_si_mpy()
{
  int i, j;
  signed int exp;

  DEBUG(1, "mpy.............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_mpy(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = (signed int)a.s[2*j+1] * (signed int)b.s[2*j+1];
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%d ", j, a.s[2*j+1]);
	DEBUG(2, "b=%d d=%d exp=%d\n", b.s[2*j+1], d.i[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_mpya()
{
  int i, j;
  signed int exp;

  DEBUG(1, "mpya............");

  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();
    d.qw = si_mpya(a.qw, b.qw, c.qw);
    for (j=0; j<4; j++) {
      exp = (signed int)a.s[2*j+1] * (signed int)b.s[2*j+1] + c.i[j];
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%d ", j, a.s[2*j+1]);
	DEBUG(2, "b=%d c=%d d=%d exp=%d\n", b.s[2*j+1], c.s[2*j+1], d.i[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_mpyh()
{
  int i, j;
  signed int exp;

  DEBUG(1, "mpyh............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_mpyh(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = ((signed int)a.s[2*j] * (signed int)b.s[2*j+1]) << 16;
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%d ", j, a.s[2*j]);
	DEBUG(2, "b=%d d=%d exp=%d\n", b.s[2*j+1], d.i[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_mpyhh()
{
  int i, j;
  signed int exp;

  DEBUG(1, "mpyhh...........");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_mpyhh(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = (signed int)a.s[2*j] * (signed int)b.s[2*j];
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%d ", j, a.s[2*j]);
	DEBUG(2, "b=%d d=%d exp=%d\n", b.s[2*j], d.i[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}




int test_si_mpyhha()
{
  int i, j;
  signed int exp;

  DEBUG(1, "mpyhha..........");

  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();
    d.qw = si_mpyhha(a.qw, b.qw, c.qw);
    for (j=0; j<4; j++) {
      exp = (signed int)a.s[2*j] * (signed int)b.s[2*j] + c.i[j];
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%d ", j, a.s[2*j]);
	DEBUG(2, "b=%d c=%d d=%d exp=%d\n", b.s[2*j], c.i[j], d.i[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_mpyhhau()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "mpyhhau.........");

  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();
    d.qw = si_mpyhhau(a.qw, b.qw, c.qw);
    for (j=0; j<4; j++) {
      exp = (unsigned int)a.us[2*j] * (unsigned int)b.us[2*j] + c.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=0x%x ", j, a.us[2*j]);
	DEBUG(2, "b=0x%x c=0x%x d=0x%x exp=0x%x\n", b.us[2*j], c.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_mpyhhu()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "mpyhhu..........");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_mpyhhu(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = (unsigned int)a.us[2*j] * (unsigned int)b.us[2*j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=0x%x ", j, a.us[2*j]);
	DEBUG(2, "b=0x%x d=0x%x exp=0x%x\n", b.us[2*j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_mpyi()
{
  int i, j;
  signed int exp;

  DEBUG(1, "mpyi............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.qw = si_mpyi(a.qw, -512);
    for (j=0; j<4; j++) {
      exp = (signed int)a.s[2*j+1] * -512;
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tsigned (lit) element=%d a=%d ", j, a.s[2*j+1]);
	DEBUG(2, "b=-512 d=%d exp=%d\n", d.i[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_mpys()
{
  int i, j;
  signed int exp;

  DEBUG(1, "mpys............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_mpys(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = ((signed int)a.s[2*j+1] * (signed int)b.s[2*j+1]) >> 16;
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%d ", j, a.s[2*j+1]);
	DEBUG(2, "b=%d d=%d exp=%d\n", b.s[2*j+1], d.i[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_mpyu()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "mpyu............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_mpyu(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = (unsigned int)a.us[2*j+1] * (unsigned int)b.us[2*j+1];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%d ", j, a.us[2*j+1]);
	DEBUG(2, "b=%d d=%d exp=%d\n", b.us[2*j+1], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_mpyui()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "mpyui...........");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.qw = si_mpyui(a.qw, 511);
    for (j=0; j<4; j++) {
      exp = (unsigned int)a.us[2*j+1] * (unsigned int)(511);
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%d ", j, a.us[2*j+1]);
	DEBUG(2, "b=511 d=%d exp=%d\n", d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_nand()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "nand............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_nand(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = ~(a.ui[j] & b.ui[j]);
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_nop()
{
  DEBUG(1, "nop.............");
  si_nop();
  DEBUG(1, passed);
  return (0);
}


int test_si_nor()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "nor.............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_nor(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = ~(a.ui[j] | b.ui[j]);
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_or()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "or..............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_or(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = a.ui[j] | b.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_orbi()
{
  int j;
  unsigned char exp;

  DEBUG(1, "orbi............");
  a.vui = rand_vui();
  d.qw = si_orbi(a.qw, 267);
  for (j=0; j<16; j++) {
    exp = a.uc[j] | 267;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=267 d=0x%x exp=0x%x\n", a.uc[j], d.uc[j], exp);
      return (1);
    }
  }

  a.vui = rand_vui();
  d.qw = si_orbi(a.qw, -315);
  for (j=0; j<16; j++) {
    exp = a.uc[j] | -315;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-315 d=0x%x exp=0x%x\n", a.uc[j], d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_orc()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "orc.............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_orc(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = a.ui[j] | ~b.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_orx()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "orx.............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.qw = si_orx(a.qw);
    for (j=0; j<4; j++) {
      if (j == 0) {
	exp = a.ui[0] | a.ui[1] | a.ui[2] | a.ui[3];
      } else {
	exp = 0;
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x 0x%x 0x%x 0x%x  ", a.ui[0], a.ui[1], a.ui[2], a.ui[3]);
	DEBUG(1, "d=0x%x exp=0x%x\n", d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_orhi()
{
  int j;
  unsigned short exp;

  DEBUG(1, "orhi............");
  a.vui = rand_vui();
  d.qw = si_orhi(a.qw, 511);
  for (j=0; j<8; j++) {
    exp = a.us[j] | 511;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=511 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_ori()
{
  int j;
  unsigned int exp;

  DEBUG(1, "ori.............");
  a.vui = rand_vui();
  d.qw = si_ori(a.qw, -512);
  for (j=0; j<4; j++) {
    exp = a.ui[j] | -512;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-511 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_rchcnt()
{
  qword cnt;

  DEBUG(1, "rdchcnt.........");
  cnt = si_rchcnt(SPU_RdEventStat);
  DEBUG(1, passed);
  return (0);
}


int test_si_rdch()
{
  qword i;

  DEBUG(1, "rdch............");
  i = si_rdch(SPU_RdDec);
  DEBUG(1, passed);
  return (0);
}


int test_si_rot()
{
  int i, j;
  int count;
  unsigned int exp;

  DEBUG(1, "rot.............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_rot(a.qw, b.qw);
    for (j=0; j<4; j++) {
      count = (int)b.ui[j] & 0x1F;
      exp = (a.ui[j] << count) | (a.ui[j] >> (32-count));
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_roth()
{
  int i, j;
  int count;
  unsigned short exp;

  DEBUG(1, "roth............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_roth(a.qw, b.qw);
    for (j=0; j<8; j++) {
      count = (int)b.us[j] & 0xF;
      exp = (a.us[j] << count) | (a.us[j] >> (16-count));
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.us[j], b.us[j], d.us[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_rothi()
{
  int j;
  unsigned short exp;

  DEBUG(1, "rothi...........");
  a.vui = rand_vui();
  d.qw = si_rothi(a.qw, 0x10);
  for (j=0; j<8; j++) {
    exp = a.us[j];
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=0 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }

  a.vui = rand_vui();
  d.qw = si_rothi(a.qw, 0xF);
  for (j=0; j<8; j++) {
    exp = (a.us[j] << 15) | (a.us[j] >> 1);
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=15 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  a.vui = rand_vui();
  d.qw = si_rothi(a.qw, 54);
  for (j=0; j<8; j++) {
    exp = (a.us[j] << 6) | (a.us[j] >> 10);
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=6 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_rothm()
{
  int i, j;
  int count;
  unsigned short exp;

  DEBUG(1, "rothm...........");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_rothm(a.qw, b.qw);
    for (j=0; j<8; j++) {
      count = (int)b.us[j] & 0x1F;
      if (count > 16) {
	count = -count & 0xF;
	exp = a.us[j] >> count;
      } else {
	exp = (count) ? 0 : a.us[j];
      }
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.us[j], b.us[j], d.us[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_rothmi()
{
  int j;
  unsigned short exp;

  DEBUG(1, "rothmi..........");
  
  a.vui = rand_vui();
  d.qw = si_rothmi(a.qw, 0);
  for (j=0; j<8; j++) {
    exp = a.us[j];
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=0 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  a.vui = rand_vui();
  d.qw = si_rothmi(a.qw, -16);
  for (j=0; j<8; j++) {
    exp = 0;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-16 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  a.vui = rand_vui();
  d.qw = si_rothmi(a.qw, -7);
  for (j=0; j<8; j++) {
    exp = a.us[j] >> 7;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-7 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_roti()
{
  int j;
  unsigned int exp;

  DEBUG(1, "roti............");
  a.vui = rand_vui();
  d.qw = si_roti(a.qw, 0x20);
  for (j=0; j<4; j++) {
    exp = a.ui[j];
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=0 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }

  a.vui = rand_vui();
  d.qw = si_roti(a.qw, 0x1F);
  for (j=0; j<4; j++) {
    exp = (a.ui[j] << 31) | (a.ui[j] >> 1);
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=31 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  a.vui = rand_vui();
  d.qw = si_roti(a.qw, 39);
  for (j=0; j<4; j++) {
    exp = (a.ui[j] << 7) | (a.ui[j] >> 25);
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=7 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_rotm()
{
  int i, j;
  int count;
  unsigned int exp;

  DEBUG(1, "rotm............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_rotm(a.qw, b.qw);
    for (j=0; j<4; j++) {
      count = (int)b.ui[j] & 0x3F;
      if (count > 32) {
	count = -count & 0x1F;
	exp = a.ui[j] >> count;
      } else {
	exp = (count) ? 0 : a.ui[j];
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_rotma()
{
  int i, j;
  int count;
  signed int exp;

  DEBUG(1, "rotma...........");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_rotma(a.qw, b.qw);
    for (j=0; j<4; j++) {
      count = (int)b.i[j] & 0x3F;
      if (count > 32) {
	count = -count & 0x1F;
	exp = a.i[j] >> count;
      } else {
	if (count) {
	  exp = (a.i[j] < 0) ? -1 : 0;
	} else {
	  exp = a.i[j];
	}
      }
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.i[j], b.i[j], d.i[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_rotmah()
{
  int i, j;
  int count;
  signed short exp;

  DEBUG(1, "rotmah..........");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_rotmah(a.qw, b.qw);
    for (j=0; j<8; j++) {
      count = (int)b.us[j] & 0x1F;
      if (count > 16) {
	count = -count & 0xF;
	exp = a.s[j] >> count;
      } else {
	if (count) {
	  exp = (a.s[j] & 0x8000) ? 0xFFFF : 0;
	} else {
	  exp = a.s[j];
	}
      }
      if (exp != d.s[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.s[j], b.s[j], d.s[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_rotmahi()
{
  int j;
  signed short exp;

  DEBUG(1, "rotmahi.........");
  a.vui = rand_vui();
  d.qw = si_rotmahi(a.qw, 0);
  for (j=0; j<8; j++) {
    exp = a.s[j];
    if (exp != d.s[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=0 d=0x%x exp=0x%x\n", a.s[j], d.s[j], exp);
      return (1);
    }
  }
  a.vui = rand_vui();
  d.qw = si_rotmahi(a.qw, -16);
  for (j=0; j<8; j++) {
    exp = (a.us[j] & 0x8000) ? -1 : 0;
    if (exp != d.s[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-16 d=0x%x exp=0x%x\n", a.s[j], d.s[j], exp);
      return (1);
    }
  }
  a.vui = rand_vui();
  d.qw = si_rotmahi(a.qw, -7);
  for (j=0; j<8; j++) {
    exp = a.s[j] >> 7;
    if (exp != d.s[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-7 d=0x%x exp=0x%x\n", a.s[j], d.s[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_rotmai()
{
  int j;
  signed int exp;

  DEBUG(1, "rotmai..........");
  a.vui = rand_vui();
  d.qw = si_rotmai(a.qw, 0);
  for (j=0; j<4; j++) {
    exp = a.i[j];
    if (exp != d.i[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=0 d=0x%x exp=0x%x\n", a.i[j], d.i[j], exp);
      return (1);
    }
  }

  a.vui = rand_vui();
  d.qw = si_rotmai(a.qw, -32);
  for (j=0; j<4; j++) {
    exp = (a.i[j] < 0) ? -1 : 0;
    if (exp != d.i[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-32 d=0x%x exp=0x%x\n", a.i[j], d.i[j], exp);
      return (1);
    }
  }

  a.vui = rand_vui();
  d.qw = si_rotmai(a.qw, -13);
  for (j=0; j<4; j++) {
    exp = a.i[j] >> 13;
    if (exp != d.i[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-13 d=0x%x exp=0x%x\n", a.i[j], d.i[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_rotmi()
{
  int j;
  unsigned int exp;

  DEBUG(1, "rotmi...........");
  a.vui = rand_vui();
  d.qw = si_rotmi(a.qw, 0);
  for (j=0; j<4; j++) {
    exp = a.ui[j];
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=0 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }

  a.vui = rand_vui();
  d.qw = si_rotmi(a.qw, -32);
  for (j=0; j<4; j++) {
    exp = 0;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-32 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }

  a.vui = rand_vui();
  d.qw = si_rotmi(a.qw, -13);
  for (j=0; j<4; j++) {
    exp = a.ui[j] >> 13;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-13 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_rotqbi()
{
  int i, j;
  int cnt;
  unsigned int exp;

  DEBUG(1, "rotqbi..........");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    c.vui = rand_vui();
    d.qw = si_rotqbi(a.qw, c.qw);

    cnt = c.ui[0] & 7;

    for (j=0; j<4; j++) {
      exp = a.ui[j] << cnt;
      if (cnt) exp |= (a.ui[(j+1)&3] >> (32-cnt));

      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "count=0x%x d=0x%x exp=0x%x\n", c.ui[0], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_rotqbii()
{
  int j;
  int cnt;
  unsigned int exp;

  DEBUG(1, "rotqbii.........");
  a.vui = rand_vui();
  d.qw = si_rotqbii(a.qw, 27);
  cnt = 3;
  for (j=0; j<4; j++) {
    exp = a.ui[j] << cnt;
    if (cnt) exp |= (a.ui[(j+1)&3] >> (32-cnt));
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\t(lit) element=%d ", j);
      DEBUG(2, "count=27 d=0x%x exp=0x%x\n", d.ui[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_rotqby()
{
  int i, j;
  int cnt;
  unsigned char exp;

  DEBUG(1, "rotqby..........");

  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    c.vui = rand_vui();
    d.qw = si_rotqby(a.qw, c.qw);
    cnt = c.ui[0] & 15;
    for (j=0; j<16; j++) {
      exp = a.uc[(j+cnt) & 15];
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "count=0x%x d=0x%x exp=0x%x\n", c.ui[0], d.uc[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_rotqbybi()
{
  int i, j;
  int cnt;
  unsigned char exp;

  DEBUG(1, "rotqbybi........");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    c.vui = rand_vui();
    d.qw = si_rotqbybi(a.qw, c.qw);
    cnt = (c.ui[0] >> 3) & 15;
    for (j=0; j<16; j++) {
      exp = a.uc[(j+cnt) & 15];
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "count=0x%x d=0x%x exp=0x%x\n", c.ui[0], d.uc[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_rotqbyi()
{
  int j;
  unsigned char exp;

  DEBUG(1, "rotqbyi.........");
  a.vui = rand_vui();
  d.qw = si_rotqbyi(a.qw, 27);
  for (j=0; j<16; j++) {
    exp = a.uc[(j+27)&15];
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "count=27 d=0x%x exp=0x%x\n", d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_rotqmbi()
{
  int i, j;
  int cnt;
  unsigned int exp;

  DEBUG(1, "rotqmbi.........");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    c.vui = rand_vui();
    d.qw = si_rotqmbi(a.qw, c.qw);
    cnt = -c.i[0] & 7;
    for (j=0; j<4; j++) {
      exp = a.ui[j] >> cnt;
      if ((j > 0) && cnt) exp |= (a.ui[j-1] << (32-cnt));

      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "count=0x%x d=0x%x exp=0x%x\n", c.ui[0], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_rotqmbii()
{
  int j;
  int cnt;
  unsigned int exp;

  DEBUG(1, "rotqmbii........");
  a.vui = rand_vui();
  d.qw = si_rotqmbii(a.qw, -5);
  cnt = 5;

  for (j=0; j<4; j++) {
    exp = a.ui[j] >> cnt;
    if ((j > 0) && cnt) exp |= (a.ui[j-1] << (32-cnt));
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "count=-5 d=0x%x exp=0x%x\n", d.ui[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_rotqmby()
{
  int i, j;
  int cnt;
  unsigned char exp;

  DEBUG(1, "rotqmby.........");

  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    c.vui = rand_vui();
    d.qw = si_rotqmby(a.qw, c.qw);
    cnt = c.i[0] & 0x1F;
    if (cnt) cnt |= 0xFFFFFFE0;
    for (j=0; j<16; j++) {
      exp = 0;
      if ((j + cnt) >= 0) exp = a.uc[j+cnt];
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "count=0x%x d=0x%x exp=0x%x\n", c.ui[0], d.uc[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_rotqmbyi()
{
  int j;
  unsigned char exp;

  DEBUG(1, "rotqmbyi........");
  a.vui = rand_vui();
  d.qw = si_rotqmbyi(a.qw, -5);

  for (j=0; j<16; j++) {
    exp = (j >= 5) ? a.uc[j-5] : 0;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "count=-5 d=0x%x exp=0x%x\n", d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_rotqmbybi()
{
  int i, j;
  int cnt;
  unsigned char exp;

  DEBUG(1, "rotqmbybi.......");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    c.vui = rand_vui();
    d.qw = si_rotqmbybi(a.qw, c.qw);
    cnt = (c.i[0] >> 3) & 0x1F;
    if (cnt) cnt |= 0xFFFFFFE0;
    for (j=0; j<16; j++) {
      exp = 0;
      if ((j + cnt) >= 0) exp = a.uc[j+cnt];
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "count=0x%x d=0x%x exp=0x%x\n", c.ui[0], d.uc[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_selb()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "selb............");

  /* unsigned int */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();
    d.qw = si_selb(a.qw, b.qw, c.qw);
    for (j=0; j<4; j++) {
      exp = (a.ui[j]& ~c.ui[j]) | (b.ui[j] & c.ui[j]);
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\telement=%d a=0x%x ", j, a.ui[j]);
	DEBUG(2, "b=0x%x c=0x%x d=0x%x exp=0x%x\n", b.ui[j], c.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_sf()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "sf..............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_sf(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = b.ui[j] - a.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_sfh()
{
  int i, j;
  unsigned short exp;

  DEBUG(1, "sfh.............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_sfh(a.qw, b.qw);
    for (j=0; j<8; j++) {
      exp = b.us[j] - a.us[j];
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.us[j], b.us[j], d.us[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_sfhi()
{
  int j;
  unsigned short exp;

  DEBUG(1, "sfhi............");
  a.vui = rand_vui();
  d.qw = si_sfhi(a.qw, 511);
  for (j=0; j<8; j++) {
    exp = 511 - a.us[j];
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=511 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_sfi()
{
  int j;
  unsigned int exp;

  DEBUG(1, "sfi.............");
  a.vui = rand_vui();
  d.qw = si_sfi(a.qw, -512);
  for (j=0; j<4; j++) {
    exp = -512 - a.ui[j];
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-512 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_sfx()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "sfx.............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();
    d.qw = si_sfx(a.qw, b.qw, c.qw);
    for (j=0; j<4; j++) {
      exp = b.ui[j] + ~a.ui[j] + (c.ui[j] & 1);
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=0x%x", j, a.ui[j]);
	DEBUG(2, "b=0x%x c=0x%x d=0x%x exp=0x%x\n", b.ui[j], c.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_shl()
{
  int i, j;
  int count;
  unsigned int exp;

  DEBUG(1, "shl.............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_shl(a.qw, b.qw);
    for (j=0; j<4; j++) {
      count = (int)b.ui[j] & 0x3F;
      exp = (count > 31) ? 0 : (a.ui[j] << count);
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_shlh()
{
  int i, j;
  int count;
  unsigned short exp;

  DEBUG(1, "shlh............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_shlh(a.qw, b.qw);
    for (j=0; j<8; j++) {
      count = (int)b.us[j] & 0x1F;
      exp = (count > 15) ? 0 : (a.us[j] << count);
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.us[j], b.us[j], d.us[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_shlhi()
{
  int j;
  unsigned short exp;

  DEBUG(1, "shlhi...........");
  a.vui = rand_vui();
  d.qw = si_shlhi(a.qw, 0x0);
  for (j=0; j<8; j++) {
    exp = a.us[j];
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=0 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }

  a.vui = rand_vui();
  d.qw = si_shlhi(a.qw, 0xF);
  for (j=0; j<8; j++) {
    exp = a.us[j] << 15;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=15 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }

  a.vui = rand_vui();
  d.qw = si_shlhi(a.qw, 41);
  for (j=0; j<8; j++) {
    exp = a.us[j] << 9;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=9 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_shli()
{
  int j;
  unsigned int exp;

  DEBUG(1, "shli............");
  a.vui = rand_vui();
  d.qw = si_shli(a.qw, 0x0);
  for (j=0; j<4; j++) {
    exp = a.ui[j];
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=0 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }

  a.vui = rand_vui();
  d.qw = si_shli(a.qw, 0x1F);
  for (j=0; j<4; j++) {
    exp = a.ui[j] << 31;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=31 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }

  a.vui = rand_vui();
  d.qw = si_shli(a.qw, 71);
  for (j=0; j<4; j++) {
    exp = a.ui[j] << 7;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=7 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_shlqbi()
{
  int i, j;
  unsigned int cnt;
  unsigned int exp;

  DEBUG(1, "shlqbi..........");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    c.vui = rand_vui();
    d.qw = si_shlqbi(a.qw, c.qw);

    cnt = c.ui[0] & 7;
    for (j=0; j<4; j++) {
      exp = a.ui[j] << cnt;
      if (cnt && j<3) exp |= (a.ui[j+1] >> (32-cnt));

      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "count=0x%x d=0x%x exp=0x%x\n", c.ui[0], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_shlqbii()
{
  int j;
  unsigned int cnt;
  unsigned int exp;

  DEBUG(1, "shlqbii.........");
  a.vui = rand_vui();
  d.qw = si_shlqbii(a.qw, 27);
  cnt = 3;

  for (j=0; j<4; j++) {
    exp = a.ui[j] << cnt;
    if (cnt && (j<3)) exp |= (a.ui[j+1] >> (32-cnt));

    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "count=27 d=0x%x exp=0x%x\n", d.ui[j], exp);
      return (1);
    }
  }

  DEBUG(1, passed);
  return (0);
}


int test_si_shlqby()
{
  int i, j;
  unsigned int cnt;
  unsigned char exp;

  DEBUG(1, "shlqby..........");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    c.vui = rand_vui();
    d.qw = si_shlqby(a.qw, c.qw);
    cnt = c.ui[0] & 0x1F;
    for (j=0; j<16; j++) {
      exp = ((j+cnt) > 15) ? 0 : a.uc[j+cnt];
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "count=0x%x d=0x%x exp=0x%x\n", c.ui[0], d.uc[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_shlqbybi()
{
  int i, j;
  unsigned int cnt;
  unsigned char exp;

  DEBUG(1, "shlqbybi........");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    c.vui = rand_vui();
    d.qw = si_shlqbybi(a.qw, c.qw);
    cnt = (c.ui[0] >> 3) & 0x1F;
    for (j=0; j<16; j++) {
      exp = ((j+cnt) > 15) ? 0 : a.uc[j+cnt];
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "count=0x%x d=0x%x exp=0x%x\n", c.ui[0], d.uc[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_shlqbyi()
{
  int j;
  unsigned char exp;

  DEBUG(1, "shlqbyi.........");
  a.vui = rand_vui();
  d.qw = si_shlqbyi(a.qw, 35);

  for (j=0; j<16; j++) {
    exp = ((j+3) > 15) ? 0 : a.uc[j+3];
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(1, "count=35 d=0x%x exp=0x%x\n", d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_shufb()
{
  int i, j;
  unsigned int index;
  unsigned char exp;

  DEBUG(1, "shufb...........");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();
    d.qw = si_shufb(a.qw, b.qw, c.qw);
    for (j=0; j<16; j++) {
      index = c.uc[j];
      if (index & 0x80) {
	if (index & 0x40) {
	  exp = (index & 0x20) ? 0x80 : 0xFF;
	} else {
	  exp = 0;
	}
      } else {
	if (index & 0x10) {
	  exp = b.uc[index & 0xF];
	} else {
	  exp = a.uc[index & 0xF];
	}
      }
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.uc[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_stop()
{
  DEBUG(1, "stop............untested\n");
  return (0);
}


int test_si_stopd()
{
  DEBUG(1, "stopd...........untested\n");
  return (0);
}


int test_si_stqa()
{
#ifdef __SPU__
  int j;
  unsigned char save[16], *p;
  int result = 0;

  p = (unsigned char *)(23 & ~0xF);
  for (j=0; j<16; j++) save[j] = p[j];

  DEBUG(1, "stqa............");

  a.vui = rand_vui();
  si_stqa(a.qw, 23);
  for (j=0; j<16; j++) {
    if (a.uc[j] != p[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "imm=23 got=0x%x exp=0x%x \n", p[j], a.uc[j]);
      result = 1;
      break;
    }
  }
  if (!result) DEBUG(1, passed);

  for (j=0; j<16; j++) p[j] = save[j];
  return (result);

#else
  /* This intrinsic is only tested on the SPU since we can not 
   * guarantee the memory map when running in other environments.
   */
  DEBUG(1, "stqa............untested\n");
  return (0);
#endif
}


int test_si_stqd()
{
  int j;
  unsigned char exp;

  clear_buffer();
  DEBUG(1, "stqd............");
  a.vui = rand_vui();
  si_stqd(a.qw, si_from_ptr(buffer), 33);
  for (j=0; j<255; j++) {
    exp = 0;
    if ((j >= 32) && (j < 48)) {
      exp = a.uc[j-32];
    }
    if (exp != buffer[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "imm=33 buffer=0x%x exp=0x%x\n", buffer[j], exp);
      return (1);
    }
  }
#ifdef __SPU__
  a.vui = spu_splats((unsigned int)0);
#else /* !__SPU__ */
  a.vui = ((vector unsigned int) {0,0,0,0});
#endif /* __SPU__ */
  si_stqd(a.qw, si_from_ptr(buffer), 33);

  a.vui = rand_vui();
  si_stqd(a.qw, si_from_ptr(buffer-398), 511);
  for (j=0; j<255; j++) {
    exp = 0;
    if ((j >= 96) && (j < 112)) {
      exp = a.uc[j-96];
    }
    if (exp != buffer[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "imm=511 buffer=0x%x exp=0x%x\n", buffer[j], exp);
      return (1);
    }
  }

#ifdef __SPU__
  a.vui = spu_splats((unsigned int)0);
#else /* !__SPU__ */
  a.vui = ((vector unsigned int) {0,0,0,0});
#endif /* __SPU__ */
  si_stqd(a.qw, si_from_ptr(buffer-398), 511);

  a.vui = rand_vui();
  si_stqd(a.qw, si_from_ptr(buffer+605), -512);
  for (j=0; j<255; j++) {
    exp = 0;
    if ((j >= 80) && (j < 96)) {
      exp = a.uc[j-80];
    }
    if (exp != buffer[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "imm=-512 buffer=0x%x exp=0x%x\n", buffer[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_stqr()
{
#ifndef __SPU__
  int j;
  unsigned char exp;
#endif

  DEBUG(1, "stqr............");
#ifdef __SPU__
  /* This intrinsic is not tested on the SPU because of the lack of ability
   * to control relative addressing in an optimized compiled environment.
   */
  DEBUG(1, "untested\n");
#else
  clear_buffer();
  a.vui = rand_vui();
  si_stqr(a.qw, (unsigned int)(&buffer[37]));
  for (j=0; j<255; j++) {
    exp = 0;
    if ((j >= 32) && (j < 48)) {
      exp = a.uc[j-32];
    }
    if (exp != buffer[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "imm=buffer[37] buffer=0x%x exp=0x%x\n", buffer[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
#endif
  return (0);
}


int test_si_stqx()
{
  int j;
  unsigned char exp;

  clear_buffer();
  DEBUG(1, "stqx............");
  a.vui = rand_vui();
  b.ui[0] = (unsigned int)buffer;
  c.ui[0] = rand_local() & 0xFF;
  si_stqx(a.qw, b.qw, c.qw);
  for (j=0; j<16; j++) {
    exp = 0;
    if ((j >= (c.i[0] & 0xF0)) && (j < ((c.i[0] + 16) & 0xF0))) {
      exp = a.uc[j - (c.ui[0] & 0xF0)];
    }
    if (exp != buffer[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "c=0x%x buffer=0x%x exp=0x%x\n", c.ui[0], buffer[j], exp);
      return (1);
    }
  }
#ifdef __SPU__
  a.vui = spu_splats((unsigned int)0);
#else /* !__SPU__ */
  a.vui = ((vector unsigned int) {0,0,0,0});
#endif /* __SPU__ */
  si_stqx(a.qw, b.qw, c.qw);

  a.vui = rand_vui();
  b.ui[0] = rand_local() & 0xFF;
  c.ui[0] = (unsigned int)buffer;
  si_stqx(a.qw, b.qw, c.qw);
  for (j=0; j<16; j++) {
    exp = 0;
    if ((j >= (b.i[0] & 0xF0)) && (j < ((b.i[0] + 16) & 0xF0))) {
      exp = a.uc[j - (b.ui[0] & 0xF0)];
    }
    if (exp != buffer[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "b=0x%x buffer=0x%x exp=0x%x\n", b.ui[0], buffer[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_sumb()
{
  int i, j;
  int idx;
  unsigned short exp;

  DEBUG(1, "sumb............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_sumb(a.qw, b.qw);
    for (j=0; j<8; j++) {

      idx = 4 * (j>>1);
      if (j & 1) {
	exp = (unsigned short)a.uc[idx+0] + (unsigned short)a.uc[idx+1] +
	      (unsigned short)a.uc[idx+2] + (unsigned short)a.uc[idx+3];
      } else {
	exp = (unsigned short)b.uc[idx+0] + (unsigned short)b.uc[idx+1] +
	      (unsigned short)b.uc[idx+2] + (unsigned short)b.uc[idx+3];
      }
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.us[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}


int test_si_sync()
{
  DEBUG(1, "sync............");
  si_sync();
  DEBUG(1, passed);
  return (0);
}



int test_si_syncc()
{
  DEBUG(1, "syncc...........");
  si_syncc();
  DEBUG(1, passed);
  return (0);
}


int test_si_to_char()
{
  char d;

  DEBUG(1, "to_char.........");
  a.vui = rand_vui();
  d = si_to_char(a.qw);
  if (d != (char)a.c[3]) {
    DEBUG(1, failed);
    DEBUG(2, "\t\td=0x%x exp=0x%x\n", d, a.c[3]);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_to_double()
{
  double d;

  DEBUG(1, "to_double.......");
  a.vd = rand_vd(-1.0, 1.0);
  d = si_to_double(a.qw);
  if (d != a.d[0]) {
    DEBUG(1, failed);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_to_float()
{
  float d;

  DEBUG(1, "to_float........");
  a.vf = rand_vf(-1.0f, 1.0f);
  d = si_to_float(a.qw);
  if (d != a.f[0]) {
    DEBUG(1, failed);
    DEBUG(2, "\t\td=%f exp=%f\n", d, a.f[0]);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_to_int()
{
  int d;

  DEBUG(1, "to_int..........");
  a.vui = rand_vui();
  d = si_to_int(a.qw);
  if (d != (int)a.i[0]) {
    DEBUG(1, failed);
    DEBUG(2, "\t\td=0x%x exp=0x%x\n", d, a.i[0]);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_to_ptr()
{
  void * d;

  DEBUG(1, "to_ptr..........");
  a.vui = rand_vui();
  d = (void *)si_to_ptr(a.qw);
  if (d != a.p[0]) {
    DEBUG(1, failed);
    DEBUG(2, "\t\td=0x%x exp=0x%x\n", (unsigned int)d, (unsigned int)a.p[0]);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_to_llong()
{
  long long d;

  DEBUG(1, "to_llong........");
  a.vui = rand_vui();
  d = si_to_llong(a.qw);
  if (d != a.ll[0]) {
    DEBUG(1, failed);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_to_short()
{
  short d;

  DEBUG(1, "to_short........");
  a.vui = rand_vui();
  d = si_to_short(a.qw);
  if (d != (short)a.s[1]) {
    DEBUG(1, failed);
    DEBUG(2, "\t\td=0x%x exp=0x%x\n", d, a.s[1]);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_to_uchar()
{
  unsigned char d;

  DEBUG(1, "to_uchar........");
  a.vui = rand_vui();
  d = si_to_uchar(a.qw);
  if (d != (char)a.uc[3]) {
    DEBUG(1, failed);
    DEBUG(2, "\t\td=0x%x exp=0x%x\n", d, a.uc[3]);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_to_uint()
{
  unsigned int d;

  DEBUG(1, "to_uint.........");
  a.vui = rand_vui();
  d = si_to_uint(a.qw);
  if (d != a.ui[0]) {
    DEBUG(1, failed);
    DEBUG(2, "\t\td=0x%x exp=0x%x\n", d, a.ui[0]);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_to_ullong()
{
  unsigned long long d;

  DEBUG(1, "to_ullong.......");
  a.vui = rand_vui();
  d = si_to_ullong(a.qw);
  if (d != a.ull[0]) {
    DEBUG(1, failed);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}



int test_si_to_ushort()
{
  unsigned short d;

  DEBUG(1, "to_ushort.......");
  a.vui = rand_vui();
  d = si_to_ushort(a.qw);
  if (d != a.us[1]) {
    DEBUG(1, failed);
    DEBUG(2, "\t\td=0x%x exp=0x%x\n", d, a.us[1]);
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_wrch()
{

  DEBUG(1, "wrch............");
  a.vui = rand_vui();
  si_wrch(SPU_WrEventMask, a.qw);
  DEBUG(1, passed);
  return (0);
}


int test_si_xor()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "xor.............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.qw = si_xor(a.qw, b.qw);
    for (j=0; j<4; j++) {
      exp = a.ui[j] ^ b.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_xorbi()
{
  int j;
  unsigned char exp;

  DEBUG(1, "xorbi...........");
  a.vui = rand_vui();
  d.qw = si_xorbi(a.qw, 267);
  for (j=0; j<16; j++) {
    exp = a.uc[j] ^ 267;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=267 d=0x%x exp=0x%x\n", a.uc[j], d.uc[j], exp);
      return (1);
    }
  }

  a.vui = rand_vui();
  d.qw = si_xorbi(a.qw, -315);
  for (j=0; j<16; j++) {
    exp = a.uc[j] ^ -315;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-315 d=0x%x exp=0x%x\n", a.uc[j], d.uc[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_xorhi()
{
  int j;
  unsigned short exp;

  DEBUG(1, "xorhi...........");
  a.vui = rand_vui();
  d.qw = si_xorhi(a.qw, 511);
  for (j=0; j<8; j++) {
    exp = a.us[j] ^ 511;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=511 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_xori()
{
  int j;
  unsigned int exp;

  DEBUG(1, "xori............");
  a.vui = rand_vui();
  d.qw = si_xori(a.qw, -512);
  for (j=0; j<4; j++) {
    exp = a.ui[j] ^ -512;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "a=0x%x b=-511 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_si_xsbh()
{
  int i, j;
  signed short exp;

  DEBUG(1, "xsbh............");

  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.qw = si_xsbh(a.qw);
    for (j=0; j<8; j++) {
      exp = a.c[2*j+1];
      if (exp != d.s[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x d=0x%x exp=0x%x\n", a.c[2*j+1], d.s[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_xshw()
{
  int i, j;
  signed int exp;

  DEBUG(1, "xshw............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.qw = si_xshw(a.qw);
    for (j=0; j<4; j++) {
      exp = a.s[2*j+1];
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x d=0x%x exp=0x%x\n", a.s[2*j+1], d.i[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_si_xswd()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "xswd............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.qw = si_xswd(a.qw);
    for (j=0; j<4; j++) {
      if (j & 1) {
	exp = a.ui[j];
      } else {
	exp = (a.ui[j+1] & 0x80000000) ? 0xFFFFFFFF : 0;
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x d=0x%x exp=0x%x\n", a.ui[j|1], d.ui[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


DECLARE_MAIN()
{
  DECLARE_TEST_VARS();
#ifdef __SPU__
#ifndef STANDALONE
  unsigned int *ptr = NULL;
#endif
#endif
  SCALE_SAMPLES(samples);

  INVOKE_TEST(test_si_a);
  INVOKE_TEST(test_si_absdb);
  INVOKE_TEST(test_si_addx);
  INVOKE_TEST(test_si_ah);
  INVOKE_TEST(test_si_ahi);
  INVOKE_TEST(test_si_ai);
  INVOKE_TEST(test_si_and);
  INVOKE_TEST(test_si_andbi);
  INVOKE_TEST(test_si_andc);
  INVOKE_TEST(test_si_andhi);
  INVOKE_TEST(test_si_andi);
  INVOKE_TEST(test_si_avgb);
#ifdef __SPU__
#ifndef STANDALONE
  /* Install a no-op interrupt handler that disables interrupts 
   */
  *ptr = 0x35480000;	/* iretd */
  INVOKE_TEST(test_si_bisled);
  INVOKE_TEST(test_si_bisledd);
  INVOKE_TEST(test_si_bislede);
#endif /* !STANDALONE */
#endif /* __SPU__ */
  INVOKE_TEST(test_si_bg);
  INVOKE_TEST(test_si_bgx);
  INVOKE_TEST(test_si_cbd);
  INVOKE_TEST(test_si_cbx);
  INVOKE_TEST(test_si_cdd);
  INVOKE_TEST(test_si_cdx);
  INVOKE_TEST(test_si_ceq);
  INVOKE_TEST(test_si_ceqb);
  INVOKE_TEST(test_si_ceqbi);
  INVOKE_TEST(test_si_ceqh);
  INVOKE_TEST(test_si_ceqhi);
  INVOKE_TEST(test_si_ceqi);
  INVOKE_TEST(test_si_cflts);
  INVOKE_TEST(test_si_cfltu);
  INVOKE_TEST(test_si_cg);
  INVOKE_TEST(test_si_cgt);
  INVOKE_TEST(test_si_cgtb);
  INVOKE_TEST(test_si_cgtbi);
  INVOKE_TEST(test_si_cgth);
  INVOKE_TEST(test_si_cgthi);
  INVOKE_TEST(test_si_cgti);
  INVOKE_TEST(test_si_cgx);
  INVOKE_TEST(test_si_chd);
  INVOKE_TEST(test_si_chx);
  INVOKE_TEST(test_si_clgt);
  INVOKE_TEST(test_si_clgtb);
  INVOKE_TEST(test_si_clgtbi);
  INVOKE_TEST(test_si_clgth);
  INVOKE_TEST(test_si_clgthi);
  INVOKE_TEST(test_si_clgti);
  INVOKE_TEST(test_si_clz);
  INVOKE_TEST(test_si_cntb);
  INVOKE_TEST(test_si_csflt);
  INVOKE_TEST(test_si_cuflt);
  INVOKE_TEST(test_si_cwd);
  INVOKE_TEST(test_si_cwx);
  INVOKE_TEST(test_si_dfa);
#if defined(__SPU_EDP__) || defined(__PPU__)
  INVOKE_TEST(test_si_dfceq);
  INVOKE_TEST(test_si_dfcgt);
  INVOKE_TEST(test_si_dfcmeq);
  INVOKE_TEST(test_si_dfcmgt);
#endif /* __SPU_EDP__ || __PPU__ */
  INVOKE_TEST(test_si_dfm);
  INVOKE_TEST(test_si_dfma);
  INVOKE_TEST(test_si_dfms);
  INVOKE_TEST(test_si_dfnma);
  INVOKE_TEST(test_si_dfnms);
  INVOKE_TEST(test_si_dfs);
#if defined(__SPU_EDP__) || defined(__PPU__)
  INVOKE_TEST(test_si_dftsv);
#endif /* __SPU_EDP__ || __PPU__ */
  INVOKE_TEST(test_si_dsync);
  INVOKE_TEST(test_si_eqv);
  INVOKE_TEST(test_si_fa);
  INVOKE_TEST(test_si_fceq);
  INVOKE_TEST(test_si_fcgt);
  INVOKE_TEST(test_si_fcmeq);
  INVOKE_TEST(test_si_fcmgt);
  INVOKE_TEST(test_si_fesd);
  INVOKE_TEST(test_si_fm);
  INVOKE_TEST(test_si_fma);
  INVOKE_TEST(test_si_fms);
  INVOKE_TEST(test_si_fnms);
  INVOKE_TEST(test_si_frds);
  INVOKE_TEST(test_si_frest);		/* also tests si_fi */
  INVOKE_TEST(test_si_from_char);
  INVOKE_TEST(test_si_from_double);
  INVOKE_TEST(test_si_from_float);
  INVOKE_TEST(test_si_from_int);
  INVOKE_TEST(test_si_from_llong);
  INVOKE_TEST(test_si_from_ptr);
  INVOKE_TEST(test_si_from_short);
  INVOKE_TEST(test_si_from_uchar);
  INVOKE_TEST(test_si_from_uint);
  INVOKE_TEST(test_si_from_ullong);
  INVOKE_TEST(test_si_from_ushort);
  INVOKE_TEST(test_si_frsqest);		/* also tests si_fi */
  INVOKE_TEST(test_si_fs);
  INVOKE_TEST(test_si_fscrrd);		/* also tests si_fscrwr */
  INVOKE_TEST(test_si_fsm);
  INVOKE_TEST(test_si_fsmb);
  INVOKE_TEST(test_si_fsmbi);
  INVOKE_TEST(test_si_fsmh);
  INVOKE_TEST(test_si_gb);
  INVOKE_TEST(test_si_gbb);
  INVOKE_TEST(test_si_gbh);
  INVOKE_TEST(test_si_heq);
  INVOKE_TEST(test_si_heqi);
  INVOKE_TEST(test_si_hgt);
  INVOKE_TEST(test_si_hgti);
  INVOKE_TEST(test_si_hlgt);
  INVOKE_TEST(test_si_hlgti);
  INVOKE_TEST(test_si_il);
  INVOKE_TEST(test_si_ila);
  INVOKE_TEST(test_si_ilh);
  INVOKE_TEST(test_si_ilhu);
  INVOKE_TEST(test_si_iohl);
  INVOKE_TEST(test_si_lnop);
  INVOKE_TEST(test_si_lqa);
  INVOKE_TEST(test_si_lqd);
  INVOKE_TEST(test_si_lqr);
  INVOKE_TEST(test_si_lqx);
  INVOKE_TEST(test_si_mfspr);	/* also tests si_mtspr */
  INVOKE_TEST(test_si_mpy);
  INVOKE_TEST(test_si_mpya);
  INVOKE_TEST(test_si_mpyh);
  INVOKE_TEST(test_si_mpyhh);
  INVOKE_TEST(test_si_mpyhha);
  INVOKE_TEST(test_si_mpyhhau);
  INVOKE_TEST(test_si_mpyhhu);
  INVOKE_TEST(test_si_mpyi);
  INVOKE_TEST(test_si_mpys);
  INVOKE_TEST(test_si_mpyu);
  INVOKE_TEST(test_si_mpyui);
  INVOKE_TEST(test_si_nand);
  INVOKE_TEST(test_si_nop);
  INVOKE_TEST(test_si_nor);
  INVOKE_TEST(test_si_or);
  INVOKE_TEST(test_si_orbi);
  INVOKE_TEST(test_si_orc);
  INVOKE_TEST(test_si_orhi);
  INVOKE_TEST(test_si_ori);
  INVOKE_TEST(test_si_orx);
  INVOKE_TEST(test_si_rchcnt);
  INVOKE_TEST(test_si_rdch);
  INVOKE_TEST(test_si_rot);
  INVOKE_TEST(test_si_roth);
  INVOKE_TEST(test_si_rothi);
  INVOKE_TEST(test_si_rothm);
  INVOKE_TEST(test_si_rothmi);
  INVOKE_TEST(test_si_roti);
  INVOKE_TEST(test_si_rotm);
  INVOKE_TEST(test_si_rotma);
  INVOKE_TEST(test_si_rotmah);
  INVOKE_TEST(test_si_rotmahi);
  INVOKE_TEST(test_si_rotmai);
  INVOKE_TEST(test_si_rotmi);
  INVOKE_TEST(test_si_rotqbi);
  INVOKE_TEST(test_si_rotqbii);
  INVOKE_TEST(test_si_rotqby);
  INVOKE_TEST(test_si_rotqbybi);
  INVOKE_TEST(test_si_rotqbyi);
  INVOKE_TEST(test_si_rotqmbi);
  INVOKE_TEST(test_si_rotqmbii);
  INVOKE_TEST(test_si_rotqmby);
  INVOKE_TEST(test_si_rotqmbybi);
  INVOKE_TEST(test_si_rotqmbyi);
  INVOKE_TEST(test_si_selb);
  INVOKE_TEST(test_si_sf);
  INVOKE_TEST(test_si_sfh);
  INVOKE_TEST(test_si_sfhi);
  INVOKE_TEST(test_si_sfi);
  INVOKE_TEST(test_si_sfx);
  INVOKE_TEST(test_si_shl);
  INVOKE_TEST(test_si_shlh);
  INVOKE_TEST(test_si_shlhi);
  INVOKE_TEST(test_si_shli);
  INVOKE_TEST(test_si_shlqbi);
  INVOKE_TEST(test_si_shlqbii);
  INVOKE_TEST(test_si_shlqby);
  INVOKE_TEST(test_si_shlqbybi);
  INVOKE_TEST(test_si_shlqbyi);
  INVOKE_TEST(test_si_shufb);
  INVOKE_TEST(test_si_stop);
  INVOKE_TEST(test_si_stopd);
  INVOKE_TEST(test_si_stqa);
  INVOKE_TEST(test_si_stqd);
  INVOKE_TEST(test_si_stqr);
  INVOKE_TEST(test_si_stqx);
  INVOKE_TEST(test_si_sumb);
  INVOKE_TEST(test_si_sync);
  INVOKE_TEST(test_si_syncc);
  INVOKE_TEST(test_si_to_char);
  INVOKE_TEST(test_si_to_double);
  INVOKE_TEST(test_si_to_float);
  INVOKE_TEST(test_si_to_int);
  INVOKE_TEST(test_si_to_llong);
  INVOKE_TEST(test_si_to_ptr);
  INVOKE_TEST(test_si_to_short);
  INVOKE_TEST(test_si_to_uchar);
  INVOKE_TEST(test_si_to_uint);
  INVOKE_TEST(test_si_to_ullong);
  INVOKE_TEST(test_si_to_ushort);
  INVOKE_TEST(test_si_wrch);
  INVOKE_TEST(test_si_xor);
  INVOKE_TEST(test_si_xorbi);
  INVOKE_TEST(test_si_xorhi);
  INVOKE_TEST(test_si_xori);
  INVOKE_TEST(test_si_xsbh);
  INVOKE_TEST(test_si_xshw);
  INVOKE_TEST(test_si_xswd);

  DISPLAY_RESULTS();
}
