/* --------------------------------------------------------------  */
/* (C)Copyright 2001,2007,                                         */
/* International Business Machines Corporation,                    */
/* Sony Computer Entertainment, Incorporated,                      */
/* Toshiba Corporation,                                            */
/*                                                                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#include <stdio.h>
#include <vec_types.h>
#ifdef __SPU__
#include <vmx2spu.h>
#else /* not __SPU__ */
#include <altivec.h>
#endif
#include "libtest.h"

typedef union {
  vec_uchar16	vuc;
  vec_char16	vc;
  vec_bchar16	vbc;
  vec_ushort8	vus;
  vec_bshort8	vbs;
  vec_short8	vs;
  vec_pixel8	vp;
  vec_uint4	vui;
  vec_int4	vi;
  vec_bint4	vbi;
  vec_float4	vf;
  unsigned char	uc[16];
  unsigned char	bc[16];
  signed char	c[16];
  unsigned short us[8];
  signed short	s[8];
  unsigned short bs[8];
  signed short	p[8];
  unsigned int	ui[4];
  unsigned int	bi[4];
  signed int	i[4];
  float		f[4];
} v128;
  
static char *passed = "passed\n";
static char *failed = "FAILED\n";
static v128 a, b;

static int samples = 200;

#define SWAP(_a, _b, _type)	{ _type _tmp; _tmp = _a, _a = _b; _b = _tmp; }

unsigned int seed = 0;


unsigned int rand16()
{
  seed = (seed * 31421 + 31421) & 0xFFFF;
  return (seed);
}

unsigned int rand32()
{
  return ((rand16() << 16) | rand16());
}

float randf(float min, float max) 
{
  union {
    unsigned int ui;
    float f;
  } val;

  val.ui = (rand32() & 0x7FFFFF) | 0x3F800000;
  val.f -= 1.0f;
  val.f *= (max - min);
  val.f += min;

  return (val.f);
}


vec_uint4 rand_vui()
{
  static v128 val;

  val.ui[0] = rand32();
  val.ui[1] = rand32();
  val.ui[2] = rand32();
  val.ui[3] = rand32();

  return (val.vui);
}


vec_float4 rand_vf(float min, float max)
{
  int i;
  static v128 val;

  for (i=0; i<4; i++) val.f[i] = randf(min, max);
  return (val.vf);
}



#define VERIFY_EQUAL(_a, _b)						\
  if (_a != _b) {							\
    DEBUG(1, failed);							\
    DEBUG(2,"\t\tequivalence not verified at line %d\n", __LINE__);	\
    return (1);								\
  }
  					


int test_vec_all_eq()
{
  int i, j;
  int idx;
  int exp, d, e;
  int count=0;

  DEBUG(1, "all_eq..........");
  exp = 1;

  /* byte */
  
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      a.uc[idx] = b.uc[idx];
    }
    d = vec_all_eq(a.vuc, b.vuc);
    for (j=0, exp=1; j<16; j++) if (a.uc[j] != b.uc[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tbyte: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_eq(a.vuc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_all_eq(a.vc, b.vc);
    VERIFY_EQUAL(d, e);
    e = vec_all_eq(a.vc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_all_eq(a.vbc, b.vuc);
    VERIFY_EQUAL(d, e);
    e = vec_all_eq(a.vbc, b.vc);
    VERIFY_EQUAL(d, e);
    e = vec_all_eq(a.vbc, b.vbc);
    VERIFY_EQUAL(d, e);
  }
  
  /* halfword */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      a.us[idx] = b.us[idx];
    }
    d = vec_all_eq(a.vus, b.vus);
    for (j=0, exp=1; j<8; j++) if (a.us[j] != b.us[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\thalfword: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_eq(a.vus, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_all_eq(a.vs, b.vs);
    VERIFY_EQUAL(d, e);
    e = vec_all_eq(a.vs, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_all_eq(a.vbs, b.vus);
    VERIFY_EQUAL(d, e);
    e = vec_all_eq(a.vbs, b.vs);
    VERIFY_EQUAL(d, e);
    e = vec_all_eq(a.vbs, b.vbs);
    VERIFY_EQUAL(d, e);
  }

  
  /* word */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      a.ui[idx] = b.ui[idx];
    }
    d = vec_all_eq(a.vui, b.vui);
    for (j=0, exp=1; j<4; j++) if (a.ui[j] != b.ui[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tword: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_eq(a.vui, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_all_eq(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
    e = vec_all_eq(a.vi, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_all_eq(a.vbi, b.vui);
    VERIFY_EQUAL(d, e);
    e = vec_all_eq(a.vbi, b.vi);
    VERIFY_EQUAL(d, e);
    e = vec_all_eq(a.vbi, b.vbi);
    VERIFY_EQUAL(d, e);
  }
  
  /* float */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      a.f[idx] = b.f[idx];
    }
    d = vec_all_eq(a.vf, b.vf);
    for (j=0, exp=1; j<4; j++) if (a.f[j] != b.f[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tfloat: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_all_ge()
{
  int i, j;
  int idx;
  int exp, d, e;
  int count=0;
  
  DEBUG(1, "all_ge..........");
  exp = 1;
  
  /* uchar */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      if (a.uc[idx] < b.uc[idx]) SWAP(a.uc[idx], b.uc[idx], unsigned char);
    }
    d = vec_all_ge(a.vuc, b.vuc);
    for (j=0, exp=1; j<16; j++) if (a.uc[j] < b.uc[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_ge(a.vuc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_all_ge(a.vbc, b.vuc);
    VERIFY_EQUAL(d, e);
  }
  
  /* char */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      if (a.c[idx] < b.c[idx]) SWAP(a.c[idx], b.c[idx], signed char);
    }
    d = vec_all_ge(a.vc, b.vc);
    for (j=0, exp=1; j<16; j++) if (a.c[j] < b.c[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tchar: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_ge(a.vc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_all_ge(a.vbc, b.vc);
    VERIFY_EQUAL(d, e);
  }

  
  /* ushort */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      if (a.us[idx] < b.us[idx]) SWAP(a.us[idx], b.us[idx], unsigned short);
    }
    d = vec_all_ge(a.vus, b.vus);
    for (j=0, exp=1; j<8; j++) if (a.us[j] < b.us[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_ge(a.vus, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_all_ge(a.vbs, b.vus);
    VERIFY_EQUAL(d, e);
  }
  
  /* short */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      if (a.s[idx] < b.s[idx]) SWAP(a.s[idx], b.s[idx], signed short);
    }
    d = vec_all_ge(a.vs, b.vs);
    for (j=0, exp=1; j<8; j++) if (a.s[j] < b.s[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tshort: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_ge(a.vs, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_all_ge(a.vbs, b.vs);
    VERIFY_EQUAL(d, e);
  }

  /* uint */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (a.ui[idx] < b.ui[idx]) SWAP(a.ui[idx], b.ui[idx], unsigned int);
    }
    d = vec_all_ge(a.vui, b.vui);
    for (j=0, exp=1; j<4; j++) if (a.ui[j] < b.ui[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_ge(a.vui, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_all_ge(a.vbi, b.vui);
    VERIFY_EQUAL(d, e);
  }
  
  /* int */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (a.i[idx] < b.i[idx]) SWAP(a.i[idx], b.i[idx], signed int);
    }
    d = vec_all_ge(a.vi, b.vi);
    for (j=0, exp=1; j<4; j++) if (a.i[j] < b.i[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tint: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_ge(a.vi, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_all_ge(a.vbi, b.vi);
    VERIFY_EQUAL(d, e);
  }
  
  /* float */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      if (a.f[idx] < b.f[idx]) SWAP(a.f[idx], b.f[idx], float);
    }
    d = vec_all_ge(a.vf, b.vf);
    for (j=0, exp=1; j<4; j++) if (a.f[j] < b.f[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tfloat: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_all_gt()
{
  int i, j;
  int idx;
  int exp, d, e;
  int count=0;
  
  DEBUG(1, "all_gt..........");
  exp = 1;
  
  /* uchar */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      if (a.uc[idx] <= b.uc[idx]) SWAP(a.uc[idx], b.uc[idx], unsigned char);
    }
    d = vec_all_gt(a.vuc, b.vuc);
    for (j=0, exp=1; j<16; j++) if (a.uc[j] <= b.uc[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_gt(a.vuc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_all_gt(a.vbc, b.vuc);
    VERIFY_EQUAL(d, e);
  }
  
  /* char */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      if (a.c[idx] <= b.c[idx]) SWAP(a.c[idx], b.c[idx], signed char);
    }
    d = vec_all_gt(a.vc, b.vc);
    for (j=0, exp=1; j<16; j++) if (a.c[j] <= b.c[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tchar: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_gt(a.vc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_all_gt(a.vbc, b.vc);
    VERIFY_EQUAL(d, e);
  }

  
  /* ushort */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      if (a.us[idx] <= b.us[idx]) SWAP(a.us[idx], b.us[idx], unsigned short);
    }
    d = vec_all_gt(a.vus, b.vus);
    for (j=0, exp=1; j<8; j++) if (a.us[j] <= b.us[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_gt(a.vus, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_all_gt(a.vbs, b.vus);
    VERIFY_EQUAL(d, e);
  }
  
  /* short */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      if (a.s[idx] <= b.s[idx]) SWAP(a.s[idx], b.s[idx], signed short);
    }
    d = vec_all_gt(a.vs, b.vs);
    for (j=0, exp=1; j<8; j++) if (a.s[j] <= b.s[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tshort: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_gt(a.vs, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_all_gt(a.vbs, b.vs);
    VERIFY_EQUAL(d, e);
  }

  /* uint */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (a.ui[idx] <= b.ui[idx]) SWAP(a.ui[idx], b.ui[idx], unsigned int);
    }
    d = vec_all_gt(a.vui, b.vui);
    for (j=0, exp=1; j<4; j++) if (a.ui[j] <= b.ui[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_gt(a.vui, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_all_gt(a.vbi, b.vui);
    VERIFY_EQUAL(d, e);
  }
  
  /* int */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (a.i[idx] <= b.i[idx]) SWAP(a.i[idx], b.i[idx], signed int);
    }
    d = vec_all_gt(a.vi, b.vi);
    for (j=0, exp=1; j<4; j++) if (a.i[j] <= b.i[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tint: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_gt(a.vi, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_all_gt(a.vbi, b.vi);
    VERIFY_EQUAL(d, e);
  }
  
  /* float */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      if (a.f[idx] <= b.f[idx]) SWAP(a.f[idx], b.f[idx], float);
    }
    d = vec_all_gt(a.vf, b.vf);
    for (j=0, exp=1; j<4; j++) if (a.f[j] <= b.f[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tfloat: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_all_in()
{
  int i, j;
  int idx;
  int exp, d;
  int count=0;
  
  DEBUG(1, "all_in..........");
  exp = 1;
  
  /* float */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      if (b.f[idx] < 0.0f)           b.f[idx]  = -b.f[idx];
      else if (a.f[idx] >  b.f[idx]) a.f[idx] -= 0.5f;
      else if (a.f[idx] < -b.f[idx]) a.f[idx]  = 0.0f;
    }
    d = vec_all_in(a.vf, b.vf);
    for (j=0, exp=1; j<4; j++) 
      if ((a.f[j] > b.f[j]) || (a.f[j] < -b.f[j])) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\td=%d exp=%d\n", d, exp);
      DEBUG(2, "\t\ta=%f %f %f %f\n", a.f[0], a.f[1], a.f[2], a.f[3]);
      DEBUG(2, "\t\tb=%f %f %f %f\n", b.f[0], b.f[1], b.f[2], b.f[3]);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_all_le()
{
  int i, j;
  int idx;
  int exp, d, e;
  int count=0;
  
  DEBUG(1, "all_le..........");
  exp = 1;
  
  /* uchar */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      if (a.uc[idx] > b.uc[idx]) SWAP(a.uc[idx], b.uc[idx], unsigned char);
    }
    d = vec_all_le(a.vuc, b.vuc);
    for (j=0, exp=1; j<16; j++) if (a.uc[j] > b.uc[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_le(a.vuc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_all_le(a.vbc, b.vuc);
    VERIFY_EQUAL(d, e);
  }
  
  /* char */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      if (a.c[idx] > b.c[idx]) SWAP(a.c[idx], b.c[idx], signed char);
    }
    d = vec_all_le(a.vc, b.vc);
    for (j=0, exp=1; j<16; j++) if (a.c[j] > b.c[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tchar: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_le(a.vc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_all_le(a.vbc, b.vc);
    VERIFY_EQUAL(d, e);
  }

  
  /* ushort */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      if (a.us[idx] > b.us[idx]) SWAP(a.us[idx], b.us[idx], unsigned short);
    }
    d = vec_all_le(a.vus, b.vus);
    for (j=0, exp=1; j<8; j++) if (a.us[j] > b.us[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_le(a.vus, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_all_le(a.vbs, b.vus);
    VERIFY_EQUAL(d, e);
  }
  
  /* short */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      if (a.s[idx] > b.s[idx]) SWAP(a.s[idx], b.s[idx], signed short);
    }
    d = vec_all_le(a.vs, b.vs);
    for (j=0, exp=1; j<8; j++) if (a.s[j] > b.s[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tshort: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_le(a.vs, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_all_le(a.vbs, b.vs);
    VERIFY_EQUAL(d, e);
  }

  /* uint */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (a.ui[idx] > b.ui[idx]) SWAP(a.ui[idx], b.ui[idx], unsigned int);
    }
    d = vec_all_le(a.vui, b.vui);
    for (j=0, exp=1; j<4; j++) if (a.ui[j] > b.ui[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_le(a.vui, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_all_le(a.vbi, b.vui);
    VERIFY_EQUAL(d, e);
  }
  
  /* int */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (a.i[idx] > b.i[idx]) SWAP(a.i[idx], b.i[idx], signed int);
    }
    d = vec_all_le(a.vi, b.vi);
    for (j=0, exp=1; j<4; j++) if (a.i[j] > b.i[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tint: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_le(a.vi, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_all_le(a.vbi, b.vi);
    VERIFY_EQUAL(d, e);
  }
  
  /* float */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      if (a.f[idx] > b.f[idx]) SWAP(a.f[idx], b.f[idx], float);
    }
    d = vec_all_le(a.vf, b.vf);
    for (j=0, exp=1; j<4; j++) if (a.f[j] > b.f[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tfloat: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}

int test_vec_all_lt()
{
  int i, j;
  int idx;
  int exp, d, e;
  int count=0;
  
  DEBUG(1, "all_lt..........");
  exp = 1;
  
  /* uchar */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      if (a.uc[idx] >= b.uc[idx]) SWAP(a.uc[idx], b.uc[idx], unsigned char);
    }
    d = vec_all_lt(a.vuc, b.vuc);
    for (j=0, exp=1; j<16; j++) if (a.uc[j] >= b.uc[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_lt(a.vuc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_all_lt(a.vbc, b.vuc);
    VERIFY_EQUAL(d, e);
  }
  
  /* char */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      if (a.c[idx] >= b.c[idx]) SWAP(a.c[idx], b.c[idx], signed char);
    }
    d = vec_all_lt(a.vc, b.vc);
    for (j=0, exp=1; j<16; j++) if (a.c[j] >= b.c[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tchar: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_lt(a.vc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_all_lt(a.vbc, b.vc);
    VERIFY_EQUAL(d, e);
  }

  /* ushort */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      if (a.us[idx] >= b.us[idx]) SWAP(a.us[idx], b.us[idx], unsigned short);
    }
    d = vec_all_lt(a.vus, b.vus);
    for (j=0, exp=1; j<8; j++) if (a.us[j] >= b.us[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_lt(a.vus, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_all_lt(a.vbs, b.vus);
    VERIFY_EQUAL(d, e);
  }
  
  /* short */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      if (a.s[idx] >= b.s[idx]) SWAP(a.s[idx], b.s[idx], signed short);
    }
    d = vec_all_lt(a.vs, b.vs);
    for (j=0, exp=1; j<8; j++) if (a.s[j] >= b.s[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tshort: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_lt(a.vs, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_all_lt(a.vbs, b.vs);
    VERIFY_EQUAL(d, e);
  }

  /* uint */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (a.ui[idx] >= b.ui[idx]) SWAP(a.ui[idx], b.ui[idx], unsigned int);
    }
    d = vec_all_lt(a.vui, b.vui);
    for (j=0, exp=1; j<4; j++) if (a.ui[j] >= b.ui[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_lt(a.vui, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_all_lt(a.vbi, b.vui);
    VERIFY_EQUAL(d, e);
  }
  
  /* int */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (a.i[idx] >= b.i[idx]) SWAP(a.i[idx], b.i[idx], signed int);
    }
    d = vec_all_lt(a.vi, b.vi);
    for (j=0, exp=1; j<4; j++) if (a.i[j] >= b.i[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tint: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_lt(a.vi, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_all_lt(a.vbi, b.vi);
    VERIFY_EQUAL(d, e);
  }
  
  /* float */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      if (a.f[idx] >= b.f[idx]) SWAP(a.f[idx], b.f[idx], float);
    }
    d = vec_all_lt(a.vf, b.vf);
    for (j=0, exp=1; j<4; j++) if (a.f[j] >= b.f[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tfloat: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_all_nan()
{
  int i, j;
  int idx;
  int exp, d;
  int count=0;
  
  DEBUG(1, "all_nan.........");
  exp = 1;
  
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (i & 1) a.ui[idx]  = (a.ui[idx] & ~0x007FFFFF) | (rand32() & 0x007FFFFF);
      else       a.ui[idx] |= 0x7F800000;
    }
    d = vec_all_nan(a.vf);
    for (j=0, exp=1; j<4; j++) 
      if (((a.ui[j] & 0x7F800000) != 0x7F800000) ||
	  ((a.ui[j] & 0x007FFFFF) == 0)) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\td=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}



int test_vec_all_ne()
{
  int i, j;
  int idx;
  int exp, d, e;
  int count=0;

  DEBUG(1, "all_ne..........");
  exp = 1;

  /* byte */
  
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      a.uc[idx] = (unsigned char)rand16();
    }
    d = vec_all_ne(a.vuc, b.vuc);
    for (j=0, exp=1; j<16; j++) if (a.uc[j] == b.uc[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tbyte: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_ne(a.vuc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_all_ne(a.vc, b.vc);
    VERIFY_EQUAL(d, e);
    e = vec_all_ne(a.vc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_all_ne(a.vbc, b.vuc);
    VERIFY_EQUAL(d, e);
    e = vec_all_ne(a.vbc, b.vc);
    VERIFY_EQUAL(d, e);
    e = vec_all_ne(a.vbc, b.vbc);
    VERIFY_EQUAL(d, e);
  }
  
  /* halfword */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      a.us[idx] = (unsigned short)rand16();
    }
    d = vec_all_ne(a.vus, b.vus);
    for (j=0, exp=1; j<8; j++) if (a.us[j] == b.us[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\thalfword: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_ne(a.vus, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_all_ne(a.vs, b.vs);
    VERIFY_EQUAL(d, e);
    e = vec_all_ne(a.vs, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_all_ne(a.vbs, b.vus);
    VERIFY_EQUAL(d, e);
    e = vec_all_ne(a.vbs, b.vs);
    VERIFY_EQUAL(d, e);
    e = vec_all_ne(a.vbs, b.vbs);
    VERIFY_EQUAL(d, e);
  }

  
  /* word */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      a.ui[idx] = (unsigned int)rand32();
    }
    d = vec_all_ne(a.vui, b.vui);
    for (j=0, exp=1; j<4; j++) if (a.ui[j] == b.ui[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tword: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_all_ne(a.vui, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_all_ne(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
    e = vec_all_ne(a.vi, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_all_ne(a.vbi, b.vui);
    VERIFY_EQUAL(d, e);
    e = vec_all_ne(a.vbi, b.vi);
    VERIFY_EQUAL(d, e);
    e = vec_all_ne(a.vbi, b.vbi);
    VERIFY_EQUAL(d, e);
  }
  
  /* float */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      a.f[idx] = randf(-2.0, 2.0);
    }
    d = vec_all_ne(a.vf, b.vf);
    for (j=0, exp=1; j<4; j++) if (a.f[j] == b.f[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tfloat: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_all_nge()
{
  int i, j;
  int idx;
  int exp, d;
  int count=0;
  
  DEBUG(1, "all_nge.........");
  exp = 1;
  
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      if (a.f[idx] >= b.f[idx]) SWAP(a.f[idx], b.f[idx], float);
    }
    d = vec_all_nge(a.vf, b.vf);
    for (j=0, exp=1; j<4; j++) if (a.f[j] >= b.f[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\td=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_all_ngt()
{
  int i, j;
  int idx;
  int exp, d;
  int count=0;
  
  DEBUG(1, "all_ngt.........");
  exp = 1;
  
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      if (a.f[idx] > b.f[idx]) SWAP(a.f[idx], b.f[idx], float);
    }
    d = vec_all_ngt(a.vf, b.vf);
    for (j=0, exp=1; j<4; j++) if (a.f[j] > b.f[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\td=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_all_nle()
{
  int i, j;
  int idx;
  int exp, d;
  int count=0;
  
  DEBUG(1, "all_nle.........");
  exp = 1;
  

  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      if (a.f[idx] <= b.f[idx]) SWAP(a.f[idx], b.f[idx], float);
    }
    d = vec_all_nle(a.vf, b.vf);
    for (j=0, exp=1; j<4; j++) if (a.f[j] <= b.f[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\td=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}

int test_vec_all_nlt()
{
  int i, j;
  int idx;
  int exp, d;
  int count=0;
  
  DEBUG(1, "all_nlt.........");
  exp = 1;
  
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      if (a.f[idx] < b.f[idx]) SWAP(a.f[idx], b.f[idx], float);
    }
    d = vec_all_nlt(a.vf, b.vf);
    for (j=0, exp=1; j<4; j++) if (a.f[j] < b.f[j]) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\td=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_all_numeric()
{
  int i, j;
  int idx;
  int exp, d;
  int count=0;
  
  DEBUG(1, "all_numeric.....");
  exp = 1;
  
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      for (j=0; j<4; j++) a.ui[j] = rand32() | 0x7F800000;
    } else {
      idx = rand16() & 0x3;
      a.ui[idx]  = (a.ui[idx] & ~0x7F800000) | (rand32() & 0x7F800000);
    }
    d = vec_all_numeric(a.vf);
    for (j=0, exp=1; j<4; j++) 
      if ((a.ui[j] & 0x7F800000) == 0x7F800000) exp = 0;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\td=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_any_eq()
{
  int i, j;
  int idx;
  int exp, d, e;
  int count=0;

  DEBUG(1, "any_eq..........");
  exp = 1;

  /* byte */
  
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      a.uc[idx] = b.uc[idx];
    }
    d = vec_any_eq(a.vuc, b.vuc);
    for (j=0, exp=0; j<16; j++) if (a.uc[j] == b.uc[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tbyte: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_eq(a.vuc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_any_eq(a.vc, b.vc);
    VERIFY_EQUAL(d, e);
    e = vec_any_eq(a.vc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_any_eq(a.vbc, b.vuc);
    VERIFY_EQUAL(d, e);
    e = vec_any_eq(a.vbc, b.vc);
    VERIFY_EQUAL(d, e);
    e = vec_any_eq(a.vbc, b.vbc);
    VERIFY_EQUAL(d, e);
  }
  
  /* halfword */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      a.us[idx] = b.us[idx];
    }
    d = vec_any_eq(a.vus, b.vus);
    for (j=0, exp=0; j<8; j++) if (a.us[j] == b.us[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\thalfword: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_eq(a.vus, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_any_eq(a.vs, b.vs);
    VERIFY_EQUAL(d, e);
    e = vec_any_eq(a.vs, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_any_eq(a.vbs, b.vus);
    VERIFY_EQUAL(d, e);
    e = vec_any_eq(a.vbs, b.vs);
    VERIFY_EQUAL(d, e);
    e = vec_any_eq(a.vbs, b.vbs);
    VERIFY_EQUAL(d, e);
  }

  
  /* word */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      a.ui[idx] = b.ui[idx];
    }
    d = vec_any_eq(a.vui, b.vui);
    for (j=0, exp=0; j<4; j++) if (a.ui[j] == b.ui[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tword: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_eq(a.vui, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_any_eq(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
    e = vec_any_eq(a.vi, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_any_eq(a.vbi, b.vui);
    VERIFY_EQUAL(d, e);
    e = vec_any_eq(a.vbi, b.vi);
    VERIFY_EQUAL(d, e);
    e = vec_any_eq(a.vbi, b.vbi);
    VERIFY_EQUAL(d, e);
  }
  
  /* float */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      a.f[idx] = b.f[idx];
    }
    d = vec_any_eq(a.vf, b.vf);
    for (j=0, exp=0; j<4; j++) if (a.f[j] == b.f[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tfloat: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_any_ge()
{
  int i, j;
  int idx;
  int exp, d, e;
  int count=0;
  
  DEBUG(1, "any_ge..........");
  exp = 1;
  
  /* uchar */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      if (a.uc[idx] < b.uc[idx]) SWAP(a.uc[idx], b.uc[idx], unsigned char);
    }
    d = vec_any_ge(a.vuc, b.vuc);
    for (j=0, exp=0; j<16; j++) if (a.uc[j] >= b.uc[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_ge(a.vuc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_any_ge(a.vbc, b.vuc);
    VERIFY_EQUAL(d, e);
  }
  
  /* char */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      if (a.c[idx] < b.c[idx]) SWAP(a.c[idx], b.c[idx], signed char);
    }
    d = vec_any_ge(a.vc, b.vc);
    for (j=0, exp=0; j<16; j++) if (a.c[j] >= b.c[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tchar: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_ge(a.vc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_any_ge(a.vbc, b.vc);
    VERIFY_EQUAL(d, e);
  }

  
  /* ushort */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      if (a.us[idx] < b.us[idx]) SWAP(a.us[idx], b.us[idx], unsigned short);
    }
    d = vec_any_ge(a.vus, b.vus);
    for (j=0, exp=0; j<8; j++) if (a.us[j] >= b.us[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_ge(a.vus, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_any_ge(a.vbs, b.vus);
    VERIFY_EQUAL(d, e);
  }
  
  /* short */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      if (a.s[idx] < b.s[idx]) SWAP(a.s[idx], b.s[idx], signed short);
    }
    d = vec_any_ge(a.vs, b.vs);
    for (j=0, exp=0; j<8; j++) if (a.s[j] >= b.s[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tshort: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_ge(a.vs, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_any_ge(a.vbs, b.vs);
    VERIFY_EQUAL(d, e);
  }

  /* uint */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (a.ui[idx] < b.ui[idx]) SWAP(a.ui[idx], b.ui[idx], unsigned int);
    }
    d = vec_any_ge(a.vui, b.vui);
    for (j=0, exp=0; j<4; j++) if (a.ui[j] >= b.ui[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_ge(a.vui, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_any_ge(a.vbi, b.vui);
    VERIFY_EQUAL(d, e);
  }
  
  /* int */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (a.i[idx] < b.i[idx]) SWAP(a.i[idx], b.i[idx], signed int);
    }
    d = vec_any_ge(a.vi, b.vi);
    for (j=0, exp=0; j<4; j++) if (a.i[j] >= b.i[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tint: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_ge(a.vi, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_any_ge(a.vbi, b.vi);
    VERIFY_EQUAL(d, e);
  }
  
  /* float */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      if (a.f[idx] < b.f[idx]) SWAP(a.f[idx], b.f[idx], float);
    }
    d = vec_any_ge(a.vf, b.vf);
    for (j=0, exp=0; j<4; j++) if (a.f[j] >= b.f[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tfloat: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_any_gt()
{
  int i, j;
  int idx;
  int exp, d, e;
  int count=0;
  
  DEBUG(1, "any_gt..........");
  exp = 1;
  
  /* uchar */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      if (a.uc[idx] <= b.uc[idx]) SWAP(a.uc[idx], b.uc[idx], unsigned char);
    }
    d = vec_any_gt(a.vuc, b.vuc);
    for (j=0, exp=0; j<16; j++) if (a.uc[j] > b.uc[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_gt(a.vuc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_any_gt(a.vbc, b.vuc);
    VERIFY_EQUAL(d, e);
  }
  
  /* char */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      if (a.c[idx] <= b.c[idx]) SWAP(a.c[idx], b.c[idx], signed char);
    }
    d = vec_any_gt(a.vc, b.vc);
    for (j=0, exp=0; j<16; j++) if (a.c[j] > b.c[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tchar: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_gt(a.vc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_any_gt(a.vbc, b.vc);
    VERIFY_EQUAL(d, e);
  }

  
  /* ushort */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      if (a.us[idx] <= b.us[idx]) SWAP(a.us[idx], b.us[idx], unsigned short);
    }
    d = vec_any_gt(a.vus, b.vus);
    for (j=0, exp=0; j<8; j++) if (a.us[j] > b.us[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_gt(a.vus, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_any_gt(a.vbs, b.vus);
    VERIFY_EQUAL(d, e);
  }
  
  /* short */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      if (a.s[idx] <= b.s[idx]) SWAP(a.s[idx], b.s[idx], signed short);
    }
    d = vec_any_gt(a.vs, b.vs);
    for (j=0, exp=0; j<8; j++) if (a.s[j] > b.s[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tshort: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_gt(a.vs, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_any_gt(a.vbs, b.vs);
    VERIFY_EQUAL(d, e);
  }

  /* uint */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (a.ui[idx] <= b.ui[idx]) SWAP(a.ui[idx], b.ui[idx], unsigned int);
    }
    d = vec_any_gt(a.vui, b.vui);
    for (j=0, exp=0; j<4; j++) if (a.ui[j] > b.ui[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_gt(a.vui, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_any_gt(a.vbi, b.vui);
    VERIFY_EQUAL(d, e);
  }
  
  /* int */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (a.i[idx] <= b.i[idx]) SWAP(a.i[idx], b.i[idx], signed int);
    }
    d = vec_any_gt(a.vi, b.vi);
    for (j=0, exp=0; j<4; j++) if (a.i[j] > b.i[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tint: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_gt(a.vi, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_any_gt(a.vbi, b.vi);
    VERIFY_EQUAL(d, e);
  }
  
  /* float */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      if (a.f[idx] <= b.f[idx]) SWAP(a.f[idx], b.f[idx], float);
    }
    d = vec_any_gt(a.vf, b.vf);
    for (j=0, exp=0; j<4; j++) if (a.f[j] > b.f[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tfloat: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}




int test_vec_any_le()
{
  int i, j;
  int idx;
  int exp, d, e;
  int count=0;
  
  DEBUG(1, "any_le..........");
  exp = 1;
  
  /* uchar */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      if (a.uc[idx] > b.uc[idx]) SWAP(a.uc[idx], b.uc[idx], unsigned char);
    }
    d = vec_any_le(a.vuc, b.vuc);
    for (j=0, exp=0; j<16; j++) if (a.uc[j] <= b.uc[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_le(a.vuc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_any_le(a.vbc, b.vuc);
    VERIFY_EQUAL(d, e);
  }
  
  /* char */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      if (a.c[idx] > b.c[idx]) SWAP(a.c[idx], b.c[idx], signed char);
    }
    d = vec_any_le(a.vc, b.vc);
    for (j=0, exp=0; j<16; j++) if (a.c[j] <= b.c[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tchar: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_le(a.vc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_any_le(a.vbc, b.vc);
    VERIFY_EQUAL(d, e);
  }

  
  /* ushort */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      if (a.us[idx] > b.us[idx]) SWAP(a.us[idx], b.us[idx], unsigned short);
    }
    d = vec_any_le(a.vus, b.vus);
    for (j=0, exp=0; j<8; j++) if (a.us[j] <= b.us[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_le(a.vus, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_any_le(a.vbs, b.vus);
    VERIFY_EQUAL(d, e);
  }
  
  /* short */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      if (a.s[idx] > b.s[idx]) SWAP(a.s[idx], b.s[idx], signed short);
    }
    d = vec_any_le(a.vs, b.vs);
    for (j=0, exp=0; j<8; j++) if (a.s[j] <= b.s[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tshort: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_le(a.vs, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_any_le(a.vbs, b.vs);
    VERIFY_EQUAL(d, e);
  }

  /* uint */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (a.ui[idx] > b.ui[idx]) SWAP(a.ui[idx], b.ui[idx], unsigned int);
    }
    d = vec_any_le(a.vui, b.vui);
    for (j=0, exp=0; j<4; j++) if (a.ui[j] <= b.ui[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_le(a.vui, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_any_le(a.vbi, b.vui);
    VERIFY_EQUAL(d, e);
  }
  
  /* int */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (a.i[idx] > b.i[idx]) SWAP(a.i[idx], b.i[idx], signed int);
    }
    d = vec_any_le(a.vi, b.vi);
    for (j=0, exp=0; j<4; j++) if (a.i[j] <= b.i[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tint: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_le(a.vi, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_any_le(a.vbi, b.vi);
    VERIFY_EQUAL(d, e);
  }
  
  /* float */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      if (a.f[idx] > b.f[idx]) SWAP(a.f[idx], b.f[idx], float);
    }
    d = vec_any_le(a.vf, b.vf);
    for (j=0, exp=0; j<4; j++) if (a.f[j] <= b.f[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tfloat: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}

int test_vec_any_lt()
{
  int i, j;
  int idx;
  int exp, d, e;
  int count=0;
  
  DEBUG(1, "any_lt..........");
  exp = 1;
  
  /* uchar */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      if (a.uc[idx] >= b.uc[idx]) SWAP(a.uc[idx], b.uc[idx], unsigned char);
    }
    d = vec_any_lt(a.vuc, b.vuc);
    for (j=0, exp=0; j<16; j++) if (a.uc[j] < b.uc[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_lt(a.vuc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_any_lt(a.vbc, b.vuc);
    VERIFY_EQUAL(d, e);
  }
  
  /* char */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      if (a.c[idx] >= b.c[idx]) SWAP(a.c[idx], b.c[idx], signed char);
    }
    d = vec_any_lt(a.vc, b.vc);
    for (j=0, exp=0; j<16; j++) if (a.c[j] < b.c[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tchar: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_lt(a.vc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_any_lt(a.vbc, b.vc);
    VERIFY_EQUAL(d, e);
  }

  /* ushort */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      if (a.us[idx] >= b.us[idx]) SWAP(a.us[idx], b.us[idx], unsigned short);
    }
    d = vec_any_lt(a.vus, b.vus);
    for (j=0, exp=0; j<8; j++) if (a.us[j] < b.us[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_lt(a.vus, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_any_lt(a.vbs, b.vus);
    VERIFY_EQUAL(d, e);
  }
  
  /* short */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      if (a.s[idx] >= b.s[idx]) SWAP(a.s[idx], b.s[idx], signed short);
    }
    d = vec_any_lt(a.vs, b.vs);
    for (j=0, exp=0; j<8; j++) if (a.s[j] < b.s[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tshort: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_lt(a.vs, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_any_lt(a.vbs, b.vs);
    VERIFY_EQUAL(d, e);
  }

  /* uint */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (a.ui[idx] >= b.ui[idx]) SWAP(a.ui[idx], b.ui[idx], unsigned int);
    }
    d = vec_any_lt(a.vui, b.vui);
    for (j=0, exp=0; j<4; j++) if (a.ui[j] < b.ui[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_lt(a.vui, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_any_lt(a.vbi, b.vui);
    VERIFY_EQUAL(d, e);
  }
  
  /* int */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
      b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (a.i[idx] >= b.i[idx]) SWAP(a.i[idx], b.i[idx], signed int);
    }
    d = vec_any_lt(a.vi, b.vi);
    for (j=0, exp=0; j<4; j++) if (a.i[j] < b.i[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tint: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_lt(a.vi, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_any_lt(a.vbi, b.vi);
    VERIFY_EQUAL(d, e);
  }
  
  /* float */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      if (a.f[idx] >= b.f[idx]) SWAP(a.f[idx], b.f[idx], float);
    }
    d = vec_any_lt(a.vf, b.vf);
    for (j=0, exp=0; j<4; j++) if (a.f[j] < b.f[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tfloat: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_any_nan()
{
  int i, j;
  int idx;
  int exp, d;
  int count=0;
  
  DEBUG(1, "any_nan.........");
  exp = 1;
  
  /* float */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      if (i & 1) a.ui[idx]  = (a.ui[idx] & ~0x007FFFFF) | (rand32() & 0x007FFFFF);
      else       a.ui[idx] |= 0x7F800000;
    }
    d = vec_any_nan(a.vf);
    for (j=0, exp=0; j<4; j++) 
      if (((a.ui[j] & 0x7F800000) == 0x7F800000) &&
	  ((a.ui[j] & 0x007FFFFF) != 0)) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\td=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}



int test_vec_any_ne()
{
  int i, j;
  int idx;
  int exp, d, e;
  int count=0;

  DEBUG(1, "any_ne..........");
  exp = 1;

  /* byte */
  
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = b.vui = rand_vui();
    } else {
      idx = rand16() & 0xF;
      a.uc[idx] = (unsigned char)rand16();
    }
    d = vec_any_ne(a.vuc, b.vuc);
    for (j=0, exp=0; j<16; j++) if (a.uc[j] != b.uc[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tbyte: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_ne(a.vuc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_any_ne(a.vc, b.vc);
    VERIFY_EQUAL(d, e);
    e = vec_any_ne(a.vc, b.vbc);
    VERIFY_EQUAL(d, e);
    e = vec_any_ne(a.vbc, b.vuc);
    VERIFY_EQUAL(d, e);
    e = vec_any_ne(a.vbc, b.vc);
    VERIFY_EQUAL(d, e);
    e = vec_any_ne(a.vbc, b.vbc);
    VERIFY_EQUAL(d, e);
  }
  
  /* halfword */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = b.vui = rand_vui();
    } else {
      idx = rand16() & 0x7;
      a.us[idx] = (unsigned short)rand16();
    }
    d = vec_any_ne(a.vus, b.vus);
    for (j=0, exp=0; j<8; j++) if (a.us[j] != b.us[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\thalfword: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_ne(a.vus, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_any_ne(a.vs, b.vs);
    VERIFY_EQUAL(d, e);
    e = vec_any_ne(a.vs, b.vbs);
    VERIFY_EQUAL(d, e);
    e = vec_any_ne(a.vbs, b.vus);
    VERIFY_EQUAL(d, e);
    e = vec_any_ne(a.vbs, b.vs);
    VERIFY_EQUAL(d, e);
    e = vec_any_ne(a.vbs, b.vbs);
    VERIFY_EQUAL(d, e);
  }

  
  /* word */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vui = b.vui = rand_vui();
    } else {
      idx = rand16() & 0x3;
      a.ui[idx] = (unsigned int)rand32();
    }
    d = vec_any_ne(a.vui, b.vui);
    for (j=0, exp=0; j<4; j++) if (a.ui[j] != b.ui[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tword: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
    /* verify equivalent variations */
    e = vec_any_ne(a.vui, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_any_ne(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
    e = vec_any_ne(a.vi, b.vbi);
    VERIFY_EQUAL(d, e);
    e = vec_any_ne(a.vbi, b.vui);
    VERIFY_EQUAL(d, e);
    e = vec_any_ne(a.vbi, b.vi);
    VERIFY_EQUAL(d, e);
    e = vec_any_ne(a.vbi, b.vbi);
    VERIFY_EQUAL(d, e);
  }
  
  /* float */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      a.f[idx] = randf(-2.0, 2.0);
    }
    d = vec_any_ne(a.vf, b.vf);
    for (j=0, exp=0; j<4; j++) if (a.f[j] != b.f[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tfloat: d=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_any_nge()
{
  int i, j;
  int idx;
  int exp, d;
  int count=0;
  
  DEBUG(1, "any_nge.........");
  exp = 1;
  
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      if (a.f[idx] >= b.f[idx]) SWAP(a.f[idx], b.f[idx], float);
    }
    d = vec_any_nge(a.vf, b.vf);
    for (j=0, exp=0; j<4; j++) if (a.f[j] < b.f[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\td=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_any_ngt()
{
  int i, j;
  int idx;
  int exp, d;
  int count=0;
  
  DEBUG(1, "any_ngt.........");
  exp = 1;
  
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      if (a.f[idx] > b.f[idx]) SWAP(a.f[idx], b.f[idx], float);
    }
    d = vec_any_ngt(a.vf, b.vf);
    for (j=0, exp=0; j<4; j++) if (a.f[j] <= b.f[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\td=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_any_nle()
{
  int i, j;
  int idx;
  int exp, d;
  int count=0;
  
  DEBUG(1, "any_nle.........");
  exp = 1;
  

  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      if (a.f[idx] <= b.f[idx]) SWAP(a.f[idx], b.f[idx], float);
    }
    d = vec_any_nle(a.vf, b.vf);
    for (j=0, exp=0; j<4; j++) if (a.f[j] > b.f[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\td=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}

int test_vec_any_nlt()
{
  int i, j;
  int idx;
  int exp, d;
  int count=0;
  
  DEBUG(1, "any_nlt.........");
  exp = 1;
  
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-2.0, 2.0);
      b.vf = rand_vf(-2.0, 2.0);
    } else {
      idx = rand16() & 0x3;
      if (a.f[idx] < b.f[idx]) SWAP(a.f[idx], b.f[idx], float);
    }
    d = vec_any_nlt(a.vf, b.vf);
    for (j=0, exp=0; j<4; j++) if (a.f[j] >= b.f[j]) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\td=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_any_numeric()
{
  int i, j;
  int idx;
  int exp, d;
  int count=0;
  
  DEBUG(1, "any_numeric.....");
  exp = 1;
  
  /* float */
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      for (j=0; j<4; j++) a.ui[j] = rand32() | 0x7F800000;
    } else {
      idx = rand16() & 0x3;
      a.ui[idx]  = (a.ui[idx] & ~0x7F800000) | (rand32() & 0x7F800000);
    }
    d = vec_any_numeric(a.vf);
    for (j=0, exp=0; j<4; j++) 
      if ((a.ui[j] & 0x7F800000) != 0x7F800000) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\td=%d exp=%d\n", d, exp);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


int test_vec_any_out()
{
  int i, j;
  int idx;
  int exp, d;
  int count=0;
  
  DEBUG(1, "any_out.........");
  exp = 1;
  
  for (i=0; ((i<samples) || (count=0)); i++) {
    if (exp) {
      a.vf = rand_vf(-0.2, 0.2);
      b.vf = rand_vf(-0.2, 4.0);
    } else {
      idx = rand16() & 0x3;
      b.f[idx] *= 0.5f;
    }
    d = vec_any_out(a.vf, b.vf);
    for (j=0, exp=0; j<4; j++) 
      if ((a.f[j] > b.f[j]) || (a.f[j] < -b.f[j])) exp = 1;
    if (d != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\td=%d exp=%d\n", d, exp);
      DEBUG(2, "\t\ta=%f %f %f %f\n", a.f[0], a.f[1], a.f[2], a.f[3]);
      DEBUG(2, "\t\tb=%f %f %f %f\n", b.f[0], b.f[1], b.f[2], b.f[3]);
      return (1);
    }
    count += exp;
  }
  
  DEBUG(1, passed);
  return (0);
}


DECLARE_MAIN()
{
  DECLARE_TEST_VARS();

  SCALE_SAMPLES(samples);

  INVOKE_TEST(test_vec_all_eq);
  INVOKE_TEST(test_vec_all_ge);
  INVOKE_TEST(test_vec_all_gt);
  INVOKE_TEST(test_vec_all_in);
  INVOKE_TEST(test_vec_all_le);
  INVOKE_TEST(test_vec_all_lt);
  INVOKE_TEST(test_vec_all_nan);
  INVOKE_TEST(test_vec_all_ne);
  INVOKE_TEST(test_vec_all_nge);
  INVOKE_TEST(test_vec_all_ngt);
  INVOKE_TEST(test_vec_all_nle);
  INVOKE_TEST(test_vec_all_nlt);
  INVOKE_TEST(test_vec_all_numeric);
  INVOKE_TEST(test_vec_any_eq);
  INVOKE_TEST(test_vec_any_ge);
  INVOKE_TEST(test_vec_any_gt);
  INVOKE_TEST(test_vec_any_le);
  INVOKE_TEST(test_vec_any_lt);
  INVOKE_TEST(test_vec_any_nan);
  INVOKE_TEST(test_vec_any_ne);
  INVOKE_TEST(test_vec_any_nge);
  INVOKE_TEST(test_vec_any_ngt);
  INVOKE_TEST(test_vec_any_nle);
  INVOKE_TEST(test_vec_any_nlt);
  INVOKE_TEST(test_vec_any_numeric);
  INVOKE_TEST(test_vec_any_out);
  
  DISPLAY_RESULTS();
}
