/* --------------------------------------------------------------  */
/* (C)Copyright 2001,2007,                                         */
/* International Business Machines Corporation,                    */
/* Sony Computer Entertainment, Incorporated,                      */
/* Toshiba Corporation,                                            */
/*                                                                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#include <stdio.h>
#include <vec_types.h>
#ifndef __SPU__
#include <altivec.h>
#include <spu2vmx.h>
#endif
#include "libtest.h"

typedef union {
  vec_uchar16	vuc;
  vec_char16	vc;
  vec_ushort8	vus;
  vec_short8	vs;
  vec_uint4	vui;
  vec_int4	vi;
  vec_ullong2	vull;
  vec_llong2	vll;
  vec_float4	vf;
  vec_double2	vd;
  unsigned char	uc[16];
  signed char	c[16];
  unsigned short us[8];
  signed short	s[8];
  unsigned int	ui[4];
  signed int	i[4];
  long long	ll[2];
  unsigned long long ull[2];
  float		f[4];
  double	d[2];
} v128;
  
static char *passed = "passed\n";
static char *failed = "FAILED\n";
static v128 a, b, c, d, e;


static int samples = 100;

unsigned int seed = 0;

unsigned int rand_local()
{
  seed = seed * 69607 + 54329;
  return (seed);
}

vec_uint4 rand_vui()
{
  static v128 val;

  val.ui[0] = rand_local();
  val.ui[1] = rand_local();
  val.ui[2] = rand_local();
  val.ui[3] = rand_local();

  return (val.vui);
}

unsigned long long rand_ullong()
{
  return ((unsigned long long)(rand_local) | ((unsigned long long)(rand_local) << 32));
}



float rand_float(float min, float max)
{
  union {
    float f;
    unsigned int ui;
  } x;

  x.ui = (rand_local() & 0x7FFFFF) | 0x3F800000;
  x.f -= 1.0f;
  x.f *= (max - min);
  x.f += min;
  return (x.f);
}

vec_float4 rand_vf(float min, float max)
{
  int i;
  static v128 val;

  for (i=0; i<4; i++) val.f[i] = rand_float(min, max);
  return (val.vf);
}


double rand_double(double min, double max)
{
  union {
    unsigned int ui[2];
    double d;
  } x;

  x.ui[0] = (rand_local() & 0x000FFFFF) | 0x3FF00000;
  x.ui[1] = rand_local();
  x.d -= 1.0;
  x.d *= max - min;
  x.d += min;
  return (x.d);
}

vec_double2 rand_vd(double min, double max)
{
  int i;
  static v128 val;

  for (i=0; i<2; i++) val.d[i] = rand_double(min, max);
  return (val.vd);
}


#define VERIFY_EQUAL(_a, _b)	if (verify_equal(__LINE__, _a, _b)) return (1);

int verify_equal(int line, v128 d, v128 e)
{
  if ((d.ui[0] != e.ui[0]) ||
      (d.ui[1] != e.ui[1]) ||
      (d.ui[2] != e.ui[2]) ||
      (d.ui[3] != e.ui[3])) {
    DEBUG(1, failed);
    DEBUG(2, "\t\tequivalence not verified at line %d\n", line);
    DEBUG(2, "\t\td=0x%x 0x%x 0x%x 0x%x\n", d.ui[0], d.ui[1], d.ui[2], d.ui[3]);
    DEBUG(2, "\t\te=0x%x 0x%x 0x%x 0x%x\n", e.ui[0], e.ui[1], e.ui[2], e.ui[3]);
    return (1);
  }
  return (0);
}      


int test_spu_absd()
{
  int i, j;

  DEBUG(1, "absd............");

  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vuc = spu_absd(a.vuc, b.vuc);
    for (j=0; j<16; j++) {
      unsigned char exp;
      exp = (a.uc[j] > b.uc[j]) ? (a.uc[j] - b.uc[j]) : (b.uc[j] - a.uc[j]);
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.uc[j], b.uc[j], d.uc[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_spu_add()
{
  int i, j;

  DEBUG(1, "add.............");
  
  /* word */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_add(a.vui, b.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = a.ui[j] + b.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_add(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
  }

  /* halfword */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vus = spu_add(a.vus, b.vus);
    for (j=0; j<8; j++) {
      unsigned short exp;
      exp = a.us[j] + b.us[j];
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tushort: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.us[j], b.us[j], d.us[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vs = spu_add(a.vs, b.vs);
    VERIFY_EQUAL(d, e);
  }

  /* float */
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    b.vf = rand_vf(-4.0f, 4.0f);
    d.vf = spu_add(a.vf, b.vf);
    for (j=0; j<4; j++) {
      float exp;
      exp = a.f[j] + b.f[j];
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\float: element=%d ", j);
	DEBUG(2, "a=%f b=%f d=%f exp=%f\n", a.f[j], b.f[j], d.f[j], exp);
	return (1);
      }
    }
  }

  /* double */
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0, 4.0);
    b.vd = rand_vd(-4.0, 4.0);
    d.vd = spu_add(a.vd, b.vd);
    for (j=0; j<2; j++) {
      double exp;
      exp = a.d[j] + b.d[j];
      if (exp != d.d[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tdouble: element=%d ", j);
	DEBUG(2, "a=%f b=%f d=%f exp=%f\n", a.d[j], b.d[j], d.d[j], exp);
	return (1);
      }
    }
  }


  /* immediate forms */

  /* short */
  for (i=0; i<samples; i++) {
    unsigned short imm;
    a.vui = rand_vui();
    imm   = (unsigned short)rand_local();
    d.vus = spu_add(a.vus, imm);
    for (j=0; j<8; j++) {
      unsigned short exp;
      exp = a.us[j] + imm;
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tushort immediate: element=%d ", j);
	DEBUG(2, "a=0x%x imm=0x%x d=0x%x exp=0x%x\n", a.us[j], imm, d.us[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vs = spu_add(a.vs, (short)imm);
    VERIFY_EQUAL(d, e);
  }
    
  /* int */
  for (i=0; i<samples; i++) {
    unsigned int imm;
    a.vui = rand_vui();
    imm   = rand_local();
    d.vui = spu_add(a.vui, imm);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = a.ui[j] + imm;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint immediate: element=%d ", j);
	DEBUG(2, "a=0x%x imm=0x%x d=0x%x exp=0x%x\n", a.ui[j], imm, d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_add(a.vi, (int)imm);
    VERIFY_EQUAL(d, e);
  }
    
  /* immmediate literals 10-bit
   */
  a.vui = rand_vui();
  d.vus = spu_add(a.vus, 511);
  for (j=0; j<8; j++) {
    unsigned short exp;
    exp = a.us[j] + 511;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=511 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vs = spu_add(a.vs, 511);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vui = spu_add(a.vui, -512);
  for (j=0; j<4; j++) {
    unsigned int exp;
    exp = a.ui[j] - 512;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=-512 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vi = spu_add(a.vi, -512);
  VERIFY_EQUAL(d, e);

  DEBUG(1, passed);
  return (0);
}

int test_spu_addx()
{
  int i, j;

  DEBUG(1, "addx............");
  
  /* word */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();
    d.vui = spu_addx(a.vui, b.vui, c.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = a.ui[j] + b.ui[j] + (c.ui[j] & 1);
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d a=0x%x ", j, a.ui[j]);
	DEBUG(2, "b=0x%x c=0x%x d=0x%x exp=0x%x\n", b.ui[j], c.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_addx(a.vi, b.vi, c.vi);
    VERIFY_EQUAL(d, e);
  }
  DEBUG(1, passed);
  return (0);
}



int test_spu_and()
{
  int i, j;

  DEBUG(1, "and.............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_and(a.vui, b.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = a.ui[j] & b.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_and(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
    e.vus = spu_and(a.vus, b.vus);
    VERIFY_EQUAL(d, e);
    e.vs = spu_and(a.vs, b.vs);
    VERIFY_EQUAL(d, e);
    e.vuc = spu_and(a.vuc, b.vuc);
    VERIFY_EQUAL(d, e);
    e.vc = spu_and(a.vc, b.vc);
    VERIFY_EQUAL(d, e);
    e.vf = spu_and(a.vf, b.vf);
    VERIFY_EQUAL(d, e);
    e.vll = spu_and(a.vll, b.vll);
    VERIFY_EQUAL(d, e);
    e.vull = spu_and(a.vull, b.vull);
    VERIFY_EQUAL(d, e);
    e.vd = spu_and(a.vd, b.vd);
    VERIFY_EQUAL(d, e);


  }


  /* immediate forms */
  /* int */
  for (i=0; i<samples; i++) {
    unsigned int imm;
    a.vui = rand_vui();
    imm   = (unsigned int)rand_local();
    d.vui = spu_and(a.vui, imm);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = a.ui[j] & imm;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint immediate: element=%d ", j);
	DEBUG(2, "a=0x%x imm=0x%x d=0x%x exp=0x%x\n", a.ui[j], imm, d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_and(a.vi, (signed int)(imm));
    VERIFY_EQUAL(d, e);
  }

  /* short */
  for (i=0; i<samples; i++) {
    unsigned short imm;
    a.vui = rand_vui();
    imm   = (unsigned short)rand_local();
    d.vus = spu_and(a.vus, imm);
    for (j=0; j<8; j++) {
      unsigned short exp;
      exp = a.us[j] & imm;
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tushort immediate: element=%d ", j);
	DEBUG(2, "a=0x%x imm=0x%x d=0x%x exp=0x%x\n", a.us[j], imm, d.us[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vs = spu_and(a.vs, (signed short)(imm));
    VERIFY_EQUAL(d, e);
  }

  /* byte */
  for (i=0; i<samples; i++) {
    unsigned char imm;
    a.vui = rand_vui();
    imm   = (unsigned char)rand_local();
    d.vuc = spu_and(a.vuc, imm);
    for (j=0; j<16; j++) {
      unsigned char exp;
      exp = a.uc[j] & imm;
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuchar immediate: element=%d ", j);
	DEBUG(2, "a=0x%x imm=0x%x d=0x%x exp=0x%x\n", a.uc[j], imm, d.uc[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vc = spu_and(a.vc, (signed char)(imm));
    VERIFY_EQUAL(d, e);
  }
    
  /* immmediate literals 10-bit
   */
  a.vui = rand_vui();
  d.vuc = spu_and(a.vuc, 267);
  for (j=0; j<16; j++) {
    unsigned char exp;
    exp = a.uc[j] & 267;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=267 d=0x%x exp=0x%x\n", a.uc[j], d.uc[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vc = spu_and(a.vc, 267);
  VERIFY_EQUAL(d, e);


  a.vui = rand_vui();
  d.vus = spu_and(a.vus, 511);
  for (j=0; j<8; j++) {
    unsigned short exp;
    exp = a.us[j] & 511;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=511 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vs = spu_and(a.vs, 511);
  VERIFY_EQUAL(d, e);


  a.vui = rand_vui();
  d.vuc = spu_and(a.vuc, -315);
  for (j=0; j<16; j++) {
    unsigned char exp;
    exp = a.uc[j] & -315;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=-315 d=0x%x exp=0x%x\n", a.uc[j], d.uc[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vc = spu_and(a.vc, -315);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vui = spu_and(a.vui, -512);
  for (j=0; j<4; j++) {
    unsigned int exp;
    exp = a.ui[j] & -512;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=-511 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vi = spu_and(a.vi, -512);
  VERIFY_EQUAL(d, e);

  DEBUG(1, passed);
  return (0);
}


int test_spu_andc()
{
  int i, j;

  DEBUG(1, "andc............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_andc(a.vui, b.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = a.ui[j] & ~b.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_andc(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
    e.vus = spu_andc(a.vus, b.vus);
    VERIFY_EQUAL(d, e);
    e.vs = spu_andc(a.vs, b.vs);
    VERIFY_EQUAL(d, e);
    e.vuc = spu_andc(a.vuc, b.vuc);
    VERIFY_EQUAL(d, e);
    e.vc = spu_andc(a.vc, b.vc);
    VERIFY_EQUAL(d, e);
    e.vf = spu_andc(a.vf, b.vf);
    VERIFY_EQUAL(d, e);
    e.vll = spu_andc(a.vll, b.vll);
    VERIFY_EQUAL(d, e);
    e.vull = spu_andc(a.vull, b.vull);
    VERIFY_EQUAL(d, e);
    e.vd = spu_andc(a.vd, b.vd);
    VERIFY_EQUAL(d, e);
  }

  DEBUG(1, passed);
  return (0);
}


int test_spu_avg()
{
  int i, j;

  DEBUG(1, "avg.............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vuc = spu_avg(a.vuc, b.vuc);
    for (j=0; j<16; j++) {
      unsigned char exp;
      exp = (unsigned char)(((unsigned int)a.uc[j] + (unsigned int)(b.uc[j]) + 1) >> 1);
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.uc[j], b.uc[j], d.uc[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}


#ifdef __SPU__
extern volatile int bisled_count;
extern void bisled_handler(void);


int test_spu_bisled()
{
  int i;
  unsigned int count, status;

  bisled_count = 0;

  DEBUG(1, "bisled..........");

  (void)spu_bisled(bisled_handler);
  if (bisled_count != 0) {
    DEBUG(1, failed);
    DEBUG(2, "\t\thandler called when not expected\n");
    return (1);
  }
  spu_writech(SPU_WrEventMask, 0x20);
  spu_writech(SPU_WrDec, 0);
  for (i=0; i<1000; i++) {
    (void)spu_bisled(bisled_handler);
    if (bisled_count) break;
  }
  spu_writech(SPU_WrEventMask, 0);

  /* Process (phantom) events 
   */
  count = spu_readchcnt(SPU_RdEventStat);
  if (count) {
    status = spu_readch(SPU_RdEventStat);
    if (status & 0x20) {
      spu_writech(SPU_WrEventAck, status);
    }
  }


  if (bisled_count == 0) {
    /* NOTE: this test will fail at this point if it is run
     * on the stand-alone spu simulator (spusim). Successful
     * execution of this test depends upon using the full system
     * simulator.
     */
    DEBUG(1, failed);
    DEBUG(2, "\t\thandler never called when expected\n");
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_spu_bisled_d()
{
  int i;
  unsigned int count, status;

  bisled_count = 0;

  DEBUG(1, "bisled_d........");
  
  (void)spu_bisled_d(bisled_handler);
  if (bisled_count != 0) {
    DEBUG(1, failed);
    DEBUG(2, "\t\thandler called when not expected\n");
    return (1);
  }
  spu_writech(SPU_WrEventMask, 0x20);
  spu_writech(SPU_WrDec, 0);
  for (i=0; i<1000; i++) {
    (void)spu_bisled_d(bisled_handler);
    if (bisled_count) break;
  }
  spu_writech(SPU_WrEventMask, 0);
  /* Process (phantom) events 
   */
  count = spu_readchcnt(SPU_RdEventStat);
  if (count) {
    status = spu_readch(SPU_RdEventStat);
    if (status & 0x20) {
      spu_writech(SPU_WrEventAck, status);
    }
  }

  if (bisled_count == 0) {
    /* NOTE: this test will fail at this point if it is run
     * on the stand-alone spu simulator (spusim). Successful
     * execution of this test depends upon using the full system
     * simulator.
     */
    DEBUG(1, failed);
    DEBUG(2, "\t\thandler never called when expected\n");
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}


int test_spu_bisled_e()
{
  bisled_count = 0;

  DEBUG(1, "bisled_e........");
  
  (void)spu_bisled_e(bisled_handler);
  if (bisled_count != 0) {
    DEBUG(1, failed);
    DEBUG(2, "\t\thandler called when not expected\n");
    return (1);
  }
  DEBUG(1, passed);
  return (0);
}

#endif /* __SPU__ */

int test_spu_cmpabseq()
{
  int i, j;

  DEBUG(1, "cmpabseq........");
  
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    for (j=0; j<4; j++) {
      if (rand_local() & 0x10) {
	b.ui[j] = a.ui[j];
	if (rand_local() & 0x10) {
	  b.ui[j] ^= 0x80000000;
	}
      }
    }
    d.vui = spu_cmpabseq(a.vf, b.vf);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = ((a.f[j] == b.f[j]) || (-a.f[j] == b.f[j]) || (a.f[j] == -b.f[j])) ? 0xFFFFFFFF : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tfloat: element=%d ", j);
	DEBUG(2, "a=%f b=%f d=0x%x exp=0x%x\n", a.f[j], b.f[j], d.ui[j], exp);
	return (1);
      }
    }
  }

  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0, 4.0);
    for (j=0; j<2; j++) {
      if (rand_local() & 0x10) {
	b.ull[j] = a.ull[j];
	if (rand_local() & 0x10) {
	  b.ull[j] ^= 0x8000000000000000ULL;
	}
      }
    }
    d.vull = spu_cmpabseq(a.vd, b.vd);
    for (j=0; j<2; j++) {
      unsigned long long exp;
      exp = ((a.d[j] == b.d[j]) || (-a.d[j] == b.d[j]) || (a.d[j] == -b.d[j])) ? 0xFFFFFFFFFFFFFFFFULL : 0ULL;
      if (exp != d.ull[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tdouble: element=%d ", j);
	DEBUG(2, "a=%f b=%f d=0x%llx exp=0x%llx\n", a.f[j], b.f[j], d.ull[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}


int test_spu_cmpabsgt()
{
  int i, j;

  DEBUG(1, "cmpabsgt........");
  
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    b.vf = rand_vf(-4.0f, 4.0f);
    d.vui = spu_cmpabsgt(a.vf, b.vf);
    for (j=0; j<4; j++) {
      unsigned int exp;
      float abs_a, abs_b;

      abs_a = (a.f[j] < 0.0f) ? -a.f[j] : a.f[j];
      abs_b = (b.f[j] < 0.0f) ? -b.f[j] : b.f[j];
      exp = (abs_a > abs_b) ? 0xFFFFFFFF : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tfloat: element=%d ", j);
	DEBUG(2, "a=%f b=%f d=0x%x exp=0x%x\n", a.f[j], b.f[j], d.ui[j], exp);
	return (1);
      }
    }
  }

  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0, 4.0);
    b.vd = rand_vd(-4.0, 4.0);
    d.vull = spu_cmpabsgt(a.vd, b.vd);
    for (j=0; j<2; j++) {
      unsigned long long exp;
      double abs_a, abs_b;

      abs_a = (a.d[j] < 0.0f) ? -a.d[j] : a.d[j];
      abs_b = (b.d[j] < 0.0f) ? -b.d[j] : b.d[j];
      exp = (abs_a > abs_b) ? 0xFFFFFFFFFFFFFFFFULL : 0ULL;
      if (exp != d.ull[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tdouble: element=%d ", j);
	DEBUG(2, "a=%f b=%f d=0x%llx exp=0x%llx\n", a.f[j], b.f[j], d.ull[j], exp);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_spu_cmpeq()
{
  int i, j;

  DEBUG(1, "cmpeq...........");
  
  /* byte */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    for (j=0; j<16; j++) if (b.uc[j] & 0x10) b.uc[j] = a.uc[j];
    d.vuc = spu_cmpeq(a.vuc, b.vuc);
    for (j=0; j<16; j++) {
      unsigned char exp;
      exp = (a.uc[j] == b.uc[j]) ? 0xFF : 0;
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tbyte: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.uc[j], b.uc[j], d.uc[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vuc = spu_cmpeq(a.vc, b.vc);
    VERIFY_EQUAL(d, e);
  }

  /* short */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    for (j=0; j<8; j++) if (b.us[j] & 0x10) b.us[j] = a.us[j];
    d.vus = spu_cmpeq(a.vus, b.vus);
    for (j=0; j<8; j++) {
      unsigned short exp;
      exp = (a.us[j] == b.us[j]) ? 0xFFFF : 0;
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\thalfword: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.us[j], b.us[j], d.us[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vus = spu_cmpeq(a.vs, b.vs);
    VERIFY_EQUAL(d, e);
  }

  /* int */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    for (j=0; j<4; j++) if (b.ui[j] & 0x10) b.ui[j] = a.ui[j];
    d.vui = spu_cmpeq(a.vui, b.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = (a.ui[j] == b.ui[j]) ? 0xFFFFFFFF : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tword: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vui = spu_cmpeq(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
  }

  /* float */
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    b.vf = rand_vf(-4.0f, 4.0f);
    for (j=0; j<4; j++) if (b.ui[j] & 0x10) b.ui[j] = a.ui[j];
    d.vui = spu_cmpeq(a.vf, b.vf);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = (a.f[j] == b.f[j]) ? 0xFFFFFFFF : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tfloat: element=%d ", j);
	DEBUG(2, "a=%f b=%f d=0x%x exp=0x%x\n", a.f[j], b.f[j], d.ui[j], exp);
	return (1);
      }
    }
  }

  /* double */
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0, 4.0);
    b.vd = rand_vd(-4.0, 4.0);
    for (j=0; j<2; j++) if (b.ui[j] & 0x10) b.ull[j] = a.ull[j];
    d.vull = spu_cmpeq(a.vd, b.vd);
    for (j=0; j<2; j++) {
      unsigned long long exp;
      exp = (a.d[j] == b.d[j]) ? 0xFFFFFFFFFFFFFFFFULL : 0ULL;
      if (exp != d.ull[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tdouble: element=%d ", j);
	DEBUG(2, "a=%f b=%f d=0x%llx exp=0x%llx\n", a.f[j], b.f[j], d.ull[j], exp);
	return (1);
      }
    }
  }

  /* immediate forms */
  /* byte */
  for (i=0; i<samples; i++) {
    unsigned char imm;
    a.vui = rand_vui();
    imm   = a.uc[rand_local() & 15];
    d.vuc = spu_cmpeq(a.vuc, imm);
    for (j=0; j<16; j++) {
      unsigned char exp;
       exp = (a.uc[j] == imm) ? 0xFF : 0;
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuchar immediate: element=%d ", j);
	DEBUG(2, "a=0x%x imm=0x%x d=0x%x exp=0x%x\n", a.uc[j], imm, d.uc[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vuc = spu_cmpeq(a.vc, (signed char)imm);
    VERIFY_EQUAL(d, e);
  }

  /* short */
  for (i=0; i<samples; i++) {
    unsigned short imm;
    a.vui = rand_vui();
    imm   = a.us[rand_local() & 7];
    d.vus = spu_cmpeq(a.vus, imm);
    for (j=0; j<8; j++) {
      unsigned short exp;
       exp = (a.us[j] == imm) ? 0xFFFF : 0;
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tushort immediate: element=%d ", j);
	DEBUG(2, "a=0x%x imm=0x%x d=0x%x exp=0x%x\n", a.us[j], imm, d.us[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vus = spu_cmpeq(a.vs, (short)imm);
    VERIFY_EQUAL(d, e);
  }
    
  /* int */
  for (i=0; i<samples; i++) {
    unsigned int imm;
    a.vui = rand_vui();
    imm   = a.ui[rand_local() & 3];
    d.vui = spu_cmpeq(a.vui, imm);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = (a.ui[j] == imm) ? 0xFFFFFFFF : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint immediate: element=%d ", j);
	DEBUG(2, "a=0x%x imm=0x%x d=0x%x exp=0x%x\n", a.ui[j], imm, d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vui = spu_cmpeq(a.vi, (int)imm);
    VERIFY_EQUAL(d, e);
  }
    
  /* immmediate literals 10-bit
   */
  a.vui = rand_vui();
  a.uc[rand_local() & 15] = 291;
  d.vuc = spu_cmpeq(a.vuc, 291);
  for (j=0; j<16; j++) {
    unsigned char exp;
    exp = (a.uc[j] == 35) ? 0xFF : 0;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=35 d=0x%x exp=0x%x\n", a.uc[j], d.uc[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vuc = spu_cmpeq(a.vc, 291);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  a.us[rand_local() & 7] = 511;
  d.vus = spu_cmpeq(a.vus, 511);
  for (j=0; j<8; j++) {
    unsigned short exp;
    exp = (a.us[j] == 511) ? 0xFFFF : 0;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=511 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vus = spu_cmpeq(a.vs, 511);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  a.ui[rand_local() & 3] = -512;
  d.vui = spu_cmpeq(a.vui, -512);
  for (j=0; j<4; j++) {
    unsigned int exp;
    exp = (a.ui[j] == (unsigned)(-512)) ? 0xFFFFFFFF : 0;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=-512 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vui = spu_cmpeq(a.vi, -512);
  VERIFY_EQUAL(d, e);

  DEBUG(1, passed);
  return (0);
}


int test_spu_cmpgt()
{
  int i, j;

  DEBUG(1, "cmpgt...........");
  
  /* uchar */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vuc = spu_cmpgt(a.vuc, b.vuc);
    for (j=0; j<16; j++) {
      unsigned char exp;
      exp = (a.uc[j] > b.uc[j]) ? 0xFF : 0;
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuchar: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.uc[j], b.uc[j], d.uc[j], exp);
	return (1);
      }
    }
  }

  /* char */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vuc = spu_cmpgt(a.vc, b.vc);
    for (j=0; j<16; j++) {
      unsigned char exp;
      exp = (a.c[j] > b.c[j]) ? 0xFF : 0;
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tchar: element=%d ", j);
	DEBUG(2, "a=%d b=%d d=0x%x exp=0x%x\n", a.c[j], b.c[j], d.uc[j], exp);
	return (1);
      }
    }
  }

  /* ushort */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vus = spu_cmpgt(a.vus, b.vus);
    for (j=0; j<8; j++) {
      unsigned short exp;
      exp = (a.us[j] > b.us[j]) ? 0xFFFF : 0;
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tushort: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.us[j], b.us[j], d.us[j], exp);
	return (1);
      }
    }
  }

  /* short */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vus = spu_cmpgt(a.vs, b.vs);
    for (j=0; j<8; j++) {
      unsigned short exp;
      exp = (a.s[j] > b.s[j]) ? 0xFFFF : 0;
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tshort: element=%d ", j);
	DEBUG(2, "a=%d b=%d d=0x%x exp=0x%x\n", a.s[j], b.s[j], d.us[j], exp);
	return (1);
      }
    }
  }

  /* uint */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_cmpgt(a.vui, b.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = (a.ui[j] > b.ui[j]) ? 0xFFFFFFFF : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }

  /* int */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_cmpgt(a.vi, b.vi);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = (a.i[j] > b.i[j]) ? 0xFFFFFFFF : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", j);
	DEBUG(2, "a=%d b=%d d=0x%x exp=0x%x\n", a.i[j], b.i[j], d.ui[j], exp);
	return (1);
      }
    }
  }

  /* float */
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    b.vf = rand_vf(-4.0f, 4.0f);
    d.vui = spu_cmpgt(a.vf, b.vf);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = (a.f[j] > b.f[j]) ? 0xFFFFFFFF : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tfloat: element=%d ", j);
	DEBUG(2, "a=%f b=%f d=0x%x exp=0x%x\n", a.f[j], b.f[j], d.ui[j], exp);
	return (1);
      }
    }
  }


  /* double */
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0, 4.0);
    b.vd = rand_vd(-4.0, 4.0);
    d.vull = spu_cmpgt(a.vd, b.vd);
    for (j=0; j<2; j++) {
      unsigned long long exp;
      exp = (a.d[j] > b.d[j]) ? 0xFFFFFFFFFFFFFFFFULL : 0ULL;
      if (exp != d.ull[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tdouble: element=%d ", j);
	DEBUG(2, "a=%f b=%f d=0x%llx exp=0x%llx\n", a.f[j], b.f[j], d.ull[j], exp);
	return (1);
      }
    }
  }

  /* immediate forms */
  /* uchar */
  for (i=0; i<samples; i++) {
    unsigned char imm;
    a.vui = rand_vui();
    imm   = (unsigned char)rand_local();
    d.vuc = spu_cmpgt(a.vuc, imm);
    for (j=0; j<16; j++) {
      unsigned char exp;
       exp = (a.uc[j] > imm) ? 0xFF : 0;
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuchar immediate: element=%d ", j);
	DEBUG(2, "a=0x%x imm=0x%x d=0x%x exp=0x%x\n", a.uc[j], imm, d.uc[j], exp);
	return (1);
      }
    }
  }

  /* char */
  for (i=0; i<samples; i++) {
    signed char imm;
    a.vui = rand_vui();
    imm   = (signed char)rand_local();
    d.vuc = spu_cmpgt(a.vc, imm);
    for (j=0; j<16; j++) {
      unsigned char exp;
       exp = (a.c[j] > imm) ? 0xFF : 0;
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tchar immediate: element=%d ", j);
	DEBUG(2, "a=%d imm=%d d=0x%x exp=0x%x\n", a.c[j], imm, d.uc[j], exp);
	return (1);
      }
    }
  }

  /* ushort */
  for (i=0; i<samples; i++) {
    unsigned short imm;
    a.vui = rand_vui();
    imm   = (unsigned short)rand_local();
    d.vus = spu_cmpgt(a.vus, imm);
    for (j=0; j<8; j++) {
      unsigned short exp;
       exp = (a.us[j] > imm) ? 0xFFFF : 0;
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tushort immediate: element=%d ", j);
	DEBUG(2, "a=0x%x imm=0x%x d=0x%x exp=0x%x\n", a.us[j], imm, d.us[j], exp);
	return (1);
      }
    }
  }

  /* short */
  for (i=0; i<samples; i++) {
    signed short imm;
    a.vui = rand_vui();
    imm   = (signed short)rand_local();
    d.vus = spu_cmpgt(a.vs, imm);
    for (j=0; j<8; j++) {
      unsigned short exp;
       exp = (a.s[j] > imm) ? 0xFFFF : 0;
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tshort immediate: element=%d ", j);
	DEBUG(2, "a=%d imm=%d d=0x%x exp=0x%x\n", a.s[j], imm, d.us[j], exp);
	return (1);
      }
    }
  }

  /* uint */
  for (i=0; i<samples; i++) {
    unsigned int imm;
    a.vui = rand_vui();
    imm   = (unsigned int)(rand_local());
    d.vui = spu_cmpgt(a.vui, imm);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = (a.ui[j] > imm) ? 0xFFFFFFFF : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint immediate: element=%d ", j);
	DEBUG(2, "a=0x%x imm=0x%x d=0x%x exp=0x%x\n", a.ui[j], imm, d.ui[j], exp);
	return (1);
      }
    }
  }

  /* int */
  for (i=0; i<samples; i++) {
    signed int imm;
    a.vui = rand_vui();
    imm   = (signed int)(rand_local());
    d.vui = spu_cmpgt(a.vi, imm);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = (a.i[j] > imm) ? 0xFFFFFFFF : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tint immediate: element=%d ", j);
	DEBUG(2, "a=%d imm=%d d=0x%x exp=0x%x\n", a.i[j], imm, d.ui[j], exp);
	return (1);
      }
    }
  }
    
  /* immmediate literals 10-bit
   */
  a.vui = rand_vui();
  a.us[rand_local() & 7] = 511;
  d.vus = spu_cmpgt(a.vus, 511);
  for (j=0; j<8; j++) {
    unsigned short exp;
    exp = (a.us[j] > 511) ? 0xFFFF : 0;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=511 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }

  d.vus = spu_cmpgt(a.vs, 511);
  for (j=0; j<8; j++) {
    unsigned short exp;
    exp = (a.s[j] > 511) ? 0xFFFF : 0;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tshort immediate (literal): element=%d ", j);
      DEBUG(2, "a=%d b=511 d=0x%x exp=0x%x\n", a.s[j], d.us[j], exp);
      return (1);
    }
  }


  a.vui = rand_vui();
  a.ui[rand_local() & 3] = -512;
  d.vui = spu_cmpgt(a.vui, -512);
  for (j=0; j<4; j++) {
    unsigned int exp;
    exp = (a.ui[j] > (unsigned)(-512)) ? 0xFFFFFFFF : 0;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=-512 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }

  a.vui = rand_vui();
  a.uc[rand_local() & 15] = 97;
  d.vuc = spu_cmpgt(a.vuc, 97);
  for (j=0; j<16; j++) {
    unsigned char exp;
    exp = (a.uc[j] > 97) ? 0xFFFFFFFF : 0;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=97 d=0x%x exp=0x%x\n", a.uc[j], d.uc[j], exp);
      return (1);
    }
  }

  d.vui = spu_cmpgt(a.vi, -512);
  for (j=0; j<4; j++) {
    unsigned int exp;
    exp = (a.i[j] > -512) ? 0xFFFFFFFF : 0;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint immediate (literal): element=%d ", j);
      DEBUG(2, "a=%d b=-512 d=0x%x exp=0x%x\n", a.i[j], d.ui[j], exp);
      return (1);
    }
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_cntb()
{
  int i, j;

  DEBUG(1, "cntb............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.vuc = spu_cntb(a.vuc);
    for (j=0; j<16; j++) {
      unsigned char exp;
      unsigned char element;

      for (exp = 0, element = a.uc[j]; element != 0; element >>= 1) {
	if (element & 1) exp++;
      }
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x d=%d exp=%d\n", a.uc[j], d.uc[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vuc = spu_cntb(a.vc);
    VERIFY_EQUAL(d, e);
  }
  DEBUG(1, passed);
  return (0);
}

int test_spu_cntlz()
{
  int i, j;

  DEBUG(1, "cntlz...........");
  
  for (i=0; i<samples; i++) {
    a.ui[0] = (unsigned int)(rand_local) >> ((unsigned int)(rand_local() & 31));
    a.ui[1] = (unsigned int)(rand_local) >> ((unsigned int)(rand_local() & 31));
    a.ui[2] = (unsigned int)(rand_local) >> ((unsigned int)(rand_local() & 31));
    a.ui[3] = (unsigned int)(rand_local) >> ((unsigned int)(rand_local() & 31));
    d.vui = spu_cntlz(a.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      unsigned int bit;

      for (exp = 0, bit = 0x80000000; bit != 0; bit >>= 1) {
	if ((a.ui[j] & bit) == 0) exp++;
	else break;
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x d=%d exp=%d\n", a.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vui = spu_cntlz(a.vi);
    VERIFY_EQUAL(d, e);
    e.vui = spu_cntlz(a.vf);
    VERIFY_EQUAL(d, e);
  }
  DEBUG(1, passed);
  return (0);
}


int test_spu_convtf()
{
  int i, j;

  DEBUG(1, "convtf..........");
  
  /* uint */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.vf = spu_convtf(a.vui, 5);
    for (j=0; j<4; j++) {
      float exp;

      exp = (float)a.ui[j];
      exp *= 0.03125f;
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", j);
	DEBUG(2, "a=0x%x d=%f exp=%f\n", a.ui[j], d.f[j], exp);
	return (1);
      }
    }
  }

  /* int */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.vf = spu_convtf(a.vi, 2);
    for (j=0; j<4; j++) {
      float exp;
      exp = (float)a.i[j];
      exp *= 0.25f;
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tint: element=%d ", j);
	DEBUG(2, "a=%d d=%f exp=%f\n", a.i[j], d.f[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}


int test_spu_convts()
{
  int i, j;

  DEBUG(1, "convts..........");
  
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-1024.0, 1024.0);
    d.vi = spu_convts(a.vf, 22);
    for (j=0; j<4; j++) {
      signed int exp;

      if (a.f[j] >= 512.0f) exp = 0x7FFFFFFF;
      else if (a.f[j] <= -512.0f) exp = 0x80000000;
      else exp = (int)(a.f[j] * 4194304.0f);

      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f d=%d exp=%d\n", a.f[j], d.i[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_convtu()
{
  int i, j;

  DEBUG(1, "convtu..........");
  
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-1024.0, 1024.0);
    d.vui = spu_convtu(a.vf, 22);
    for (j=0; j<4; j++) {
      unsigned int exp;

      if (a.f[j] >= 1024.0f) exp = 0xFFFFFFFF;
      else if (a.f[j] <= 0.0f) exp = 0x00000000;
      else exp = (unsigned int)(a.f[j] * 4194304.0f);

      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f d=0x%x exp=0x%x\n", a.f[j], d.ui[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}


int test_spu_dsync()
{
  DEBUG(1, "dsync...........");
  spu_dsync();
  DEBUG(1, passed);
  return (0);
}


int test_spu_eqv()
{
  int i, j;

  DEBUG(1, "eqv.............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_eqv(a.vui, b.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = ~(a.ui[j] ^ b.ui[j]);
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_eqv(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
    e.vus = spu_eqv(a.vus, b.vus);
    VERIFY_EQUAL(d, e);
    e.vs = spu_eqv(a.vs, b.vs);
    VERIFY_EQUAL(d, e);
    e.vuc = spu_eqv(a.vuc, b.vuc);
    VERIFY_EQUAL(d, e);
    e.vc = spu_eqv(a.vc, b.vc);
    VERIFY_EQUAL(d, e);
    e.vf = spu_eqv(a.vf, b.vf);
    VERIFY_EQUAL(d, e);
    e.vll = spu_eqv(a.vll, b.vll);
    VERIFY_EQUAL(d, e);
    e.vull = spu_eqv(a.vull, b.vull);
    VERIFY_EQUAL(d, e);
    e.vd = spu_eqv(a.vd, b.vd);
    VERIFY_EQUAL(d, e);
  }
  DEBUG(1, passed);
  return (0);
}


int test_spu_extend()
{
  int i, j;

  DEBUG(1, "extend..........");
  
  /* char to short */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.vs = spu_extend(a.vc);
    for (j=0; j<8; j++) {
      signed short exp;
      exp = a.c[2*j+1];
      if (exp != d.s[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tchar to short: element=%d ", j);
	DEBUG(2, "a=0x%x d=0x%x exp=0x%x\n", a.c[2*j+1], d.s[j], exp);
	return (1);
      }
    }
  }

  /* short to int */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.vi = spu_extend(a.vs);
    for (j=0; j<4; j++) {
      signed int exp;
      exp = a.s[2*j+1];
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tshort to int: element=%d ", j);
	DEBUG(2, "a=0x%x d=0x%x exp=0x%x\n", a.s[2*j+1], d.i[j], exp);
	return (1);
      }
    }
  }


  /* int to long long */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.vll = spu_extend(a.vi);
    for (j=0; j<4; j++) {
      unsigned int exp;
      if (j & 1) {
	exp = a.ui[j];
      } else {
	exp = (a.ui[j+1] & 0x80000000) ? 0xFFFFFFFF : 0;
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tint to long long: element=%d ", j);
	DEBUG(2, "a=0x%x d=0x%x exp=0x%x\n", a.ui[j|1], d.ui[j], exp);
	return (1);
      }
    }
  }


  /* float to double */
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    d.vd = spu_extend(a.vf);
    for (j=0; j<4; j++) {
      unsigned int exp, mantissa, sign, in;
      signed int exponent;
      
      in = a.ui[j&~1];
      mantissa = in & 0x007FFFFF;
      if (j & 1) {
	exp = mantissa << 29;
      } else {
	sign = in & 0x80000000;
	exponent  = (signed int)((in >> 23) & 0xFF) - 127;
	exp = sign | ((unsigned int)(exponent + 1023) << 20) | (mantissa >> 3);
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tfloat to double: element=%d ", j);
	DEBUG(2, "a=0x%x d=0x%x exp=0x%x\n", in, d.ui[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_extract()
{
  int i, element;

  DEBUG(1, "extract.........");
  

  /* uchar */
  for (i=0; i<samples; i++) {
    unsigned char scalar, exp;
    a.vui = rand_vui();
    element = rand_local();
    scalar = spu_extract(a.vuc, element);
    exp = a.uc[element & 15];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar: element=%d ", element);
      DEBUG(2, "d=0x%x exp=0x%x\n", scalar, exp);
      return (1);
    }
    scalar = spu_extract(a.vuc, 14);
    exp = a.uc[14];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar: element=14 (lit) ");
      DEBUG(2, "d=0x%x exp=0x%x\n", scalar, exp);
      return (1);
    }
  }

  /* char */
  for (i=0; i<samples; i++) {
    signed char scalar, exp;
    a.vui = rand_vui();
    element = rand_local();
    scalar = spu_extract(a.vc, element);
    exp = a.c[element & 15];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tchar: element=%d ", element);
      DEBUG(2, "d=%d exp=%d\n", scalar, exp);
      return (1);
    }
    scalar = spu_extract(a.vc, 4);
    exp = a.c[4];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tchar: element=4 (lit) ");
      DEBUG(2, "d=%d exp=%d\n", scalar, exp);
      return (1);
    }
  }

  /* ushort */
  for (i=0; i<samples; i++) {
    unsigned short scalar, exp;
    a.vui = rand_vui();
    element = rand_local();
    scalar = spu_extract(a.vus, element);
    exp = a.us[element & 7];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort: element=%d ", element);
      DEBUG(2, "d=0x%x exp=0x%x\n", scalar, exp);
      return (1);
    }
    scalar = spu_extract(a.vus, 6);
    exp = a.us[6];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort: element=6 (lit) ");
      DEBUG(2, "d=0x%x exp=0x%x\n", scalar, exp);
      return (1);
    }
  }

  /* short */
  for (i=0; i<samples; i++) {
    signed short scalar, exp;
    a.vui = rand_vui();
    element = rand_local();
    scalar = spu_extract(a.vs, element);
    exp = a.s[element & 7];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tshort: element=%d ", element);
      DEBUG(2, "d=%d exp=%d\n", scalar, exp);
      return (1);
    }
    scalar = spu_extract(a.vs, 0);
    exp = a.s[0];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tshort: element=0 (lit) ");
      DEBUG(2, "d=%d exp=%d\n", scalar, exp);
      return (1);
    }
  }

  /* uint */
  for (i=0; i<samples; i++) {
    unsigned int scalar, exp;
    a.vui = rand_vui();
    element = rand_local();
    scalar = spu_extract(a.vui, element);
    exp = a.ui[element & 3];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint: element=%d ", element);
      DEBUG(2, "d=0x%x exp=0x%x\n", scalar, exp);
      return (1);
    }
    scalar = spu_extract(a.vui, 2);
    exp = a.ui[2];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint: element=2 (lit) ");
      DEBUG(2, "d=0x%x exp=0x%x\n", scalar, exp);
      return (1);
    }
  }

  /* int */
  for (i=0; i<samples; i++) {
    signed int scalar, exp;
    a.vui = rand_vui();
    element = rand_local();
    scalar = spu_extract(a.vi, element);
    exp = a.i[element & 3];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tint: element=%d ", element);
      DEBUG(2, "d=%d exp=%d\n", scalar, exp);
      return (1);
    }
    scalar = spu_extract(a.vi, 1);
    exp = a.ui[1];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tint: element=1 (lit) ");
      DEBUG(2, "d=%d exp=%d\n", scalar, exp);
      return (1);
    }
  }

  /* float */
  for (i=0; i<samples; i++) {
    float scalar, exp;
    a.vf = rand_vf(-1.0f, 1.0f);
    element = rand_local();
    scalar = spu_extract(a.vf, element);
    exp = a.f[element & 3];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tfloat: element=%d ", element);
      DEBUG(2, "d=%f exp=%f\n", scalar, exp);
      return (1);
    }
    scalar = spu_extract(a.vf, 0);
    exp = a.f[0];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tint: element=0 (lit) ");
      DEBUG(2, "d=%f exp=%f\n", scalar, exp);
      return (1);
    }
  }

  /* ullong */
  for (i=0; i<samples; i++) {
    unsigned long long  scalar, exp;
    a.vui = rand_vui();
    element = rand_local();
    scalar = spu_extract(a.vull, element);
    exp = a.ull[element & 1];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tullong: element=%d\n", element);
      return (1);
    }
    scalar = spu_extract(a.vull, 1);
    exp = a.ull[1];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tullong: element=1 (lit)\n");
      return (1);
    }
  }

  /* llong */
  for (i=0; i<samples; i++) {
    signed long long  scalar, exp;
    a.vui = rand_vui();
    element = rand_local();
    scalar = spu_extract(a.vll, element);
    exp = a.ull[element & 1];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tllong: element=%d\n", element);
      return (1);
    }
    scalar = spu_extract(a.vll, 0);
    exp = a.ll[0];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tllong: element=0 (lit)\n");
      return (1);
    }
  }

  /* double */
  for (i=0; i<samples; i++) {
    double  scalar, exp;
    a.vd = rand_vd(-4.0, 4.0);
    element = rand_local();
    scalar = spu_extract(a.vd, element);
    exp = a.d[element & 1];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tdouble: element=%d\n", element);
      return (1);
    }
    scalar = spu_extract(a.vd, 1);
    exp = a.d[1];
    if (scalar != exp) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tullong: element=1 (lit)\n");
      return (1);
    }
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_gather()
{
  int i, j, k;

  DEBUG(1, "gather..........");
  
  /* byte */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.vui = spu_gather(a.vuc);
    for (j=0; j<4; j++) {
      unsigned int exp;

      exp = 0;
      if (j==0) {
	for (k=0; k<16; k++) {
	  if (a.uc[k] & 1) exp |= (1 << (15-k));
	}
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuchar: element=%d ", j);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vui = spu_gather(a.vc);
    VERIFY_EQUAL(d, e);
  }

  /* halfword */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.vui = spu_gather(a.vus);
    for (j=0; j<4; j++) {
      unsigned int exp;

      exp = 0;
      if (j==0) {
	for (k=0; k<8; k++) {
	  if (a.us[k] & 1) exp |= (1 << (7-k));
	}
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tushort: element=%d ", j);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vui = spu_gather(a.vs);
    VERIFY_EQUAL(d, e);
  }

  /* word */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.vui = spu_gather(a.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;

      exp = 0;
      if (j==0) {
	for (k=0; k<4; k++) {
	  if (a.ui[k] & 1) exp |= (1 << (3-k));
	}
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", j);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vui = spu_gather(a.vi);
    VERIFY_EQUAL(d, e);
    e.vui = spu_gather(a.vf);
    VERIFY_EQUAL(d, e);
  }

  DEBUG(1, passed);
  return (0);
}


int test_spu_genb()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "genb............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_genb(a.vui, b.vui);
    for (j=0; j<4; j++) {
      exp = (b.ui[j] > a.ui[j]) ? 0 : 1;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=0x%x ", j, a.ui[j]);
	DEBUG(2, "b=0x%x d=0x%x exp=0x%x\n", b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_genb(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
  }
  DEBUG(1, passed);
  return (0);
}


int test_spu_genbx()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "genbx...........");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();
    d.vui = spu_genbx(a.vui, b.vui, c.vui);
    for (j=0; j<4; j++) {
      if (c.ui[j] & 1) {
	exp = (a.ui[j] > b.ui[j]) ? 1 : 0;
      } else {
	exp = (a.ui[j] >= b.ui[j]) ? 1 : 0;
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=0x%x ", j, a.ui[j]);
	DEBUG(2, "b=0x%x c=0x%x d=0x%x exp=0x%x\n", b.ui[j], c.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_genbx(a.vi, b.vi, c.vi);
    VERIFY_EQUAL(d, e);
  }
  DEBUG(1, passed);
  return (0);
}


int test_spu_genc()
{
  int i, j;
  unsigned int sum, exp;

  DEBUG(1, "genc............");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_genc(a.vui, b.vui);
    for (j=0; j<4; j++) {
      sum = a.ui[j] + b.ui[j];
      exp = ((sum < a.ui[j]) && (sum < b.ui[j])) ? 1 : 0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=0x%x ", j, a.ui[j]);
	DEBUG(2, "b=0x%x d=0x%x exp=0x%x\n", b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_genc(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
  }
  DEBUG(1, passed);
  return (0);
}


int test_spu_gencx()
{
  int i, j;
  unsigned int sum, exp;

  DEBUG(1, "gencx...........");
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();
    d.vui = spu_gencx(a.vui, b.vui, c.vui);
    for (j=0; j<4; j++) {
      sum = a.ui[j] + b.ui[j] + (c.ui[j] & 1);
      if (c.ui[j] & 1) {
	exp = ((sum <= a.ui[j]) && (sum <= b.ui[j])) ? 1 : 0;
      } else {
	exp = ((sum < a.ui[j]) && (sum < b.ui[j])) ? 1 : 0;
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=0x%x ", j, a.ui[j]);
	DEBUG(2, "b=0x%x c=0x%x d=0x%x exp=0x%x\n", b.ui[j], c.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_gencx(a.vi, b.vi, c.vi);
    VERIFY_EQUAL(d, e);
  }
  DEBUG(1, passed);
  return (0);
}


int test_spu_hcmpeq()
{
  int i;

  DEBUG(1, "hcmpeq..........");
  
  for (i=0; i<samples; i++) {
    unsigned int a, b;
    a = (unsigned int)rand_local();
    b = (unsigned int)rand_local();
    if (a == b) continue;
    spu_hcmpeq(a, b);
  }

  for (i=0; i<samples; i++) {
    unsigned int a;
    a = (unsigned int)rand_local();
    if (a == 511) continue;
    spu_hcmpeq(a, (unsigned int)(511));
  }

  for (i=0; i<samples; i++) {
    signed int a, b;
    a = (signed int)rand_local();
    b = (signed int)rand_local();
    if (a == b) continue;
    spu_hcmpeq(a, b);
  }

  for (i=0; i<samples; i++) {
    signed int a;
    a = (signed int)rand_local();
    if (a == -512) continue;
    spu_hcmpeq(a, -512);
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_hcmpgt()
{
  int i;

  DEBUG(1, "hcmpgt..........");
  
  for (i=0; i<samples; i++) {
    unsigned int a, b;
    a = (unsigned int)rand_local();
    b = (unsigned int)rand_local();
    if (a > b) continue;
    spu_hcmpgt(a, b);
  }

  for (i=0; i<samples; i++) {
    unsigned int a;
    a = (unsigned int)rand_local() & 0x7FF;
    if (a > 511) continue;
    spu_hcmpgt(a, (unsigned int)(511));
  }

  for (i=0; i<samples; i++) {
    signed int a, b;
    a = (signed int)rand_local();
    b = (signed int)rand_local();
    if (a > b) continue;
    spu_hcmpgt(a, b);
  }

  for (i=0; i<samples; i++) {
    signed int a;
    a = (signed int)rand_local();
    if (a > -512) continue;
    spu_hcmpgt(a, -512);
  }

  DEBUG(1, passed);
  return (0);
}


#ifdef __SPU__
int test_spu_ienable_idisable()
{
  int count = 0;
  unsigned int stat1, stat2, stat3;

  stat1 = spu_readch(SPU_RdMachStat);
  spu_ienable();
  stat2 = spu_readch(SPU_RdMachStat);
  spu_idisable();
  stat3 = spu_readch(SPU_RdMachStat);

  DEBUG(1, "ienable.........");
  if (((stat1 & 1) == 1) ||
      ((stat2 & 1) == 0)) {
    DEBUG(1, failed);
    count++;
  } else {
    DEBUG(1, passed);
  }

  DEBUG(1, "idisable........");
  if ((stat3 & 1) == 1) {
    DEBUG(1, failed);
    count++;
  } else {
    DEBUG(1, passed);
  }
  return (count);
}

#endif /* __SPU__ */

int test_spu_insert()
{
  int i, j, element;

  DEBUG(1, "insert..........");
  

  /* byte */
  for (i=0; i<samples; i++) {
    unsigned char a, exp;
    b.vui = rand_vui();
    a = (unsigned char)rand_local();
    element = (int)rand_local();
    d.vuc = spu_insert(a, b.vuc, element);
    for (j=0; j<16; j++) {
      exp = (j == (element & 15)) ? a : b.uc[j];
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuchar: element=%d ", element);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.uc[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vc = spu_insert((signed char)(a), b.vc, element);
    VERIFY_EQUAL(d, e);
  }

  /* halfword */
  for (i=0; i<samples; i++) {
    unsigned short a, exp;
    b.vui = rand_vui();
    a = (unsigned short)rand_local();
    element = (int)rand_local();
    d.vus = spu_insert(a, b.vus, element);
    for (j=0; j<8; j++) {
      exp = (j == (element & 7)) ? a : b.us[j];
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tushort: element=%d ", element);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.us[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vs = spu_insert((signed short)(a), b.vs, element);
    VERIFY_EQUAL(d, e);
  }

  /* word */
  for (i=0; i<samples; i++) {
    unsigned int a, exp;
    b.vui = rand_vui();
    a = (unsigned int)rand_local();
    element = (int)rand_local();
    d.vui = spu_insert(a, b.vui, element);
    for (j=0; j<4; j++) {
      exp = (j == (element & 3)) ? a : b.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", element);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_insert((signed int)(a), b.vi, element);
    VERIFY_EQUAL(d, e);
  }

  /* doubleword */
  for (i=0; i<samples; i++) {
    unsigned long long a, exp;
    b.vui = rand_vui();
    a = (unsigned long long)rand_local();
    element = (int)rand_local();
    d.vull = spu_insert(a, b.vull, element);
    for (j=0; j<2; j++) {
      exp = (j == (element & 1)) ? a : b.ull[j];
      if (exp != d.ull[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tullong: element=%d\n", element);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vll = spu_insert((signed long long)(a), b.vll, element);
    VERIFY_EQUAL(d, e);
  }

  /* float */
  for (i=0; i<samples; i++) {
    float a, exp;
    b.vf = rand_vf(-4.0f, 4.0f);
    a = rand_float(-4.0f, 4.0f);
    element = (int)rand_local();
    d.vf = spu_insert(a, b.vf, element);
    for (j=0; j<4; j++) {
      exp = (j == (element & 3)) ? a : b.f[j];
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tfloat: element=%d ", element);
	DEBUG(2, "d=%f exp=%f\n", d.f[j], exp);
	return (1);
      }
    }
  }

  /* double */
  for (i=0; i<samples; i++) {
    double a, exp;
    b.vd = rand_vd(-4.0, 4.0);
    a = rand_double(-4.0, 4.0);
    element = (int)rand_local();
    d.vd = spu_insert(a, b.vd, element);
    for (j=0; j<2; j++) {
      exp = (j == (element & 1)) ? a : b.d[j];
      if (exp != d.d[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tdouble: element=%d\n", element);
	return (1);
      }
    }
  }

  /* immediate forms */
  /* byte */
  {
    unsigned char a;
    a = (unsigned char)rand_local();
    b.vui = rand_vui();

    e.vuc = b.vuc;
    e.uc[0] = a;
    d.vuc = spu_insert(a, b.vuc, 0);
    VERIFY_EQUAL(d, e);
    d.vc = spu_insert((signed char)(a), b.vc, 0);
    VERIFY_EQUAL(d, e);

    e.vuc = b.vuc;
    e.uc[1] = a;
    d.vuc = spu_insert(a, b.vuc, 1);
    VERIFY_EQUAL(d, e);
    d.vc = spu_insert((signed char)(a), b.vc, 1);
    VERIFY_EQUAL(d, e);

    e.vuc = b.vuc;
    e.uc[6] = a;
    d.vuc = spu_insert(a, b.vuc, 6);
    VERIFY_EQUAL(d, e);
    d.vc = spu_insert((signed char)(a), b.vc, 6);
    VERIFY_EQUAL(d, e);

    e.vuc = b.vuc;
    e.uc[15] = a;
    d.vuc = spu_insert(a, b.vuc, 15);
    VERIFY_EQUAL(d, e);
    d.vc = spu_insert((signed char)(a), b.vc, 15);
    VERIFY_EQUAL(d, e);
  }


  /* short */
  {
    unsigned short a;
    a = (unsigned short)rand_local();
    b.vui = rand_vui();

    e.vus = b.vus;
    e.us[0] = a;
    d.vus = spu_insert(a, b.vus, 0);
    VERIFY_EQUAL(d, e);
    d.vs = spu_insert((signed short)(a), b.vs, 0);
    VERIFY_EQUAL(d, e);

    e.vus = b.vus;
    e.us[4] = a;
    d.vus = spu_insert(a, b.vus, 4);
    VERIFY_EQUAL(d, e);
    d.vs = spu_insert((signed short)(a), b.vs, 4);
    VERIFY_EQUAL(d, e);

    e.vus = b.vus;
    e.us[7] = a;
    d.vus = spu_insert(a, b.vus, 7);
    VERIFY_EQUAL(d, e);
    d.vs = spu_insert((signed short)(a), b.vs, 7);
    VERIFY_EQUAL(d, e);
  }

  /* int */
  {
    unsigned int a;
    a = (unsigned int)rand_local();
    b.vui = rand_vui();

    e.vui = b.vui;
    e.ui[0] = a;
    d.vui = spu_insert(a, b.vui, 0);
    VERIFY_EQUAL(d, e);
    d.vi = spu_insert((signed int)(a), b.vi, 0);
    VERIFY_EQUAL(d, e);

    e.vui = b.vui;
    e.ui[1] = a;
    d.vui = spu_insert(a, b.vui, 1);
    VERIFY_EQUAL(d, e);
    d.vi = spu_insert((signed int)(a), b.vi, 1);
    VERIFY_EQUAL(d, e);

    e.vui = b.vui;
    e.ui[3] = a;
    d.vui = spu_insert(a, b.vui, 3);
    VERIFY_EQUAL(d, e);
    d.vi = spu_insert((signed int)(a), b.vi, 3);
    VERIFY_EQUAL(d, e);
  }

  /* long long */
  {
    unsigned long long a;
    a = (unsigned long long)rand_local();
    b.vui = rand_vui();

    e.vui = b.vui;
    e.ull[0] = a;
    d.vull = spu_insert(a, b.vull, 0);
    VERIFY_EQUAL(d, e);
    d.vll = spu_insert((signed long long)(a), b.vll, 0);
    VERIFY_EQUAL(d, e);

    e.vui = b.vui;
    e.ull[1] = a;
    d.vull = spu_insert(a, b.vull, 1);
    VERIFY_EQUAL(d, e);
    d.vll = spu_insert((signed long long)(a), b.vll, 1);
    VERIFY_EQUAL(d, e);
  }

  /* float */
  {
    float a;
    a = rand_float(-1.0f, 1.0f);
    b.vui = rand_vui();

    e.vui = b.vui;
    e.f[0] = a;
    d.vf = spu_insert(a, b.vf, 0);
    VERIFY_EQUAL(d, e);

    e.vui = b.vui;
    e.f[2] = a;
    d.vf = spu_insert(a, b.vf, 2);
    VERIFY_EQUAL(d, e);

    e.vui = b.vui;
    e.f[3] = a;
    d.vf = spu_insert(a, b.vf, 3);
    VERIFY_EQUAL(d, e);
  }

  /* double */
  {
    double a;
    a = rand_double(-1.0, 1.0);
    b.vui = rand_vui();

    e.vui = b.vui;
    e.d[0] = a;
    d.vd = spu_insert(a, b.vd, 0);
    VERIFY_EQUAL(d, e);

    e.vui = b.vui;
    e.d[1] = a;
    d.vd = spu_insert(a, b.vd, 1);
    VERIFY_EQUAL(d, e);
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_madd()
{
  int i, j;

  DEBUG(1, "madd............");
  
  /* integer */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();
    d.vi = spu_madd(a.vs, b.vs, c.vi);
    for (j=0; j<4; j++) {
      signed int exp;
      exp = (signed int)a.s[2*j+1] * (signed int)b.s[2*j+1] + c.i[j];
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tint: element=%d a=%d ", j, a.s[2*j+1]);
	DEBUG(2, "b=%d c=%d d=%d exp=%d\n", b.s[2*j+1], c.s[2*j+1], d.i[j], exp);
	return (1);
      }
    }
  }

  /* float */
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-1.0f, 1.0f);
    b.vf = rand_vf(-1.0f, 1.0f);
    c.vf = rand_vf(-1.0f, 1.0f);
    d.vf = spu_madd(a.vf, b.vf, c.vf);
    for (j=0; j<4; j++) {
      float exp;
      exp = a.f[j] * b.f[j] + c.f[j];
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tfloat: element=%d a=%f ", j, a.f[j]);
	DEBUG(2, "b=%f c=%f d=%f exp=%f\n", b.f[j], c.f[j], d.f[j], exp);
	return (1);
      }
    }
  }

  /* double */
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-1.0, 1.0);
    b.vd = rand_vd(-1.0, 1.0);
    c.vd = rand_vd(-1.0, 1.0);
    d.vd = spu_madd(a.vd, b.vd, c.vd);
    for (j=0; j<2; j++) {
      double exp;
      exp = a.d[j] * b.d[j] + c.d[j];
      if (exp != d.d[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tdouble: element=%d a=%f ", j, a.d[j]);
	DEBUG(2, "b=%f c=%f d=%f exp=%f\n", b.d[j], c.d[j], d.d[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}


int test_spu_maskb()
{
  int i, j;
  unsigned int a;

  DEBUG(1, "maskb...........");
  
  for (i=0; i<samples; i++) {
    a = rand_local();
    d.vuc = spu_maskb(a);
    for (j=0; j<16; j++) {
      unsigned char exp;
      exp = (a & (0x8000 >> j)) ? 0xFF : 0x00;
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x d=0x%x exp=0x%x\n", a, d.uc[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vuc = spu_maskb((signed int)(a));
    VERIFY_EQUAL(d, e);
    e.vuc = spu_maskb((unsigned short)(a));
    VERIFY_EQUAL(d, e);
    e.vuc = spu_maskb((signed short)(a));
    VERIFY_EQUAL(d, e);
  }
  
  /* immediate */
  d.vuc = spu_maskb(0xA55A);
  for (j=0; j<16; j++) {
    unsigned char exp;
    exp = (0xA55A & (0x8000 >> j)) ? 0xFF : 0x00;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\t(lit): element=%d ", j);
      DEBUG(2, "a=0xA55A d=0x%x exp=0x%x\n", d.uc[j], exp);
      return (1);
    }
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_maskh()
{
  int i, j;
  unsigned int a;

  DEBUG(1, "maskh...........");
  
  for (i=0; i<samples; i++) {
    a = rand_local();
    d.vus = spu_maskh(a);
    for (j=0; j<8; j++) {
      unsigned short exp;
      exp = (a & (0x80 >> j)) ? 0xFFFF : 0x0000;
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x d=0x%x exp=0x%x\n", a, d.us[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vus = spu_maskh((signed int)(a));
    VERIFY_EQUAL(d, e);
    e.vus = spu_maskh((unsigned short)(a));
    VERIFY_EQUAL(d, e);
    e.vus = spu_maskh((signed short)(a));
    VERIFY_EQUAL(d, e);
    e.vus = spu_maskh((unsigned char)(a));
    VERIFY_EQUAL(d, e);
    e.vus = spu_maskh((signed char)(a));
    VERIFY_EQUAL(d, e);
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_maskw()
{
  int i, j;
  unsigned int a;
  
  DEBUG(1, "maskw...........");
  
  for (i=0; i<samples; i++) {
    a = rand_local();
    d.vui = spu_maskw(a);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = (a & (0x8 >> j)) ? 0xFFFFFFFF : 0x0;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x d=0x%x exp=0x%x\n", a, d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vui = spu_maskw((unsigned int)(a));
    VERIFY_EQUAL(d, e);
    e.vui = spu_maskw((signed short)(a));
    VERIFY_EQUAL(d, e);
    e.vui = spu_maskw((unsigned short)(a));
    VERIFY_EQUAL(d, e);
    e.vui = spu_maskw((signed char)(a));
    VERIFY_EQUAL(d, e);
    e.vui = spu_maskw((unsigned char)(a));
    VERIFY_EQUAL(d, e);
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_mfcdma32()
{
  DEBUG(1, "mfcdma32........untested\n");
  return (0);
}

int test_spu_mfcdma64()
{
  DEBUG(1, "mfcdma64........untested\n");
  return (0);
}

int test_spu_mfcstat()
{
  unsigned int stat;

  DEBUG(1, "mfcstat.........");

  /* In order to do a wait on any, you must specify a non-zero 
   * tag mask.
   */
  spu_writech(MFC_WrTagMask, -1);

  stat = spu_mfcstat(0);
  stat = spu_mfcstat(1);
  stat = spu_mfcstat(2);

  DEBUG(1, passed);
  return (0);
}
  
int test_spu_mfspr()
{
  /* Since there are no defined spr registers. This test will do nothing more
   * that read an spr and write it.
   */
  unsigned int spr;

  DEBUG(1, "mfspr...........");
  spr = spu_mfspr(0);
  DEBUG(1, passed);
  DEBUG(1, "mtspr...........");
  spu_mtspr(0, spr);
  DEBUG(1, passed);
  return (0);
}


#ifdef __SPU__
vec_double2 dbl_add(vec_double2 a, vec_double2 b)
{
  return (spu_add(a, b));
}

/* This function derives the round mode from a series of arithmetic
 * tests. The second parameter, loop_var, was added to keep the optimizer
 * for pulling the call to this function out of the loop. 
 */
unsigned int do_round_test(int element, int loop_var)
{
  int idx, index;
  v128 t1, t2, t3, t4;

  if ((loop_var) >= 0) {
    t1.vd = dbl_add((vec_double2)spu_splats((unsigned long long)0x3FF0000000000000ULL),
		    (vec_double2)spu_splats((unsigned long long)0x3FE0000000000001ULL));
    
    t2.vd = dbl_add((vec_double2)spu_splats((unsigned long long)0x3FF0000000000000ULL),
		    (vec_double2)spu_splats((unsigned long long)0x3FE0000000000003ULL));
    
    t3.vd = dbl_add((vec_double2)spu_splats((unsigned long long)0xBFF0000000000000ULL),
		    (vec_double2)spu_splats((unsigned long long)0xBFE0000000000001ULL));

    t4.vd = dbl_add((vec_double2)spu_splats((unsigned long long)0xBFF0000000000000ULL),
		    (vec_double2)spu_splats((unsigned long long)0xBFE0000000000003ULL));
	     
    idx = element * 2 + 1;
    index = (((t1.ui[idx] & 1) << 3) |
	     ((t2.ui[idx] & 1) << 2) |
	     ((t3.ui[idx] & 1) << 1) |
	     ((t4.ui[idx] & 1) << 0));
    
    if (index == 0) return 0;
    if (index == 5) return 1;
    
    if (index == 9) return 2;
    if (index == 6) return 3;
  }
  
  return (-1);
}
#endif 


int test_spu_mffpscr()
{
#ifdef __SPU__
  int i;
#endif

  DEBUG(1, "mffpscr.........");
  a.vui = spu_mffpscr();
  DEBUG(1, passed);

  /* Change the rounding mode */
  b = a;
  b.ui[0] ^= 0x100;

  DEBUG(1, "mtfpscr.........");
  spu_mtfpscr(b.vui);

  c.vui = spu_mffpscr();
  spu_mtfpscr(a.vui);

#ifdef __SPU__
  for (i=0; i<4; i++) {
    if (b.ui[i] != c.ui[i]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", i);
	DEBUG(2, "b=0x%x c=0x%x\n", b.ui[i], c.ui[i]);
	return (1);
    }
  }

  /* Verfiy independent rounding mode controls
   */
  for (i=0; i<16; i++) {
    int r0, r1;

    b.ui[0] = (b.ui[0] & 0xFFFF00FF) | (i << 8);
    spu_mtfpscr(b.vui);

    r0 = do_round_test(0, i);
    r1 = do_round_test(1, i);
    
    if (((r0 << 2) | r1) != i) {
      DEBUG(1, failed);
      DEBUG(2, "\tFailed validating independent rounding mode control\n");
      DEBUG(2, "\ti=%d r0=%d r1=%d\n", i, r0, r1);
      spu_mtfpscr(a.vui);
      return (1);
    }
  }
  spu_mtfpscr(a.vui);
#endif

  DEBUG(1, passed);
  return (0);
}


int test_spu_mhhadd()
{
  int i, j;

  DEBUG(1, "mhhadd..........");

  /* signed */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();
    d.vi = spu_mhhadd(a.vs, b.vs, c.vi);
    for (j=0; j<4; j++) {
      signed int exp;
      exp = (signed int)a.s[2*j] * (signed int)b.s[2*j] + c.i[j];
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tsigned element=%d a=%d ", j, a.s[2*j]);
	DEBUG(2, "b=%d c=%d d=%d exp=%d\n", b.s[2*j], c.i[j], d.i[j], exp);
	return (1);
      }
    }
  }

  /* unsigned */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();
    d.vui = spu_mhhadd(a.vus, b.vus, c.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = (unsigned int)a.us[2*j] * (unsigned int)b.us[2*j] + c.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tunsigned element=%d a=0x%x ", j, a.us[2*j]);
	DEBUG(2, "b=0x%x c=0x%x d=0x%x exp=0x%x\n", b.us[2*j], c.ui[j], d.ui[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}


int test_spu_msub()
{
  int i, j;

  DEBUG(1, "msub............");
  
  /* float */
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    b.vf = rand_vf(-4.0f, 4.0f);
    c.vf = rand_vf(-4.0f, 4.0f);
    d.vf = spu_msub(a.vf, b.vf, c.vf);
    for (j=0; j<4; j++) {
      float exp;
      exp = a.f[j] * b.f[j] - c.f[j];
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tfloat: element=%d a=%f ", j, a.f[j]);
	DEBUG(2, "b=%f c=%f d=%f exp=%f\n", b.f[j], c.f[j], d.f[j], exp);
	return (1);
      }
    }
  }

  /* double */
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0f, 4.0f);
    b.vd = rand_vd(-4.0f, 4.0f);
    c.vd = rand_vd(-4.0f, 4.0f);
    d.vd = spu_msub(a.vd, b.vd, c.vd);
    for (j=0; j<2; j++) {
      double exp;
      exp = a.d[j] * b.d[j] - c.d[j];
      if (exp != d.d[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tdouble: element=%d a=%f ", j, a.d[j]);
	DEBUG(2, "b=%f c=%f d=%f exp=%f\n", b.d[j], c.d[j], d.d[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_mul()
{
  int i, j;

  DEBUG(1, "mul.............");
  
  /* float */
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    b.vf = rand_vf(-4.0f, 4.0f);
    d.vf = spu_mul(a.vf, b.vf);
    for (j=0; j<4; j++) {
      float exp;
      exp = a.f[j] * b.f[j];
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tfloat: element=%d a=%f ", j, a.f[j]);
	DEBUG(2, "b=%f c=%f exp=%f\n", b.f[j], d.f[j], exp);
	return (1);
      }
    }
  }

  /* double */
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0f, 4.0f);
    b.vd = rand_vd(-4.0f, 4.0f);
    d.vd = spu_mul(a.vd, b.vd);
    for (j=0; j<2; j++) {
      double exp;
      exp = a.d[j] * b.d[j];
      if (exp != d.d[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tdouble: element=%d a=%f ", j, a.d[j]);
	DEBUG(2, "b=%f d=%f exp=%f\n", b.d[j], d.d[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}


int test_spu_mulh()
{
  int i, j;

  DEBUG(1, "mulh............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vi = spu_mulh(a.vs, b.vs);
    for (j=0; j<4; j++) {
      signed int exp;
      exp = ((signed int)a.s[2*j] * (signed int)b.s[2*j+1]) << 16;
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%d ", j, a.s[2*j]);
	DEBUG(2, "b=%d d=%d exp=%d\n", b.s[2*j+1], d.i[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}


int test_spu_mule()
{
  int i, j;

  DEBUG(1, "mule............");

  /* signed */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vi = spu_mule(a.vs, b.vs);
    for (j=0; j<4; j++) {
      signed int exp;
      exp = (signed int)a.s[2*j] * (signed int)b.s[2*j];
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tsigned element=%d a=%d ", j, a.s[2*j]);
	DEBUG(2, "b=%d d=%d exp=%d\n", b.s[2*j], d.i[j], exp);
	return (1);
      }
    }
  }

  /* unsigned */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_mule(a.vus, b.vus);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = (unsigned int)a.us[2*j] * (unsigned int)b.us[2*j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tunsigned element=%d a=0x%x ", j, a.us[2*j]);
	DEBUG(2, "b=0x%x d=0x%x exp=0x%x\n", b.us[2*j], d.ui[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}


int test_spu_mulo()
{
  int i, j;

  DEBUG(1, "mulo............");
  
  /* signed */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vi = spu_mulo(a.vs, b.vs);
    for (j=0; j<4; j++) {
      signed int exp;
      exp = (signed int)a.s[2*j+1] * (signed int)b.s[2*j+1];
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tsigned element=%d a=%d ", j, a.s[2*j+1]);
	DEBUG(2, "b=%d d=%d exp=%d\n", b.s[2*j+1], d.i[j], exp);
	return (1);
      }
    }

    /* immediate */
    d.vi = spu_mulo(a.vs, (signed short)(-512));
    for (j=0; j<4; j++) {
      signed int exp;
      exp = (signed int)a.s[2*j+1] * -512;
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tsigned (lit) element=%d a=%d ", j, a.s[2*j+1]);
	DEBUG(2, "b=-512 d=%d exp=%d\n", d.i[j], exp);
	return (1);
      }
    }
  }

  /* unsigned */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_mulo(a.vus, b.vus);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = (unsigned int)a.us[2*j+1] * (unsigned int)b.us[2*j+1];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tunsigned element=%d a=%d ", j, a.us[2*j+1]);
	DEBUG(2, "b=%d d=%d exp=%d\n", b.us[2*j+1], d.ui[j], exp);
	return (1);
      }
    }

    /* immediate */
    d.vui = spu_mulo(a.vus, (unsigned short)(511));
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = (unsigned int)a.us[2*j+1] * (unsigned int)(511);
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tunsigned (lit) element=%d a=%d ", j, a.us[2*j+1]);
	DEBUG(2, "b=511 d=%d exp=%d\n", d.ui[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_mulsr()
{
  int i, j;

  DEBUG(1, "mulsr...........");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vi = spu_mulsr(a.vs, b.vs);
    for (j=0; j<4; j++) {
      signed int exp;
      exp = ((signed int)a.s[2*j+1] * (signed int)b.s[2*j+1]) >> 16;
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d a=%d ", j, a.s[2*j+1]);
	DEBUG(2, "b=%d d=%d exp=%d\n", b.s[2*j+1], d.i[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_nand()
{
  int i, j;

  DEBUG(1, "nand............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_nand(a.vui, b.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = ~(a.ui[j] & b.ui[j]);
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_nand(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
    e.vus = spu_nand(a.vus, b.vus);
    VERIFY_EQUAL(d, e);
    e.vs = spu_nand(a.vs, b.vs);
    VERIFY_EQUAL(d, e);
    e.vuc = spu_nand(a.vuc, b.vuc);
    VERIFY_EQUAL(d, e);
    e.vc = spu_nand(a.vc, b.vc);
    VERIFY_EQUAL(d, e);
    e.vf = spu_nand(a.vf, b.vf);
    VERIFY_EQUAL(d, e);
    e.vll = spu_nand(a.vll, b.vll);
    VERIFY_EQUAL(d, e);
    e.vull = spu_nand(a.vull, b.vull);
    VERIFY_EQUAL(d, e);
    e.vd = spu_nand(a.vd, b.vd);
    VERIFY_EQUAL(d, e);
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_nmadd()
{
  int i, j;

  DEBUG(1, "nmadd...........");
  
  /* double */
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0f, 4.0f);
    b.vd = rand_vd(-4.0f, 4.0f);
    c.vd = rand_vd(-4.0f, 4.0f);
    d.vd = spu_nmadd(a.vd, b.vd, c.vd);
    for (j=0; j<2; j++) {

      double exp, delta;
      exp = -(a.d[j] * b.d[j] + c.d[j]);
      /* Allow for inefficient compiler optimization of the above computation
       * of the expected results.
       */
      delta = exp - d.d[j];
      if (delta < 0.0) delta = -delta;
      if (delta > 0.000000002) {
	DEBUG(1, failed);
	DEBUG(1, "delta = %f\n", 1000000.0f * delta);
	DEBUG(2, "\t\tdouble: element=%d a=%f ", j, a.d[j]);
	DEBUG(2, "b=%f c=%f d=%f exp=%f\n", b.d[j], c.d[j], d.d[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_nmsub()
{
  int i, j;

  DEBUG(1, "nmsub...........");
  
  /* float */
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    b.vf = rand_vf(-4.0f, 4.0f);
    c.vf = rand_vf(-4.0f, 4.0f);
    d.vf = spu_nmsub(a.vf, b.vf, c.vf);
    for (j=0; j<4; j++) {
      float exp, delta;
      exp = c.f[j] - a.f[j] * b.f[j];
      /* Allow for inefficient compiler optimization of the above computation
       * of the expected results.
       */
      delta = exp - d.f[j];
      if (delta < 0.0f) delta = -delta;
      if (delta > 0.000002f) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tfloat: element=%d a=%f ", j, a.f[j]);
	DEBUG(2, "b=%f c=%f d=%f exp=%f\n", b.f[j], c.f[j], d.f[j], exp);
	return (1);
      }
    }
  }

  /* double */
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0f, 4.0f);
    b.vd = rand_vd(-4.0f, 4.0f);
    c.vd = rand_vd(-4.0f, 4.0f);
    d.vd = spu_nmsub(a.vd, b.vd, c.vd);
    for (j=0; j<2; j++) {
      double exp, delta;
      exp = c.d[j] - a.d[j] * b.d[j];
      /* Allow for inefficient compiler optimization of the above computation
       * of the expected results.
       */
      delta = exp - d.d[j];
      if (delta < 0.0) delta = -delta;
      if (delta > 0.000000002) {
	DEBUG(1, failed);
	DEBUG(1, "delta = %f\n", 1000000.0f * delta);
	DEBUG(2, "\t\tdouble: element=%d a=%f ", j, a.d[j]);
	DEBUG(2, "b=%f c=%f d=%f exp=%f\n", b.d[j], c.d[j], d.d[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}


int test_spu_nor()
{
  int i, j;

  DEBUG(1, "nor.............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_nor(a.vui, b.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = ~(a.ui[j] | b.ui[j]);
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_nor(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
    e.vus = spu_nor(a.vus, b.vus);
    VERIFY_EQUAL(d, e);
    e.vs = spu_nor(a.vs, b.vs);
    VERIFY_EQUAL(d, e);
    e.vuc = spu_nor(a.vuc, b.vuc);
    VERIFY_EQUAL(d, e);
    e.vc = spu_nor(a.vc, b.vc);
    VERIFY_EQUAL(d, e);
    e.vf = spu_nor(a.vf, b.vf);
    VERIFY_EQUAL(d, e);
    e.vll = spu_nor(a.vll, b.vll);
    VERIFY_EQUAL(d, e);
    e.vull = spu_nor(a.vull, b.vull);
    VERIFY_EQUAL(d, e);
    e.vd = spu_nor(a.vd, b.vd);
    VERIFY_EQUAL(d, e);
  }

  DEBUG(1, passed);
  return (0);
}


int test_spu_or()
{
  int i, j;

  DEBUG(1, "or..............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_or(a.vui, b.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = a.ui[j] | b.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_or(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
    e.vus = spu_or(a.vus, b.vus);
    VERIFY_EQUAL(d, e);
    e.vs = spu_or(a.vs, b.vs);
    VERIFY_EQUAL(d, e);
    e.vuc = spu_or(a.vuc, b.vuc);
    VERIFY_EQUAL(d, e);
    e.vc = spu_or(a.vc, b.vc);
    VERIFY_EQUAL(d, e);
    e.vf = spu_or(a.vf, b.vf);
    VERIFY_EQUAL(d, e);
    e.vll = spu_or(a.vll, b.vll);
    VERIFY_EQUAL(d, e);
    e.vull = spu_or(a.vull, b.vull);
    VERIFY_EQUAL(d, e);
    e.vd = spu_or(a.vd, b.vd);
    VERIFY_EQUAL(d, e);


  }


  /* immediate forms */
  /* int */
  for (i=0; i<samples; i++) {
    unsigned int imm;
    a.vui = rand_vui();
    imm   = (unsigned int)rand_local();
    d.vui = spu_or(a.vui, imm);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = a.ui[j] | imm;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint immediate: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_or(a.vi, (signed int)(imm));
    VERIFY_EQUAL(d, e);
  }

  /* short */
  for (i=0; i<samples; i++) {
    unsigned short imm;
    a.vui = rand_vui();
    imm   = (unsigned short)rand_local();
    d.vus = spu_or(a.vus, imm);
    for (j=0; j<8; j++) {
      unsigned short exp;
      exp = a.us[j] | imm;
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tushort immediate: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.us[j], b.us[j], d.us[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vs = spu_or(a.vs, (signed short)(imm));
    VERIFY_EQUAL(d, e);
  }
    
  /* byte */
  for (i=0; i<samples; i++) {
    unsigned char imm;
    a.vui = rand_vui();
    imm   = (unsigned char)rand_local();
    d.vuc = spu_or(a.vuc, imm);
    for (j=0; j<16; j++) {
      unsigned char exp;
      exp = a.uc[j] | imm;
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuchar immediate: element=%d ", j);
	DEBUG(2, "a=0x%x imm=0x%x d=0x%x exp=0x%x\n", a.uc[j], imm, d.uc[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vc = spu_or(a.vc, (signed char)(imm));
    VERIFY_EQUAL(d, e);
  }

  /* immmediate literals 10-bit
   */
  a.vui = rand_vui();
  d.vuc = spu_or(a.vuc, 267);
  for (j=0; j<16; j++) {
    unsigned char exp;
    exp = a.uc[j] | 267;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=267 d=0x%x exp=0x%x\n", a.uc[j], d.uc[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vc = spu_or(a.vc, 267);
  VERIFY_EQUAL(d, e);


  a.vui = rand_vui();
  d.vus = spu_or(a.vus, 511);
  for (j=0; j<8; j++) {
    unsigned short exp;
    exp = a.us[j] | 511;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=511 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vs = spu_or(a.vs, 511);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vuc = spu_or(a.vuc, -315);
  for (j=0; j<16; j++) {
    unsigned char exp;
    exp = a.uc[j] | -315;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=-315 d=0x%x exp=0x%x\n", a.uc[j], d.uc[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vc = spu_or(a.vc, -315);
  VERIFY_EQUAL(d, e);


  a.vui = rand_vui();
  d.vui = spu_or(a.vui, -512);
  for (j=0; j<4; j++) {
    unsigned int exp;
    exp = a.ui[j] | -512;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=-511 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vi = spu_or(a.vi, -512);
  VERIFY_EQUAL(d, e);

  DEBUG(1, passed);
  return (0);
}


int test_spu_orc()
{
  int i, j;

  DEBUG(1, "orc.............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_orc(a.vui, b.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = a.ui[j] | ~b.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_orc(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
    e.vus = spu_orc(a.vus, b.vus);
    VERIFY_EQUAL(d, e);
    e.vs = spu_orc(a.vs, b.vs);
    VERIFY_EQUAL(d, e);
    e.vuc = spu_orc(a.vuc, b.vuc);
    VERIFY_EQUAL(d, e);
    e.vc = spu_orc(a.vc, b.vc);
    VERIFY_EQUAL(d, e);
    e.vf = spu_orc(a.vf, b.vf);
    VERIFY_EQUAL(d, e);
    e.vll = spu_orc(a.vll, b.vll);
    VERIFY_EQUAL(d, e);
    e.vull = spu_orc(a.vull, b.vull);
    VERIFY_EQUAL(d, e);
    e.vd = spu_orc(a.vd, b.vd);
    VERIFY_EQUAL(d, e);
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_orx()
{
  int i, j;
  unsigned int exp;

  DEBUG(1, "orx.............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    d.vui = spu_orx(a.vui);
    for (j=0; j<4; j++) {
      if (j == 0) {
	exp = a.ui[0] | a.ui[1] | a.ui[2] | a.ui[3];
      } else {
	exp = 0;
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=0x%x 0x%x 0x%x 0x%x  ", a.ui[0], a.ui[1], a.ui[2], a.ui[3]);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_orx(a.vi);
    VERIFY_EQUAL(d, e);
  }
  DEBUG(1, passed);
  return (0);
}


int test_spu_promote()
{
  int i, element;

  DEBUG(1, "promote.........");
  
  /* uchar */
  for (i=0; i<samples; i++) {
    unsigned char a, exp;
    a = (unsigned char)rand_local();
    element = (int)rand_local();
    d.vuc = spu_promote(a, element);
    exp = a;
    if (exp != d.uc[element & 15]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar: element=0x%x ", element);
      DEBUG(2, "d=0x%x exp=0x%x\n", d.uc[element & 15], exp);
      return (1);
    }
  }

  /* char */
  for (i=0; i<samples; i++) {
    signed char a, exp;
    a = (signed char)rand_local();
    element = (int)rand_local();
    d.vc = spu_promote(a, element);
    exp = a;
    if (exp != d.c[element & 15]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tchar: element=0x%x ", element);
      DEBUG(2, "d=0x%x exp=0x%x\n", d.c[element & 15], exp);
      return (1);
    }
  }

  /* ushort */
  for (i=0; i<samples; i++) {
    unsigned short a, exp;
    a = (unsigned short)rand_local();
    element = (int)rand_local();
    d.vus = spu_promote(a, element);
    exp = a;
    if (exp != d.us[element & 7]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort: element=0x%x ", element);
      DEBUG(2, "d=0x%x exp=0x%x\n", d.us[element & 7], exp);
      return (1);
    }
  }

  /* short */
  for (i=0; i<samples; i++) {
    signed short a, exp;
    a = (signed short)rand_local();
    element = (int)rand_local();
    d.vs = spu_promote(a, element);
    exp = a;
    if (exp != d.s[element & 7]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tshort: element=0x%x ", element);
      DEBUG(2, "d=0x%x exp=0x%x\n", d.s[element & 7], exp);
      return (1);
    }
  }

  /* uint */
  for (i=0; i<samples; i++) {
    unsigned int a, exp;
    a = (unsigned int)rand_local();
    element = (int)rand_local();
    d.vui = spu_promote(a, element);
    exp = a;
    if (exp != d.ui[element & 3]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint: element=0x%x ", element);
      DEBUG(2, "d=0x%x exp=0x%x\n", d.ui[element & 3], exp);
      return (1);
    }
  }

  /* int */
  for (i=0; i<samples; i++) {
    signed int a, exp;
    a = (signed int)rand_local();
    element = (int)rand_local();
    d.vi = spu_promote(a, element);
    exp = a;
    if (exp != d.i[element & 3]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tint: element=0x%x ", element);
      DEBUG(2, "d=0x%x exp=0x%x\n", d.i[element & 3], exp);
      return (1);
    }
  }


  /* unsigned long long */
  for (i=0; i<samples; i++) {
    unsigned long long a, exp;
    a = rand_ullong();
    element = (int)rand_local();
    d.vull = spu_promote(a, element);
    exp = a;
    if (exp != d.ull[element & 1]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tullong: element=0x%x\n", element);
      return (1);
    }
  }

  /* signed long long */
  for (i=0; i<samples; i++) {
    signed long long a, exp;
    a = (signed long long)rand_local();
    element = (int)rand_local();
    d.vll = spu_promote(a, element);
    exp = a;
    if (exp != d.ll[element & 1]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tllong: element=0x%x\n", element);
      return (1);
    }
  }

  /* float */
  for (i=0; i<samples; i++) {
    float a, exp;
    a = rand_float(-4.0f, 4.0f);
    element = (int)rand_local();
    d.vf = spu_promote(a, element);
    exp = a;
    if (exp != d.f[element & 3]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tfloat: element=0x%x ", element);
      DEBUG(2, "d=%f exp=%f\n", d.f[element & 3], exp);
      return (1);
    }
  }

  /* double */
  for (i=0; i<samples; i++) {
    double a, exp;
    a = rand_double(-4.0, 4.0);
    element = (int)rand_local();
    d.vd = spu_promote(a, element);
    exp = a;
    if (exp != d.d[element & 1]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tdouble: element=0x%x\n", element);
      return (1);
    }
  }

  /* immediate forms */
  {
    unsigned char a;
    a = (unsigned char)rand_local();

    b.vuc = spu_promote(a, 0);
    c.vuc = spu_promote(a, 5);
    d.vuc = spu_promote(a, 15);
    if ((a != b.uc[0]) ||
	(a != c.uc[5]) ||
	(a != d.uc[15])) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar literal a=%d b=%d c=%d d=%d\n", a, b.uc[0], c.uc[5], d.uc[15]);
    }
  }

  {
    signed char a;
    a = (signed char)rand_local();

    b.vc = spu_promote(a, 0);
    c.vc = spu_promote(a, 4);
    d.vc = spu_promote(a, 15);
    if ((a != b.c[0]) ||
	(a != c.c[4]) ||
	(a != d.c[15])) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tchar literal a=%d b=%d c=%d d=%d\n", a, b.c[0], c.c[4], d.c[15]);
    }
  }

  {
    unsigned short a;
    a = (unsigned short)rand_local();

    b.vus = spu_promote(a, 0);
    c.vus = spu_promote(a, 5);
    d.vus = spu_promote(a, 7);
    if ((a != b.us[0]) ||
	(a != c.us[5]) ||
	(a != d.us[7])) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort literal\n");
    }
  }

  {
    signed short a;
    a = (signed short)rand_local();

    b.vs = spu_promote(a, 0);
    c.vs = spu_promote(a, 2);
    d.vs = spu_promote(a, 7);
    if ((a != b.s[0]) ||
	(a != c.s[2]) ||
	(a != d.s[7])) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tshort literal\n");
    }
  }

  {
    unsigned int a;
    a = (unsigned int)rand_local();

    b.vui = spu_promote(a, 0);
    c.vui = spu_promote(a, 2);
    d.vui = spu_promote(a, 3);
    if ((a != b.ui[0]) ||
	(a != c.ui[2]) ||
	(a != d.ui[3])) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint literal\n");
    }
  }

  {
    signed int a;
    a = (signed int)rand_local();

    b.vi = spu_promote(a, 0);
    c.vi = spu_promote(a, 1);
    d.vi = spu_promote(a, 3);
    if ((a != b.i[0]) ||
	(a != c.i[1]) ||
	(a != d.i[3])) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tint literal\n");
    }
  }

  {
    float a;
    a = rand_float(-2.0f, 2.0f);

    b.vf = spu_promote(a, 0);
    c.vf = spu_promote(a, 1);
    d.vf = spu_promote(a, 3);
    if ((a != b.f[0]) ||
	(a != c.f[1]) ||
	(a != d.f[3])) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tfloat literal\n");
    }
  }

  {
    unsigned long long a;
    a = rand_ullong();

    b.vull = spu_promote(a, 0);
    c.vull = spu_promote(a, 1);
    if ((a != b.ull[0]) ||
	(a != c.ull[1])) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tullong literal\n");
    }
  }

  {
    signed long long a;
    a = ((signed long long)rand_local() << 32 |
	 (signed long long)rand_local());

    b.vll = spu_promote(a, 0);
    c.vll = spu_promote(a, 1);
    if ((a != b.ll[0]) ||
	(a != c.ll[1])) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tllong literal\n");
    }
  }

  {
    double a;
    a = rand_double(-2.0, 2.0);

    b.vd = spu_promote(a, 0);
    c.vd = spu_promote(a, 1);
    if ((a != b.d[0]) ||
	(a != c.d[1])) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tdouble literal\n");
    }
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_re()
{
  int i, j;

  DEBUG(1, "re..............");

  for (i=0; i<samples; i++) {
    a.vf = rand_vf(1.0f, 3.0f);
    d.vf = spu_re(a.vf);
    for (j=0; j<4; j++) {
      float err;
      err = a.f[j] * d.f[j];
      if ((err > 1.00025f) || (err < 0.99975)) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f d=%f err=%f\n", a.f[j], d.f[j], err);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_spu_readch()
{
  unsigned int i;

  DEBUG(1, "readch..........");
  i = spu_readch(SPU_RdDec);
  DEBUG(1, passed);
  return (0);
}

int test_spu_readchcnt()
{
  unsigned int i;

  DEBUG(1, "readchcnt.......");
  i = spu_readchcnt(SPU_RdEventStat);
  DEBUG(1, passed);
  return (0);
}

int test_spu_readchqw()
{
  vector unsigned int i;

  DEBUG(1, "readchqw........");
  i = spu_readchqw(SPU_RdDec);
  DEBUG(1, passed);
  return (0);
}


int test_spu_rl()
{
  int i, j;
  int count;

  DEBUG(1, "rl..............");
  
  /* halfword */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vus = spu_rl(a.vus, b.vs);
    for (j=0; j<8; j++) {
      unsigned short exp;
      count = (int)b.us[j] & 0xF;
      exp = (a.us[j] << count) | (a.us[j] >> (16-count));
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tushort element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.us[j], b.us[j], d.us[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vs = spu_rl(a.vs, b.vs);
    VERIFY_EQUAL(d, e);
  }

  /* word */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_rl(a.vui, b.vi);
    for (j=0; j<4; j++) {
      unsigned int exp;
      count = (int)b.ui[j] & 0x1F;
      exp = (a.ui[j] << count) | (a.ui[j] >> (32-count));
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_rl(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
  }


  /* Immediate forms */
  /* halfword */
  a.vui = rand_vui();
  d.vus = spu_rl(a.vus, 0x10);
  for (j=0; j<8; j++) {
    unsigned short exp;
    exp = a.us[j];
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=0 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vs = spu_rl(a.vs, 0x10);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vus = spu_rl(a.vus, 0xF);
  for (j=0; j<8; j++) {
    unsigned short exp;
    exp = (a.us[j] << 15) | (a.us[j] >> 1);
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=15 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vs = spu_rl(a.vs, 0xF);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vus = spu_rl(a.vus, 54);
  for (j=0; j<8; j++) {
    unsigned short exp;
    exp = (a.us[j] << 6) | (a.us[j] >> 10);
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=6 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vs = spu_rl(a.vs, 54);
  VERIFY_EQUAL(d, e);


  /* word */
  a.vui = rand_vui();
  d.vui = spu_rl(a.vui, 0x20);
  for (j=0; j<4; j++) {
    unsigned int exp;
    exp = a.ui[j];
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=0 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vi = spu_rl(a.vi, 0x20);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vui = spu_rl(a.vui, 0x1F);
  for (j=0; j<4; j++) {
    unsigned int exp;
    exp = (a.ui[j] << 31) | (a.ui[j] >> 1);
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=31 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vi = spu_rl(a.vi, 0x1F);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vui = spu_rl(a.vui, 39);
  for (j=0; j<4; j++) {
    unsigned int exp;
    exp = (a.ui[j] << 7) | (a.ui[j] >> 25);
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=7 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vi = spu_rl(a.vi, 39);
  VERIFY_EQUAL(d, e);

  DEBUG(1, passed);
  return (0);
}

int test_spu_rlmask()
{
  int i, j;
  int count;

  DEBUG(1, "rlmask..........");
  
  /* halfword */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vus = spu_rlmask(a.vus, b.vs);
    for (j=0; j<8; j++) {
      unsigned short exp;
      count = (int)b.us[j] & 0x1F;
      if (count > 16) {
	count = -count & 0xF;
	exp = a.us[j] >> count;
      } else {
	exp = (count) ? 0 : a.us[j];
      }
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tushort element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.us[j], b.us[j], d.us[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vs = spu_rlmask(a.vs, b.vs);
    VERIFY_EQUAL(d, e);
  }

  /* word */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_rlmask(a.vui, b.vi);
    for (j=0; j<4; j++) {
      unsigned int exp;
      count = (int)b.ui[j] & 0x3F;
      if (count > 32) {
	count = -count & 0x1F;
	exp = a.ui[j] >> count;
      } else {
	exp = (count) ? 0 : a.ui[j];
      }
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_rlmask(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
  }


  /* Immediate forms */
  /* halfword */
  a.vui = rand_vui();
  d.vus = spu_rlmask(a.vus, 0);
  for (j=0; j<8; j++) {
    unsigned short exp;
    exp = a.us[j];
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=0 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vs = spu_rlmask(a.vs, 0);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vus = spu_rlmask(a.vus, -16);
  for (j=0; j<8; j++) {
    unsigned short exp;
    exp = 0;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=-16 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vs = spu_rlmask(a.vs, -16);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vus = spu_rlmask(a.vus, -7);
  for (j=0; j<8; j++) {
    unsigned short exp;
    exp = a.us[j] >> 7;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=-7 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vs = spu_rlmask(a.vs, -7);
  VERIFY_EQUAL(d, e);


  /* word */
  a.vui = rand_vui();
  d.vui = spu_rlmask(a.vui, 0);
  for (j=0; j<4; j++) {
    unsigned int exp;
    exp = a.ui[j];
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=0 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vi = spu_rlmask(a.vi, 0);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vui = spu_rlmask(a.vui, -32);
  for (j=0; j<4; j++) {
    unsigned int exp;
    exp = 0;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=-32 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vi = spu_rlmask(a.vi, -32);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vui = spu_rlmask(a.vui, -13);
  for (j=0; j<4; j++) {
    unsigned int exp;
    exp = a.ui[j] >> 13;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=-13 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vi = spu_rlmask(a.vi, -13);
  VERIFY_EQUAL(d, e);

  DEBUG(1, passed);
  return (0);
}


int test_spu_rlmaska()
{
  int i, j;
  int count;

  DEBUG(1, "rlmaska.........");
  
  /* halfword */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vs = spu_rlmaska(a.vs, b.vs);
    for (j=0; j<8; j++) {
      signed short exp;
      count = (int)b.us[j] & 0x1F;
      if (count > 16) {
	count = -count & 0xF;
	exp = a.s[j] >> count;
      } else {
	if (count) {
	  exp = (a.s[j] & 0x8000) ? 0xFFFF : 0;
	} else {
	  exp = a.s[j];
	}
      }
      if (exp != d.s[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tshort element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.s[j], b.s[j], d.s[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vus = spu_rlmaska(a.vus, b.vs);
    VERIFY_EQUAL(d, e);
  }

  /* word */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vi = spu_rlmaska(a.vi, b.vi);
    for (j=0; j<4; j++) {
      signed int exp;
      count = (int)b.i[j] & 0x3F;
      if (count > 32) {
	count = -count & 0x1F;
	exp = a.i[j] >> count;
      } else {
	if (count) {
	  exp = (a.i[j] < 0) ? -1 : 0;
	} else {
	  exp = a.i[j];
	}
      }
      if (exp != d.i[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tint element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.i[j], b.i[j], d.i[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vui = spu_rlmaska(a.vui, b.vi);
    VERIFY_EQUAL(d, e);
  }


  /* Immediate forms */
  /* halfword */
  a.vui = rand_vui();
  d.vs = spu_rlmaska(a.vs, 0);
  for (j=0; j<8; j++) {
    signed short exp;
    exp = a.s[j];
    if (exp != d.s[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tshort (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=0 d=0x%x exp=0x%x\n", a.s[j], d.s[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vus = spu_rlmaska(a.vus, 0);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vs = spu_rlmaska(a.vs, -16);
  for (j=0; j<8; j++) {
    signed short exp;
    exp = (a.us[j] & 0x8000) ? -1 : 0;
    if (exp != d.s[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tshort (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=-16 d=0x%x exp=0x%x\n", a.s[j], d.s[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vus = spu_rlmaska(a.vus, -16);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vs = spu_rlmaska(a.vs, -7);
  for (j=0; j<8; j++) {
    signed short exp;
    exp = a.s[j] >> 7;
    if (exp != d.s[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tshort (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=-7 d=0x%x exp=0x%x\n", a.s[j], d.s[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vus = spu_rlmaska(a.vus, -7);
  VERIFY_EQUAL(d, e);


  /* word */
  a.vui = rand_vui();
  d.vi = spu_rlmaska(a.vi, 0);
  for (j=0; j<4; j++) {
    signed int exp;
    exp = a.i[j];
    if (exp != d.i[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tint (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=0 d=0x%x exp=0x%x\n", a.i[j], d.i[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vui = spu_rlmaska(a.vui, 0);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vi = spu_rlmaska(a.vi, -32);
  for (j=0; j<4; j++) {
    signed int exp;
    exp = (a.i[j] < 0) ? -1 : 0;
    if (exp != d.i[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tint (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=-32 d=0x%x exp=0x%x\n", a.i[j], d.i[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vui = spu_rlmaska(a.vui, -32);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vi = spu_rlmaska(a.vi, -13);
  for (j=0; j<4; j++) {
    signed int exp;
    exp = a.i[j] >> 13;
    if (exp != d.i[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tint (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=-13 d=0x%x exp=0x%x\n", a.i[j], d.i[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vui = spu_rlmaska(a.vui, -13);
  VERIFY_EQUAL(d, e);

  DEBUG(1, passed);
  return (0);
}


int test_spu_rlmaskqw()
{
  int i, j;
  int count, cnt;

  DEBUG(1, "rlmaskqw........");
  

  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    count = (int)rand_local();
    d.vui = spu_rlmaskqw(a.vui, count);

    cnt = -count & 7;

    for (j=0; j<4; j++) {
      unsigned int exp;
      
      exp = a.ui[j] >> cnt;
      if ((j > 0) && cnt) exp |= (a.ui[j-1] << (32-cnt));

      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "count=0x%x d=0x%x exp=0x%x\n", count, d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vuc = spu_rlmaskqw(a.vuc, count);
    VERIFY_EQUAL(d, e);
    e.vc = spu_rlmaskqw(a.vc, count);
    VERIFY_EQUAL(d, e);
    e.vus = spu_rlmaskqw(a.vus, count);
    VERIFY_EQUAL(d, e);
    e.vs = spu_rlmaskqw(a.vs, count);
    VERIFY_EQUAL(d, e);
    e.vi = spu_rlmaskqw(a.vi, count);
    VERIFY_EQUAL(d, e);
    e.vull = spu_rlmaskqw(a.vull, count);
    VERIFY_EQUAL(d, e);
    e.vll = spu_rlmaskqw(a.vll, count);
    VERIFY_EQUAL(d, e);
    e.vf = spu_rlmaskqw(a.vf, count);
    VERIFY_EQUAL(d, e);
    e.vd = spu_rlmaskqw(a.vd, count);
    VERIFY_EQUAL(d, e);
  }


  /* Immediate forms */
  a.vui = rand_vui();
  d.vui = spu_rlmaskqw(a.vui, -5);
  cnt = 5;

  for (j=0; j<4; j++) {
    unsigned int exp;
      
    exp = a.ui[j] >> cnt;
    if ((j > 0) && cnt) exp |= (a.ui[j-1] << (32-cnt));

    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "count=-5 d=0x%x exp=0x%x\n", d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vuc = spu_rlmaskqw(a.vuc, -5);
  VERIFY_EQUAL(d, e);
  e.vc = spu_rlmaskqw(a.vc, -5);
  VERIFY_EQUAL(d, e);
  e.vus = spu_rlmaskqw(a.vus, -5);
  VERIFY_EQUAL(d, e);
  e.vs = spu_rlmaskqw(a.vs, -5);
  VERIFY_EQUAL(d, e);
  e.vi = spu_rlmaskqw(a.vi, -5);
  VERIFY_EQUAL(d, e);
  e.vull = spu_rlmaskqw(a.vull, -5);
  VERIFY_EQUAL(d, e);
  e.vll = spu_rlmaskqw(a.vll, -5);
  VERIFY_EQUAL(d, e);
  e.vf = spu_rlmaskqw(a.vf, -5);
  VERIFY_EQUAL(d, e);
  e.vd = spu_rlmaskqw(a.vd, -5);
  VERIFY_EQUAL(d, e);

  DEBUG(1, passed);
  return (0);
}


int test_spu_rlmaskqwbyte()
{
  int i, j;
  int count, cnt;

  DEBUG(1, "rlmaskqwbyte....");
  

  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    count = (int)rand_local();
    d.vuc = spu_rlmaskqwbyte(a.vuc, count);
    cnt = count & 0x1F;
    if (cnt) cnt |= 0xFFFFFFE0;
    for (j=0; j<16; j++) {
      unsigned char exp;
      exp = 0;
      if ((j + cnt) >= 0) exp = a.uc[j+cnt];
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(1, "count=0x%x d=0x%x exp=0x%x\n", count, d.uc[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vc = spu_rlmaskqwbyte(a.vc, count);
    VERIFY_EQUAL(d, e);
    e.vus = spu_rlmaskqwbyte(a.vus, count);
    VERIFY_EQUAL(d, e);
    e.vs = spu_rlmaskqwbyte(a.vs, count);
    VERIFY_EQUAL(d, e);
    e.vui = spu_rlmaskqwbyte(a.vui, count);
    VERIFY_EQUAL(d, e);
    e.vi = spu_rlmaskqwbyte(a.vi, count);
    VERIFY_EQUAL(d, e);
    e.vull = spu_rlmaskqwbyte(a.vull, count);
    VERIFY_EQUAL(d, e);
    e.vll = spu_rlmaskqwbyte(a.vll, count);
    VERIFY_EQUAL(d, e);
    e.vf = spu_rlmaskqwbyte(a.vf, count);
    VERIFY_EQUAL(d, e);
    e.vd = spu_rlmaskqwbyte(a.vd, count);
    VERIFY_EQUAL(d, e);
  }


  /* Immediate forms */
  a.vui = rand_vui();
  d.vuc = spu_rlmaskqwbyte(a.vuc, -5);

  for (j=0; j<16; j++) {
    unsigned char exp;
    exp = (j >= 5) ? a.uc[j-5] : 0;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(1, "count=-5 d=0x%x exp=0x%x\n", d.uc[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vc = spu_rlmaskqwbyte(a.vc, -5);
  VERIFY_EQUAL(d, e);
  e.vus = spu_rlmaskqwbyte(a.vus, -5);
  VERIFY_EQUAL(d, e);
  e.vs = spu_rlmaskqwbyte(a.vs, -5);
  VERIFY_EQUAL(d, e);
  e.vui = spu_rlmaskqwbyte(a.vui, -5);
  VERIFY_EQUAL(d, e);
  e.vi = spu_rlmaskqwbyte(a.vi, -5);
  VERIFY_EQUAL(d, e);
  e.vull = spu_rlmaskqwbyte(a.vull, -5);
  VERIFY_EQUAL(d, e);
  e.vll = spu_rlmaskqwbyte(a.vll, -5);
  VERIFY_EQUAL(d, e);
  e.vf = spu_rlmaskqwbyte(a.vf, -5);
  VERIFY_EQUAL(d, e);
  e.vd = spu_rlmaskqwbyte(a.vd, -5);
  VERIFY_EQUAL(d, e);

  DEBUG(1, passed);
  return (0);
}



int test_spu_rlmaskqwbytebc()
{
  int i, j;
  int count, cnt;

  DEBUG(1, "rlmaskqwbytebc..");
  

  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    count = (int)rand_local();
    d.vuc = spu_rlmaskqwbytebc(a.vuc, count);
    cnt = (count >> 3) & 0x1F;
    if (cnt) cnt |= 0xFFFFFFE0;
    for (j=0; j<16; j++) {
      unsigned char exp;
      exp = 0;
      if ((j + cnt) >= 0) exp = a.uc[j+cnt];
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "count=0x%x d=0x%x exp=0x%x\n", count, d.uc[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vc = spu_rlmaskqwbytebc(a.vc, count);
    VERIFY_EQUAL(d, e);
    e.vus = spu_rlmaskqwbytebc(a.vus, count);
    VERIFY_EQUAL(d, e);
    e.vs = spu_rlmaskqwbytebc(a.vs, count);
    VERIFY_EQUAL(d, e);
    e.vui = spu_rlmaskqwbytebc(a.vui, count);
    VERIFY_EQUAL(d, e);
    e.vi = spu_rlmaskqwbytebc(a.vi, count);
    VERIFY_EQUAL(d, e);
    e.vull = spu_rlmaskqwbytebc(a.vull, count);
    VERIFY_EQUAL(d, e);
    e.vll = spu_rlmaskqwbytebc(a.vll, count);
    VERIFY_EQUAL(d, e);
    e.vf = spu_rlmaskqwbytebc(a.vf, count);
    VERIFY_EQUAL(d, e);
    e.vd = spu_rlmaskqwbytebc(a.vd, count);
    VERIFY_EQUAL(d, e);
  }
  DEBUG(1, passed);
  return (0);
}



int test_spu_rlqw()
{
  int i, j;
  int count, cnt;

  DEBUG(1, "rlqw............");
  

  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    count = (int)rand_local();
    d.vui = spu_rlqw(a.vui, count);

    cnt = count & 7;

    for (j=0; j<4; j++) {
      unsigned int exp;
      
      exp = a.ui[j] << cnt;
      if (cnt) exp |= (a.ui[(j+1)&3] >> (32-cnt));

      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "count=0x%x d=0x%x exp=0x%x\n", count, d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vuc = spu_rlqw(a.vuc, count);
    VERIFY_EQUAL(d, e);
    e.vc = spu_rlqw(a.vc, count);
    VERIFY_EQUAL(d, e);
    e.vus = spu_rlqw(a.vus, count);
    VERIFY_EQUAL(d, e);
    e.vs = spu_rlqw(a.vs, count);
    VERIFY_EQUAL(d, e);
    e.vi = spu_rlqw(a.vi, count);
    VERIFY_EQUAL(d, e);
    e.vull = spu_rlqw(a.vull, count);
    VERIFY_EQUAL(d, e);
    e.vll = spu_rlqw(a.vll, count);
    VERIFY_EQUAL(d, e);
    e.vf = spu_rlqw(a.vf, count);
    VERIFY_EQUAL(d, e);
    e.vd = spu_rlqw(a.vd, count);
    VERIFY_EQUAL(d, e);
  }


  /* Immediate forms */
  a.vui = rand_vui();
  d.vui = spu_rlqw(a.vui, 27);
  cnt = 3;

  for (j=0; j<4; j++) {
    unsigned int exp;
      
    exp = a.ui[j] << cnt;
    if (cnt) exp |= (a.ui[(j+1)&3] >> (32-cnt));

    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\t(lit) element=%d ", j);
      DEBUG(2, "count=27 d=0x%x exp=0x%x\n", d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vuc = spu_rlqw(a.vuc, 27);
  VERIFY_EQUAL(d, e);
  e.vc = spu_rlqw(a.vc, 27);
  VERIFY_EQUAL(d, e);
  e.vus = spu_rlqw(a.vus, 27);
  VERIFY_EQUAL(d, e);
  e.vs = spu_rlqw(a.vs, 27);
  VERIFY_EQUAL(d, e);
  e.vi = spu_rlqw(a.vi, 27);
  VERIFY_EQUAL(d, e);
  e.vull = spu_rlqw(a.vull, 27);
  VERIFY_EQUAL(d, e);
  e.vll = spu_rlqw(a.vll, 27);
  VERIFY_EQUAL(d, e);
  e.vf = spu_rlqw(a.vf, 27);
  VERIFY_EQUAL(d, e);
  e.vd = spu_rlqw(a.vd, 27);
  VERIFY_EQUAL(d, e);

  DEBUG(1, passed);
  return (0);
}


int test_spu_rlqwbyte()
{
  int i, j;
  int count, cnt;

  DEBUG(1, "rlqwbyte........");


  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    count = (int)rand_local();
    d.vuc = spu_rlqwbyte(a.vuc, count);
    cnt = count & 15;
    for (j=0; j<16; j++) {
      unsigned char exp;

      exp = a.uc[(j+cnt) & 15];
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "count=0x%x d=0x%x exp=0x%x\n", count, d.uc[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vc = spu_rlqwbyte(a.vc, count);
    VERIFY_EQUAL(d, e);
    e.vus = spu_rlqwbyte(a.vus, count);
    VERIFY_EQUAL(d, e);
    e.vs = spu_rlqwbyte(a.vs, count);
    VERIFY_EQUAL(d, e);
    e.vui = spu_rlqwbyte(a.vui, count);
    VERIFY_EQUAL(d, e);
    e.vi = spu_rlqwbyte(a.vi, count);
    VERIFY_EQUAL(d, e);
    e.vull = spu_rlqwbyte(a.vull, count);
    VERIFY_EQUAL(d, e);
    e.vll = spu_rlqwbyte(a.vll, count);
    VERIFY_EQUAL(d, e);
    e.vf = spu_rlqwbyte(a.vf, count);
    VERIFY_EQUAL(d, e);
    e.vd = spu_rlqwbyte(a.vd, count);
    VERIFY_EQUAL(d, e);
  }


  /* Immediate forms */
  a.vui = rand_vui();
  d.vui = spu_rlqwbyte(a.vui, 27);

  for (j=0; j<16; j++) {
    unsigned char exp;
    exp = a.uc[(j+27)&15];
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "count=27 d=0x%x exp=0x%x\n", d.uc[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vc = spu_rlqwbyte(a.vc, 27);
  VERIFY_EQUAL(d, e);
  e.vus = spu_rlqwbyte(a.vus, 27);
  VERIFY_EQUAL(d, e);
  e.vs = spu_rlqwbyte(a.vs, 27);
  VERIFY_EQUAL(d, e);
  e.vui = spu_rlqwbyte(a.vui, 27);
  VERIFY_EQUAL(d, e);
  e.vi = spu_rlqwbyte(a.vi, 27);
  VERIFY_EQUAL(d, e);
  e.vull = spu_rlqwbyte(a.vull, 27);
  VERIFY_EQUAL(d, e);
  e.vll = spu_rlqwbyte(a.vll, 27);
  VERIFY_EQUAL(d, e);
  e.vf = spu_rlqwbyte(a.vf, 27);
  VERIFY_EQUAL(d, e);
  e.vd = spu_rlqwbyte(a.vd, 27);
  VERIFY_EQUAL(d, e);

  DEBUG(1, passed);
  return (0);
}

int test_spu_rlqwbytebc()
{
  int i, j;
  int count, cnt;

  DEBUG(1, "rlqwbytebc......");
  

  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    count = (int)rand_local();
    d.vuc = spu_rlqwbytebc(a.vuc, count);
    cnt = (count >> 3) & 15;
    for (j=0; j<16; j++) {
      unsigned char exp;

      exp = a.uc[(j+cnt) & 15];
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "count=0x%x d=0x%x exp=0x%x\n", count, d.uc[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vc = spu_rlqwbytebc(a.vc, count);
    VERIFY_EQUAL(d, e);
    e.vus = spu_rlqwbytebc(a.vus, count);
    VERIFY_EQUAL(d, e);
    e.vs = spu_rlqwbytebc(a.vs, count);
    VERIFY_EQUAL(d, e);
    e.vui = spu_rlqwbytebc(a.vui, count);
    VERIFY_EQUAL(d, e);
    e.vi = spu_rlqwbytebc(a.vi, count);
    VERIFY_EQUAL(d, e);
    e.vull = spu_rlqwbytebc(a.vull, count);
    VERIFY_EQUAL(d, e);
    e.vll = spu_rlqwbytebc(a.vll, count);
    VERIFY_EQUAL(d, e);
    e.vf = spu_rlqwbytebc(a.vf, count);
    VERIFY_EQUAL(d, e);
    e.vd = spu_rlqwbytebc(a.vd, count);
    VERIFY_EQUAL(d, e);
  }

  DEBUG(1, passed);
  return (0);
}


int test_spu_roundtf()
{
  int i, j;

  DEBUG(1, "roundtf.........");
  
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-1024.0, 1024.0);
    d.vf = spu_roundtf(a.vd);
    for (j=0; j<4; j++) {
      float exp;
      exp = (j & 1) ? 0.0f : (float)(a.d[j>>1]);
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f d=%f exp=%f\n", a.d[j>>1], d.f[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_rsqrte()
{
  int i, j;

  DEBUG(1, "rsqrte..........");

  for (i=0; i<samples; i++) {
    a.vf = rand_vf(1.0f, 3.0f);
    d.vf = spu_rsqrte(a.vf);
    for (j=0; j<4; j++) {
      float err;
      err = a.f[j] * d.f[j] * d.f[j];
      if ((err > 1.0004f) || (err < 0.9996)) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "a=%f d=%f err=%f\n", a.f[j], d.f[j], err);
	return (1);
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}

int test_spu_sel()
{
  int i, j;

  DEBUG(1, "sel.............");

  /* unsigned int */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();
    d.vui = spu_sel(a.vui, b.vui, c.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = (a.ui[j]& ~c.ui[j]) | (b.ui[j] & c.ui[j]);
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\telement=%d a=0x%x ", j, a.ui[j]);
	DEBUG(2, "b=0x%x c=0x%x d=0x%x exp=0x%x\n", b.ui[j], c.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify all equivalent variations */
    e.vuc = spu_sel(a.vuc, b.vuc, c.vuc);
    VERIFY_EQUAL(d, e);
    e.vc = spu_sel(a.vc, b.vc, c.vuc);
    VERIFY_EQUAL(d, e);
    e.vus = spu_sel(a.vus, b.vus, c.vus);
    VERIFY_EQUAL(d, e);
    e.vs = spu_sel(a.vs, b.vs, c.vus);
    VERIFY_EQUAL(d, e);
    e.vi = spu_sel(a.vi, b.vi, c.vui);
    VERIFY_EQUAL(d, e);
    e.vf = spu_sel(a.vf, b.vf, c.vui);
    VERIFY_EQUAL(d, e);
    e.vull = spu_sel(a.vull, b.vull, c.vull);
    VERIFY_EQUAL(d, e);
    e.vll = spu_sel(a.vll, b.vll, c.vull);
    VERIFY_EQUAL(d, e);
    e.vd = spu_sel(a.vd, b.vd, c.vull);
    VERIFY_EQUAL(d, e);
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_shuffle()
{
  int i, j;

  DEBUG(1, "shuffle.........");

  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();
    d.vuc = spu_shuffle(a.vuc, b.vuc, c.vuc);
    for (j=0; j<16; j++) {
      unsigned int index;
      unsigned char exp;

      index = c.uc[j];
      if (index & 0x80) {
	if (index & 0x40) {
	  exp = (index & 0x20) ? 0x80 : 0xFF;
	} else {
	  exp = 0;
	}
      } else {
	if (index & 0x10) {
	  exp = b.uc[index & 0xF];
	} else {
	  exp = a.uc[index & 0xF];
	}
      }
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.uc[j], exp);
	return (1);
      }
    }
    /* verify all equivalent variations */
    e.vc = spu_shuffle(a.vc, b.vc, c.vuc);
    VERIFY_EQUAL(d, e);
    e.vus = spu_shuffle(a.vus, b.vus, c.vuc);
    VERIFY_EQUAL(d, e);
    e.vs = spu_shuffle(a.vs, b.vs, c.vuc);
    VERIFY_EQUAL(d, e);
    e.vui = spu_shuffle(a.vui, b.vui, c.vuc);
    VERIFY_EQUAL(d, e);
    e.vi = spu_shuffle(a.vi, b.vi, c.vuc);
    VERIFY_EQUAL(d, e);
    e.vf = spu_shuffle(a.vf, b.vf, c.vuc);
    VERIFY_EQUAL(d, e);
    e.vull = spu_shuffle(a.vull, b.vull, c.vuc);
    VERIFY_EQUAL(d, e);
    e.vll = spu_shuffle(a.vll, b.vll, c.vuc);
    VERIFY_EQUAL(d, e);
    e.vd = spu_shuffle(a.vd, b.vd, c.vuc);
    VERIFY_EQUAL(d, e);
  }
  DEBUG(1, passed);
  return (0);
}


int test_spu_sl()
{
  int i, j;
  int count;

  DEBUG(1, "sl..............");
  
  /* halfword */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vus = spu_sl(a.vus, b.vus);
    for (j=0; j<8; j++) {
      unsigned short exp;
      count = (int)b.us[j] & 0x1F;
      exp = (count > 15) ? 0 : (a.us[j] << count);
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tushort element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.us[j], b.us[j], d.us[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vs = spu_sl(a.vs, b.vus);
    VERIFY_EQUAL(d, e);
  }

  /* word */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_sl(a.vui, b.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      count = (int)b.ui[j] & 0x3F;
      exp = (count > 31) ? 0 : (a.ui[j] << count);
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_sl(a.vi, b.vui);
    VERIFY_EQUAL(d, e);
  }


  /* Immediate forms */
  /* halfword */
  a.vui = rand_vui();
  d.vus = spu_sl(a.vus, 0x0);
  for (j=0; j<8; j++) {
    unsigned short exp;
    exp = a.us[j];
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=0 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vs = spu_sl(a.vs, 0x0);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vus = spu_sl(a.vus, 0xF);
  for (j=0; j<8; j++) {
    unsigned short exp;
    exp = a.us[j] << 15;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=15 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vs = spu_sl(a.vs, 0xF);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vus = spu_sl(a.vus, 41);
  for (j=0; j<8; j++) {
    unsigned short exp;
    exp = a.us[j] << 9;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=9 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vs = spu_sl(a.vs, 41);
  VERIFY_EQUAL(d, e);


  /* word */
  a.vui = rand_vui();
  d.vui = spu_sl(a.vui, 0x0);
  for (j=0; j<4; j++) {
    unsigned int exp;
    exp = a.ui[j];
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=0 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vi = spu_sl(a.vi, 0x0);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vui = spu_sl(a.vui, 0x1F);
  for (j=0; j<4; j++) {
    unsigned int exp;
    exp = a.ui[j] << 31;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=31 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vi = spu_sl(a.vi, 0x1F);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vui = spu_sl(a.vui, 71);
  for (j=0; j<4; j++) {
    unsigned int exp;
    exp = a.ui[j] << 7;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint (lit) element=%d ", j);
      DEBUG(2, "a=0x%x b=7 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vi = spu_sl(a.vi, 71);
  VERIFY_EQUAL(d, e);

  DEBUG(1, passed);
  return (0);
}


int test_spu_slqw()
{
  int i, j;
  unsigned int count, cnt;

  DEBUG(1, "slqw............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    count = (unsigned int)rand_local();
    d.vui = spu_slqw(a.vui, count);

    cnt = count & 7;

    for (j=0; j<4; j++) {
      unsigned int exp;
      
      exp = a.ui[j] << cnt;
      if (cnt && j<3) exp |= (a.ui[j+1] >> (32-cnt));

      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "count=0x%x d=0x%x exp=0x%x\n", count, d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vuc = spu_slqw(a.vuc, count);
    VERIFY_EQUAL(d, e);
    e.vc = spu_slqw(a.vc, count);
    VERIFY_EQUAL(d, e);
    e.vus = spu_slqw(a.vus, count);
    VERIFY_EQUAL(d, e);
    e.vs = spu_slqw(a.vs, count);
    VERIFY_EQUAL(d, e);
    e.vi = spu_slqw(a.vi, count);
    VERIFY_EQUAL(d, e);
    e.vull = spu_slqw(a.vull, count);
    VERIFY_EQUAL(d, e);
    e.vll = spu_slqw(a.vll, count);
    VERIFY_EQUAL(d, e);
    e.vf = spu_slqw(a.vf, count);
    VERIFY_EQUAL(d, e);
    e.vd = spu_slqw(a.vd, count);
    VERIFY_EQUAL(d, e);
  }


  /* Immediate forms */
  a.vui = rand_vui();
  d.vui = spu_slqw(a.vui, 27);
  cnt = 3;

  for (j=0; j<4; j++) {
    unsigned int exp;
      
    exp = a.ui[j] << cnt;
    if (cnt && (j<3)) exp |= (a.ui[j+1] >> (32-cnt));

    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "count=27 d=0x%x exp=0x%x\n", d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vuc = spu_slqw(a.vuc, 27);
  VERIFY_EQUAL(d, e);
  e.vc = spu_slqw(a.vc, 27);
  VERIFY_EQUAL(d, e);
  e.vus = spu_slqw(a.vus, 27);
  VERIFY_EQUAL(d, e);
  e.vs = spu_slqw(a.vs, 27);
  VERIFY_EQUAL(d, e);
  e.vi = spu_slqw(a.vi, 27);
  VERIFY_EQUAL(d, e);
  e.vull = spu_slqw(a.vull, 27);
  VERIFY_EQUAL(d, e);
  e.vll = spu_slqw(a.vll, 27);
  VERIFY_EQUAL(d, e);
  e.vf = spu_slqw(a.vf, 27);
  VERIFY_EQUAL(d, e);
  e.vd = spu_slqw(a.vd, 27);
  VERIFY_EQUAL(d, e);

  DEBUG(1, passed);
  return (0);
}


int test_spu_slqwbyte()
{
  int i, j;
  unsigned int count, cnt;

  DEBUG(1, "slqwbyte........");
  

  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    count = (unsigned int)rand_local();
    d.vuc = spu_slqwbyte(a.vuc, count);
    cnt = count & 0x1F;
    for (j=0; j<16; j++) {
      unsigned char exp;

      exp = ((j+cnt) > 15) ? 0 : a.uc[j+cnt];
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "count=0x%x d=0x%x exp=0x%x\n", count, d.uc[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vc = spu_slqwbyte(a.vc, count);
    VERIFY_EQUAL(d, e);
    e.vus = spu_slqwbyte(a.vus, count);
    VERIFY_EQUAL(d, e);
    e.vs = spu_slqwbyte(a.vs, count);
    VERIFY_EQUAL(d, e);
    e.vui = spu_slqwbyte(a.vui, count);
    VERIFY_EQUAL(d, e);
    e.vi = spu_slqwbyte(a.vi, count);
    VERIFY_EQUAL(d, e);
    e.vull = spu_slqwbyte(a.vull, count);
    VERIFY_EQUAL(d, e);
    e.vll = spu_slqwbyte(a.vll, count);
    VERIFY_EQUAL(d, e);
    e.vf = spu_slqwbyte(a.vf, count);
    VERIFY_EQUAL(d, e);
    e.vd = spu_slqwbyte(a.vd, count);
    VERIFY_EQUAL(d, e);
  }


  /* Immediate forms */
  a.vui = rand_vui();
  d.vui = spu_slqwbyte(a.vui, 35);

  for (j=0; j<16; j++) {
    unsigned char exp;
    exp = ((j+3) > 15) ? 0 : a.uc[j+3];
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\telement=%d ", j);
      DEBUG(2, "count=35 d=0x%x exp=0x%x\n", d.uc[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vc = spu_slqwbyte(a.vc, 35);
  VERIFY_EQUAL(d, e);
  e.vus = spu_slqwbyte(a.vus, 35);
  VERIFY_EQUAL(d, e);
  e.vs = spu_slqwbyte(a.vs, 35);
  VERIFY_EQUAL(d, e);
  e.vui = spu_slqwbyte(a.vui, 35);
  VERIFY_EQUAL(d, e);
  e.vi = spu_slqwbyte(a.vi, 35);
  VERIFY_EQUAL(d, e);
  e.vull = spu_slqwbyte(a.vull, 35);
  VERIFY_EQUAL(d, e);
  e.vll = spu_slqwbyte(a.vll, 35);
  VERIFY_EQUAL(d, e);
  e.vf = spu_slqwbyte(a.vf, 35);
  VERIFY_EQUAL(d, e);
  e.vd = spu_slqwbyte(a.vd, 35);
  VERIFY_EQUAL(d, e);

  DEBUG(1, passed);
  return (0);
}


int test_spu_slqwbytebc()
{
  int i, j;
  unsigned int count, cnt;

  DEBUG(1, "slqwbytebc......");
  

  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    count = (unsigned int)rand_local();
    d.vuc = spu_slqwbytebc(a.vuc, count);
    cnt = (count >> 3) & 0x1F;
    for (j=0; j<16; j++) {
      unsigned char exp;

      exp = ((j+cnt) > 15) ? 0 : a.uc[j+cnt];
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "count=0x%x d=0x%x exp=0x%x\n", count, d.uc[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vc = spu_slqwbytebc(a.vc, count);
    VERIFY_EQUAL(d, e);
    e.vus = spu_slqwbytebc(a.vus, count);
    VERIFY_EQUAL(d, e);
    e.vs = spu_slqwbytebc(a.vs, count);
    VERIFY_EQUAL(d, e);
    e.vui = spu_slqwbytebc(a.vui, count);
    VERIFY_EQUAL(d, e);
    e.vi = spu_slqwbytebc(a.vi, count);
    VERIFY_EQUAL(d, e);
    e.vull = spu_slqwbytebc(a.vull, count);
    VERIFY_EQUAL(d, e);
    e.vll = spu_slqwbytebc(a.vll, count);
    VERIFY_EQUAL(d, e);
    e.vf = spu_slqwbytebc(a.vf, count);
    VERIFY_EQUAL(d, e);
    e.vd = spu_slqwbytebc(a.vd, count);
    VERIFY_EQUAL(d, e);
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_splats()
{
  int i, j;

  DEBUG(1, "splats..........");
  
  /* byte */
  for (i=0; i<samples; i++) {
    unsigned char a, exp;
    a = (unsigned char)rand_local();
    d.vuc = spu_splats(a);
    exp = a;
    for (j=0; j<16; j++) {
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuchar: element=%d ", j);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.uc[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vc = spu_splats((signed char)(a));
    VERIFY_EQUAL(d, e);
  }

  /* halfword */
  for (i=0; i<samples; i++) {
    unsigned short a, exp;
    a = (unsigned short)rand_local();
    d.vus = spu_splats(a);
    exp = a;
    for (j=0; j<8; j++) {
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tushort: element=%d ", j);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.us[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vs = spu_splats((signed short)(a));
    VERIFY_EQUAL(d, e);
  }

  /* word */
  for (i=0; i<samples; i++) {
    unsigned int a, exp;
    a = (unsigned int)rand_local();
    d.vui = spu_splats(a);
    exp = a;
    for (j=0; j<4; j++) {
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", j);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_splats((signed int)(a));
    VERIFY_EQUAL(d, e);
  }

  /* float */
  for (i=0; i<samples; i++) {
    float a, exp;
    a = rand_float(-1.0f, 1.0f);
    d.vf = spu_splats(a);
    exp = a;
    for (j=0; j<4; j++) {
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tfloat: element=%d ", j);
	DEBUG(2, "d=%f exp=%f\n", d.f[j], exp);
	return (1);
      }
    }
  }

  /* double word */
  for (i=0; i<samples; i++) {
    unsigned long long a, exp;
    a = (unsigned long long)rand_local() | ((unsigned long long)rand_local << 32);
    d.vull = spu_splats(a);
    exp = a;
    for (j=0; j<2; j++) {
      if (exp != d.ull[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tullong: element=%d ", j);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vll = spu_splats((signed long long)(a));
    VERIFY_EQUAL(d, e);
  }

  /* double */
  for (i=0; i<samples; i++) {
    double a, exp;
    a = rand_double(-1.0, 1.0);
    d.vd = spu_splats(a);
    exp = a;
    for (j=0; j<2; j++) {
      if (exp != d.d[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tdouble: element=%d ", j);
	DEBUG(2, "d=%f exp=%f\n", d.d[j], exp);
	return (1);
      }
    }
  }

  /* immediate forms */
  /* byte */
  {
    d.vuc = spu_splats((unsigned char)(240));
    for (j=0; j<16; j++) {
      if (240 != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuchar (lit): element=%d ", j);
	DEBUG(2, "d=%d exp=240\n", d.uc[j]);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vc = spu_splats((signed char)(-16));
    VERIFY_EQUAL(d, e);
  }

  /* short */
  {
    d.vus = spu_splats((unsigned short)(0x8234));
    for (j=0; j<8; j++) {
      if (0x8234 != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tushort (lit): element=%d ", j);
	DEBUG(2, "d=0x%x exp=0x8234\n", d.us[j]);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vs = spu_splats((signed short)(-32204));
    VERIFY_EQUAL(d, e);
  }

  /* int */
  {
    d.vui = spu_splats((unsigned int)(0xFFFF1234));
    for (j=0; j<4; j++) {
      if (0xFFFF1234 != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint (lit): element=%d ", j);
	DEBUG(2, "d=0x%x exp=0xFFFF1234\n", d.ui[j]);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_splats((signed int)(-60876));
    VERIFY_EQUAL(d, e);
  }

  /* float */
  {
    d.vf = spu_splats(-1.0f);
    for (j=0; j<4; j++) {
      if (-1.0f != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint (lit): element=%d ", j);
	DEBUG(2, "d=%d exp=-1.0\n", d.ui[j]);
	return (1);
      }
    }
  }

  /* double word */
  {
    d.vull = spu_splats((unsigned long long)(0x1234ULL));
    for (j=0; j<2; j++) {
      if (0x1234ULL != d.ull[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tullong (lit): element=%d ", j);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vll = spu_splats((signed long long)(0x1234LL));
    VERIFY_EQUAL(d, e);
  }

  /* double */
  {
    d.vd = spu_splats((double)(-2.0));
    for (j=0; j<2; j++) {
      if (-2.0 != d.d[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tdouble (lit): element=%d ", j);
	DEBUG(2, "d=%f exp=-2.0\n", d.d[j]);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_stop()
{
  DEBUG(1, "stop............untested\n");
  return (0);
}


int test_spu_sub()
{
  int i, j;

  DEBUG(1, "sub.............");
  
  /* word */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_sub(a.vui, b.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = a.ui[j] - b.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_sub(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
  }

  /* halfword */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vus = spu_sub(a.vus, b.vus);
    for (j=0; j<8; j++) {
      unsigned short exp;
      exp = a.us[j] - b.us[j];
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tushort: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.us[j], b.us[j], d.us[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vs = spu_sub(a.vs, b.vs);
    VERIFY_EQUAL(d, e);
  }

  /* float */
  for (i=0; i<samples; i++) {
    a.vf = rand_vf(-4.0f, 4.0f);
    b.vf = rand_vf(-4.0f, 4.0f);
    d.vf = spu_sub(a.vf, b.vf);
    for (j=0; j<4; j++) {
      float exp;
      exp = a.f[j] - b.f[j];
      if (exp != d.f[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\float: element=%d ", j);
	DEBUG(2, "a=%f b=%f d=%f exp=%f\n", a.f[j], b.f[j], d.f[j], exp);
	return (1);
      }
    }
  }

  /* double */
  for (i=0; i<samples; i++) {
    a.vd = rand_vd(-4.0, 4.0);
    b.vd = rand_vd(-4.0, 4.0);
    d.vd = spu_sub(a.vd, b.vd);
    for (j=0; j<2; j++) {
      double exp;
      exp = a.d[j] - b.d[j];
      if (exp != d.d[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tdouble: element=%d ", j);
	DEBUG(2, "a=%f b=%f d=%f exp=%f\n", a.d[j], b.d[j], d.d[j], exp);
	return (1);
      }
    }
  }

  /* immediate forms */

  /* short */
  for (i=0; i<samples; i++) {
    unsigned short imm;
    a.vui = rand_vui();
    imm   = (unsigned short)rand_local();
    d.vus = spu_sub(imm, a.vus);
    for (j=0; j<8; j++) {
      unsigned short exp;
      exp = imm - a.us[j];
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tushort immediate: element=%d ", j);
	DEBUG(2, "a=0x%x imm=0x%x d=0x%x exp=0x%x\n", a.us[j], imm, d.us[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vs = spu_sub((short)imm, a.vs);
    VERIFY_EQUAL(d, e);
  }
    
  /* int */
  for (i=0; i<samples; i++) {
    unsigned int imm;
    a.vui = rand_vui();
    imm   = rand_local();
    d.vui = spu_sub(imm, a.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = imm - a.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint immediate: element=%d ", j);
	DEBUG(2, "a=0x%x imm=0x%x d=0x%x exp=0x%x\n", a.ui[j], imm, d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_sub((int)imm, a.vi);
    VERIFY_EQUAL(d, e);
  }
    
  /* immmediate literals 10-bit
   */
  a.vui = rand_vui();
  d.vus = spu_sub(511, a.vus);
  for (j=0; j<8; j++) {
    unsigned short exp;
    exp = 511 - a.us[j];
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=511 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vs = spu_sub(511, a.vs);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vui = spu_sub(-512, a.vui);
  for (j=0; j<4; j++) {
    unsigned int exp;
    exp = -512 - a.ui[j];
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=-512 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vi = spu_sub(-512, a.vi);
  VERIFY_EQUAL(d, e);

  DEBUG(1, passed);
  return (0);
}

int test_spu_subx()
{
  int i, j;

  DEBUG(1, "subx............");
  
  /* word */
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    c.vui = rand_vui();
    d.vui = spu_subx(a.vui, b.vui, c.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = a.ui[j] + ~b.ui[j] + (c.ui[j] & 1);
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d a=0x%x ", j, a.ui[j]);
	DEBUG(2, "b=0x%x c=0x%x d=0x%x exp=0x%x\n", b.ui[j], c.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_subx(a.vi, b.vi, c.vi);
    VERIFY_EQUAL(d, e);
  }
  DEBUG(1, passed);
  return (0);
}


int test_spu_sumb()
{
  int i, j;
  int idx;

  DEBUG(1, "sumb............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vus = spu_sumb(a.vuc, b.vuc);
    for (j=0; j<8; j++) {
      unsigned short exp;

      idx = 4 * (j>>1);
      if (j & 1) {
	exp = (unsigned short)a.uc[idx+0] + (unsigned short)a.uc[idx+1] +
	      (unsigned short)a.uc[idx+2] + (unsigned short)a.uc[idx+3];
      } else {
	exp = (unsigned short)b.uc[idx+0] + (unsigned short)b.uc[idx+1] +
	      (unsigned short)b.uc[idx+2] + (unsigned short)b.uc[idx+3];
      }
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\telement=%d ", j);
	DEBUG(2, "d=0x%x exp=0x%x\n", d.us[j], exp);
	return (1);
      }
    }
  }

  DEBUG(1, passed);
  return (0);
}

int test_spu_sync()
{
  DEBUG(1, "sync............");
  spu_sync();
  DEBUG(1, passed);
  return (0);
}


int test_spu_sync_c()
{
  DEBUG(1, "sync_c..........");
  spu_sync_c();
  DEBUG(1, passed);
  return (0);
}


#define	NEG_DENORM	1
#define	POS_DENORM	2
#define	NEG_ZERO	4
#define	POS_ZERO	8
#define NEG_INF		16
#define POS_INF		32
#define NAN		64

struct _testsv_samples {
  unsigned long long v;
  unsigned int sv;
} testsv_samples[] = {
  {0x0000000000000000ULL, POS_ZERO},
  {0x8000000000000000ULL, NEG_ZERO},
  {0x0000000000000001ULL, POS_DENORM},
  {0x0000000080000000ULL, POS_DENORM},
  {0x0000000100000000ULL, POS_DENORM},
  {0x0008000000000000ULL, POS_DENORM},
  {0x000FFFFFFFFFFFFFULL, POS_DENORM},
  {0x00000000FFF00000ULL, POS_DENORM},
  {0x8000000000000001ULL, NEG_DENORM},
  {0x8000000080000000ULL, NEG_DENORM},
  {0x8000000100000000ULL, NEG_DENORM},
  {0x8008000000000000ULL, NEG_DENORM},
  {0x800FFFFFFFFFFFFFULL, NEG_DENORM},
  {0x80000000FFF00000ULL, NEG_DENORM},
  {0x0010000000000000ULL, 0},
  {0x0010000000000001ULL, 0},
  {0x3FF0000000000000ULL, 0},
  {0x3FF00000FFF00000ULL, 0},
  {0xBFF0000000000000ULL, 0},
  {0xBFF00000FFF00000ULL, 0},
  {0x7FE0000000000000ULL, 0},
  {0x7FEFFFFFFFFFFFFFULL, 0},
  {0x8010000000000000ULL, 0},
  {0x8010000000000001ULL, 0},
  {0xFFE0000000000000ULL, 0},
  {0xFFEFFFFFFFFFFFFFULL, 0},
  {0x7FF0000000000000ULL, POS_INF},
  {0xFFF0000000000000ULL, NEG_INF},
  {0x7FF0000000000001ULL, NAN},
  {0x7FF0000080000000ULL, NAN},
  {0x7FF0000100000000ULL, NAN},
  {0x7FFFFFFFFFFFFFFFULL, NAN},
  {0xFFF0000000000001ULL, NAN},
  {0xFFF0000080000000ULL, NAN},
  {0xFFF0000100000000ULL, NAN},
  {0xFFFFFFFFFFFFFFFFULL, NAN}
};

int test_spu_testsv()
{
  int i, j, e[2];
  v128 d[128], a;
  unsigned long long exp;
  int cnt = sizeof(testsv_samples)/sizeof(struct _testsv_samples);

  DEBUG(1, "testsv..........");
  /* For each sample on element 0 */
  for (e[0]=0; e[0]<cnt; e[0]++) {

    /* For each sample on element 1 */
    for (e[1]=0; e[1]<cnt; e[1]++) {
      a.ull[0] = testsv_samples[e[0]].v;
      a.ull[1] = testsv_samples[e[1]].v;

      d[0].vull = spu_testsv(a.vd, 0);
      d[1].vull = spu_testsv(a.vd, 1);
      d[2].vull = spu_testsv(a.vd, 2);
      d[3].vull = spu_testsv(a.vd, 3);
      d[4].vull = spu_testsv(a.vd, 4);
      d[5].vull = spu_testsv(a.vd, 5);
      d[6].vull = spu_testsv(a.vd, 6);
      d[7].vull = spu_testsv(a.vd, 7);
      d[8].vull = spu_testsv(a.vd, 8);
      d[9].vull = spu_testsv(a.vd, 9);
      d[10].vull = spu_testsv(a.vd, 10);
      d[11].vull = spu_testsv(a.vd, 11);
      d[12].vull = spu_testsv(a.vd, 12);
      d[13].vull = spu_testsv(a.vd, 13);
      d[14].vull = spu_testsv(a.vd, 14);
      d[15].vull = spu_testsv(a.vd, 15);
      d[16].vull = spu_testsv(a.vd, 16);
      d[17].vull = spu_testsv(a.vd, 17);
      d[18].vull = spu_testsv(a.vd, 18);
      d[19].vull = spu_testsv(a.vd, 19);
      d[20].vull = spu_testsv(a.vd, 20);
      d[21].vull = spu_testsv(a.vd, 21);
      d[22].vull = spu_testsv(a.vd, 22);
      d[23].vull = spu_testsv(a.vd, 23);
      d[24].vull = spu_testsv(a.vd, 24);
      d[25].vull = spu_testsv(a.vd, 25);
      d[26].vull = spu_testsv(a.vd, 26);
      d[27].vull = spu_testsv(a.vd, 27);
      d[28].vull = spu_testsv(a.vd, 28);
      d[29].vull = spu_testsv(a.vd, 29);
      d[30].vull = spu_testsv(a.vd, 30);
      d[31].vull = spu_testsv(a.vd, 31);
      d[32].vull = spu_testsv(a.vd, 32);
      d[33].vull = spu_testsv(a.vd, 33);
      d[34].vull = spu_testsv(a.vd, 34);
      d[35].vull = spu_testsv(a.vd, 35);
      d[36].vull = spu_testsv(a.vd, 36);
      d[37].vull = spu_testsv(a.vd, 37);
      d[38].vull = spu_testsv(a.vd, 38);
      d[39].vull = spu_testsv(a.vd, 39);
      d[40].vull = spu_testsv(a.vd, 40);
      d[41].vull = spu_testsv(a.vd, 41);
      d[42].vull = spu_testsv(a.vd, 42);
      d[43].vull = spu_testsv(a.vd, 43);
      d[44].vull = spu_testsv(a.vd, 44);
      d[45].vull = spu_testsv(a.vd, 45);
      d[46].vull = spu_testsv(a.vd, 46);
      d[47].vull = spu_testsv(a.vd, 47);
      d[48].vull = spu_testsv(a.vd, 48);
      d[49].vull = spu_testsv(a.vd, 49);
      d[50].vull = spu_testsv(a.vd, 50);
      d[51].vull = spu_testsv(a.vd, 51);
      d[52].vull = spu_testsv(a.vd, 52);
      d[53].vull = spu_testsv(a.vd, 53);
      d[54].vull = spu_testsv(a.vd, 54);
      d[55].vull = spu_testsv(a.vd, 55);
      d[56].vull = spu_testsv(a.vd, 56);
      d[57].vull = spu_testsv(a.vd, 57);
      d[58].vull = spu_testsv(a.vd, 58);
      d[59].vull = spu_testsv(a.vd, 59);
      d[60].vull = spu_testsv(a.vd, 60);
      d[61].vull = spu_testsv(a.vd, 61);
      d[62].vull = spu_testsv(a.vd, 62);
      d[63].vull = spu_testsv(a.vd, 63);
      d[64].vull = spu_testsv(a.vd, 64);
      d[65].vull = spu_testsv(a.vd, 65);
      d[66].vull = spu_testsv(a.vd, 66);
      d[67].vull = spu_testsv(a.vd, 67);
      d[68].vull = spu_testsv(a.vd, 68);
      d[69].vull = spu_testsv(a.vd, 69);
      d[70].vull = spu_testsv(a.vd, 70);
      d[71].vull = spu_testsv(a.vd, 71);
      d[72].vull = spu_testsv(a.vd, 72);
      d[73].vull = spu_testsv(a.vd, 73);
      d[74].vull = spu_testsv(a.vd, 74);
      d[75].vull = spu_testsv(a.vd, 75);
      d[76].vull = spu_testsv(a.vd, 76);
      d[77].vull = spu_testsv(a.vd, 77);
      d[78].vull = spu_testsv(a.vd, 78);
      d[79].vull = spu_testsv(a.vd, 79);
      d[80].vull = spu_testsv(a.vd, 80);
      d[81].vull = spu_testsv(a.vd, 81);
      d[82].vull = spu_testsv(a.vd, 82);
      d[83].vull = spu_testsv(a.vd, 83);
      d[84].vull = spu_testsv(a.vd, 84);
      d[85].vull = spu_testsv(a.vd, 85);
      d[86].vull = spu_testsv(a.vd, 86);
      d[87].vull = spu_testsv(a.vd, 87);
      d[88].vull = spu_testsv(a.vd, 88);
      d[89].vull = spu_testsv(a.vd, 89);
      d[90].vull = spu_testsv(a.vd, 90);
      d[91].vull = spu_testsv(a.vd, 91);
      d[92].vull = spu_testsv(a.vd, 92);
      d[93].vull = spu_testsv(a.vd, 93);
      d[94].vull = spu_testsv(a.vd, 94);
      d[95].vull = spu_testsv(a.vd, 95);
      d[96].vull = spu_testsv(a.vd, 96);
      d[97].vull = spu_testsv(a.vd, 97);
      d[98].vull = spu_testsv(a.vd, 98);
      d[99].vull = spu_testsv(a.vd, 99);
      d[100].vull = spu_testsv(a.vd, 100);
      d[101].vull = spu_testsv(a.vd, 101);
      d[102].vull = spu_testsv(a.vd, 102);
      d[103].vull = spu_testsv(a.vd, 103);
      d[104].vull = spu_testsv(a.vd, 104);
      d[105].vull = spu_testsv(a.vd, 105);
      d[106].vull = spu_testsv(a.vd, 106);
      d[107].vull = spu_testsv(a.vd, 107);
      d[108].vull = spu_testsv(a.vd, 108);
      d[109].vull = spu_testsv(a.vd, 109);
      d[110].vull = spu_testsv(a.vd, 110);
      d[111].vull = spu_testsv(a.vd, 111);
      d[112].vull = spu_testsv(a.vd, 112);
      d[113].vull = spu_testsv(a.vd, 113);
      d[114].vull = spu_testsv(a.vd, 114);
      d[115].vull = spu_testsv(a.vd, 115);
      d[116].vull = spu_testsv(a.vd, 116);
      d[117].vull = spu_testsv(a.vd, 117);
      d[118].vull = spu_testsv(a.vd, 118);
      d[119].vull = spu_testsv(a.vd, 119);
      d[120].vull = spu_testsv(a.vd, 120);
      d[121].vull = spu_testsv(a.vd, 121);
      d[122].vull = spu_testsv(a.vd, 122);
      d[123].vull = spu_testsv(a.vd, 123);
      d[124].vull = spu_testsv(a.vd, 124);
      d[125].vull = spu_testsv(a.vd, 125);
      d[126].vull = spu_testsv(a.vd, 126);
      d[127].vull = spu_testsv(a.vd, 127);

      // Check that the correct result is returned
      for (i=0; i<128; i++) {
	for (j=0; j<2; j++) {
	  exp = (i & testsv_samples[e[j]].sv) ? 0xFFFFFFFFFFFFFFFFULL : 0ULL;

	  if (exp != d[i].ull[j]) {
	    DEBUG(1, failed);
	    DEBUG(2, "\t\tsv=%d element=%d samples=%d %d a=%f ", i, j, e[0], e[1], a.d[j]);
	    DEBUG(2, "d=0x%llx exp=0x%llx\n", d[i].ull[j], exp);
	    return (1);
	  }
	}
      }
    }
  }
  DEBUG(1, passed);
  return (0);
}


int test_spu_writech()
{
  DEBUG(1, "writech.........");

  spu_writech(SPU_WrEventMask, 0x0);

  DEBUG(1, passed);
  return (0);
}

int test_spu_writechqw()
{
  DEBUG(1, "writechqw.......");

#ifdef __SPU__
  a.vui = spu_splats((unsigned int)0);
#else /* !__SPU__ */
  a.vui = ((vector unsigned int) {0,0,0,0});
#endif /* __SPU__ */
  spu_writechqw(SPU_WrEventMask, a.vuc);
  spu_writechqw(SPU_WrEventMask, a.vc);
  spu_writechqw(SPU_WrEventMask, a.vus);
  spu_writechqw(SPU_WrEventMask, a.vs);
  spu_writechqw(SPU_WrEventMask, a.vui);
  spu_writechqw(SPU_WrEventMask, a.vi);
  spu_writechqw(SPU_WrEventMask, a.vull);
  spu_writechqw(SPU_WrEventMask, a.vll);
  spu_writechqw(SPU_WrEventMask, a.vf);
  spu_writechqw(SPU_WrEventMask, a.vd);

  DEBUG(1, passed);
  return (0);
}


int test_spu_xor()
{
  int i, j;

  DEBUG(1, "xor.............");
  
  for (i=0; i<samples; i++) {
    a.vui = rand_vui();
    b.vui = rand_vui();
    d.vui = spu_xor(a.vui, b.vui);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = a.ui[j] ^ b.ui[j];
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_xor(a.vi, b.vi);
    VERIFY_EQUAL(d, e);
    e.vus = spu_xor(a.vus, b.vus);
    VERIFY_EQUAL(d, e);
    e.vs = spu_xor(a.vs, b.vs);
    VERIFY_EQUAL(d, e);
    e.vuc = spu_xor(a.vuc, b.vuc);
    VERIFY_EQUAL(d, e);
    e.vc = spu_xor(a.vc, b.vc);
    VERIFY_EQUAL(d, e);
    e.vf = spu_xor(a.vf, b.vf);
    VERIFY_EQUAL(d, e);
    e.vll = spu_xor(a.vll, b.vll);
    VERIFY_EQUAL(d, e);
    e.vull = spu_xor(a.vull, b.vull);
    VERIFY_EQUAL(d, e);
    e.vd = spu_xor(a.vd, b.vd);
    VERIFY_EQUAL(d, e);


  }


  /* immediate forms */
  /* int */
  for (i=0; i<samples; i++) {
    unsigned int imm;
    a.vui = rand_vui();
    imm   = (unsigned int)rand_local();
    d.vui = spu_xor(a.vui, imm);
    for (j=0; j<4; j++) {
      unsigned int exp;
      exp = a.ui[j] ^ imm;
      if (exp != d.ui[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuint immediate: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.ui[j], b.ui[j], d.ui[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vi = spu_xor(a.vi, (signed int)(imm));
    VERIFY_EQUAL(d, e);
  }

  /* short */
  for (i=0; i<samples; i++) {
    unsigned short imm;
    a.vui = rand_vui();
    imm   = (unsigned short)rand_local();
    d.vus = spu_xor(a.vus, imm);
    for (j=0; j<8; j++) {
      unsigned short exp;
      exp = a.us[j] ^ imm;
      if (exp != d.us[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tushort immediate: element=%d ", j);
	DEBUG(2, "a=0x%x b=0x%x d=0x%x exp=0x%x\n", a.us[j], b.us[j], d.us[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vs = spu_xor(a.vs, (signed short)(imm));
    VERIFY_EQUAL(d, e);
  }
    
  /* byte */
  for (i=0; i<samples; i++) {
    unsigned char imm;
    a.vui = rand_vui();
    imm   = (unsigned char)rand_local();
    d.vuc = spu_xor(a.vuc, imm);
    for (j=0; j<16; j++) {
      unsigned char exp;
      exp = a.uc[j] ^ imm;
      if (exp != d.uc[j]) {
	DEBUG(1, failed);
	DEBUG(2, "\t\tuchar immediate: element=%d ", j);
	DEBUG(2, "a=0x%x imm=0x%x d=0x%x exp=0x%x\n", a.uc[j], imm, d.uc[j], exp);
	return (1);
      }
    }
    /* verify equivalent variations */
    e.vc = spu_xor(a.vc, (signed char)imm);
    VERIFY_EQUAL(d, e);
  }

  /* immmediate literals 10-bit
   */
  a.vui = rand_vui();
  d.vuc = spu_xor(a.vuc, 267);
  for (j=0; j<16; j++) {
    unsigned char exp;
    exp = a.uc[j] ^ 267;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=267 d=0x%x exp=0x%x\n", a.uc[j], d.uc[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vc = spu_xor(a.vc, 267);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vus = spu_xor(a.vus, 511);
  for (j=0; j<8; j++) {
    unsigned short exp;
    exp = a.us[j] ^ 511;
    if (exp != d.us[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tushort immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=511 d=0x%x exp=0x%x\n", a.us[j], d.us[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vs = spu_xor(a.vs, 511);
  VERIFY_EQUAL(d, e);

  a.vui = rand_vui();
  d.vuc = spu_xor(a.vuc, -315);
  for (j=0; j<16; j++) {
    unsigned char exp;
    exp = a.uc[j] ^ -315;
    if (exp != d.uc[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuchar immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=-315 d=0x%x exp=0x%x\n", a.uc[j], d.uc[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vc = spu_xor(a.vc, -315);
  VERIFY_EQUAL(d, e);


  a.vui = rand_vui();
  d.vui = spu_xor(a.vui, -512);
  for (j=0; j<4; j++) {
    unsigned int exp;
    exp = a.ui[j] ^ -512;
    if (exp != d.ui[j]) {
      DEBUG(1, failed);
      DEBUG(2, "\t\tuint immediate (literal): element=%d ", j);
      DEBUG(2, "a=0x%x b=-511 d=0x%x exp=0x%x\n", a.ui[j], d.ui[j], exp);
      return (1);
    }
  }
  /* verify equivalent variations */
  e.vi = spu_xor(a.vi, -512);
  VERIFY_EQUAL(d, e);

  DEBUG(1, passed);
  return (0);
}

DECLARE_MAIN()
{
  DECLARE_TEST_VARS();
#ifdef __SPU__
#ifndef STANDALONE
  volatile unsigned int *ptr = NULL;
#endif /* !STANDALONE */
#endif /* __SPU__ */

  SCALE_SAMPLES(samples);

#ifdef __SPU__
  spu_mtfpscr(spu_splats((unsigned int)0));
#else /* !__SPU__ */
  spu_mtfpscr(((vector unsigned int) {0,0,0,0}));
#endif /* __SPU__ */

  INVOKE_TEST(test_spu_absd);
  INVOKE_TEST(test_spu_add);
  INVOKE_TEST(test_spu_addx);
  INVOKE_TEST(test_spu_and);
  INVOKE_TEST(test_spu_andc);
  INVOKE_TEST(test_spu_avg);
#ifdef __SPU__
#ifndef STANDALONE
   /* Install a no-op interrupt handler that disables interrupts 
   */
  *ptr = 0x35480000;	/* iretd */
  INVOKE_TEST(test_spu_bisled);
  INVOKE_TEST(test_spu_bisled_d);
  INVOKE_TEST(test_spu_bisled_e);
#endif /* !STANDALONE */
#endif /* __SPU__ */
  INVOKE_TEST(test_spu_cmpabseq);
  INVOKE_TEST(test_spu_cmpabsgt);
  INVOKE_TEST(test_spu_cmpeq);
  INVOKE_TEST(test_spu_cmpgt);
  INVOKE_TEST(test_spu_cntb);
  INVOKE_TEST(test_spu_cntlz);
  INVOKE_TEST(test_spu_convtf);
  INVOKE_TEST(test_spu_convts);
  INVOKE_TEST(test_spu_convtu);
  INVOKE_TEST(test_spu_dsync);
  INVOKE_TEST(test_spu_eqv);
  INVOKE_TEST(test_spu_extend);
  INVOKE_TEST(test_spu_extract);
  INVOKE_TEST(test_spu_gather);
  INVOKE_TEST(test_spu_genb);
  INVOKE_TEST(test_spu_genbx);
  INVOKE_TEST(test_spu_genc);
  INVOKE_TEST(test_spu_gencx);
  INVOKE_TEST(test_spu_hcmpeq);
  INVOKE_TEST(test_spu_hcmpgt);
#ifdef __SPU__
#ifndef STANDALONE
  *ptr = 0x35400000;		/* iret */

  INVOKE_TEST(test_spu_ienable_idisable);
#endif /* !STANDALONE */
#endif /* __SPU__ */
  INVOKE_TEST(test_spu_insert);
  INVOKE_TEST(test_spu_madd);
  INVOKE_TEST(test_spu_maskb);
  INVOKE_TEST(test_spu_maskh);
  INVOKE_TEST(test_spu_maskw);
  INVOKE_TEST(test_spu_mfcdma32);
  INVOKE_TEST(test_spu_mfcdma64);
  INVOKE_TEST(test_spu_mfcstat);
  INVOKE_TEST(test_spu_mfspr);		/* also tests mfspr */
  INVOKE_TEST(test_spu_mffpscr);	/* also tests mtfpscr */
  INVOKE_TEST(test_spu_mhhadd);
  INVOKE_TEST(test_spu_msub);
  INVOKE_TEST(test_spu_mul);
  INVOKE_TEST(test_spu_mulh);
  INVOKE_TEST(test_spu_mule);
  INVOKE_TEST(test_spu_mulo);
  INVOKE_TEST(test_spu_mulsr);
  INVOKE_TEST(test_spu_nand);
  INVOKE_TEST(test_spu_nmadd);
  INVOKE_TEST(test_spu_nmsub);
  INVOKE_TEST(test_spu_nor);
  INVOKE_TEST(test_spu_or);
  INVOKE_TEST(test_spu_orc);
  INVOKE_TEST(test_spu_orx);
  INVOKE_TEST(test_spu_promote);
  INVOKE_TEST(test_spu_re);
  INVOKE_TEST(test_spu_readch);
  INVOKE_TEST(test_spu_readchcnt);
  INVOKE_TEST(test_spu_readchqw);
  INVOKE_TEST(test_spu_rl);
  INVOKE_TEST(test_spu_rlmask);
  INVOKE_TEST(test_spu_rlmaska);
  INVOKE_TEST(test_spu_rlmaskqw);
  INVOKE_TEST(test_spu_rlmaskqwbyte);
  INVOKE_TEST(test_spu_rlmaskqwbytebc);
  INVOKE_TEST(test_spu_rlqw);
  INVOKE_TEST(test_spu_rlqwbyte);
  INVOKE_TEST(test_spu_rlqwbytebc);
  INVOKE_TEST(test_spu_roundtf);
  INVOKE_TEST(test_spu_rsqrte);
  INVOKE_TEST(test_spu_sel);
  INVOKE_TEST(test_spu_shuffle);
  INVOKE_TEST(test_spu_sl);
  INVOKE_TEST(test_spu_slqw);
  INVOKE_TEST(test_spu_slqwbyte);
  INVOKE_TEST(test_spu_slqwbytebc);
  INVOKE_TEST(test_spu_splats);
  INVOKE_TEST(test_spu_stop);
  INVOKE_TEST(test_spu_sub);
  INVOKE_TEST(test_spu_subx);
  INVOKE_TEST(test_spu_sumb);
  INVOKE_TEST(test_spu_sync);
  INVOKE_TEST(test_spu_sync_c);
  INVOKE_TEST(test_spu_testsv);
  INVOKE_TEST(test_spu_writech);
  INVOKE_TEST(test_spu_writechqw);
  INVOKE_TEST(test_spu_xor);

  DISPLAY_RESULTS();
}
